package org.exoplatform.faces.cms.listener;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;

public class SelectorActionListener extends ExoActionListener {

  private UIInput uiInput_;

  public SelectorActionListener(UIInput uiInput) {
    uiInput_ = uiInput;
  }

  public void execute(ExoActionEvent event) throws Exception {
    UISimpleForm uiForm = (UISimpleForm) event.getComponent();
    String action = event.getAction();
    SessionContainer sC = SessionContainer.getInstance();
    SelectorActionClassMaps maps = (SelectorActionClassMaps) sC
        .getComponentInstanceOfType(SelectorActionClassMaps.class);
    Class selectorClazz = (Class) maps.mappings.get(action);
    Selector selector = (Selector) uiForm.getSibling(selectorClazz);
    selector.setHandler((SelectorHandler) maps.handlers.get(uiInput_.getName()));
    String[] nonResolvedParams = (String[]) maps.parameters.get(uiInput_.getName()); 
    if(nonResolvedParams != null)
      selector.setParameters(resolveParams(nonResolvedParams));
    uiForm.setRenderedSibling(selectorClazz);
  }

  private String[] resolveParams(String[] nonResolvedParams) {
    String[] resolvedParams = new String[nonResolvedParams.length];
    for (int i = 0; i < nonResolvedParams.length; i++) {
      String nonResolvedParam = nonResolvedParams[i];
      if (nonResolvedParam.startsWith("#")) {
        String temp = nonResolvedParam.substring(1);
        UIInput refInput = ((UISimpleForm)uiInput_.getParent()).getUIInput(temp);
        if (refInput != null) {
          resolvedParams[i] = refInput.getValue();
        } else
          resolvedParams[i] = nonResolvedParam;
      } else {
        resolvedParams[i] = nonResolvedParam;
      }
    }
    return resolvedParams;
  }

}
