/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.display.dynamic.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.portlets.content.display.model.ContentConfigComparator;
import org.exoplatform.portlets.content.display.model.ContentConfigManipulationsUtils;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentConfig.java,v 1.4 2004/11/03 04:24:55 tuan08 Exp $
 */
public class UIContentConfig extends UIExoCommand implements
    ContentConfigConstants {

  private static final String NEW_ACTION = "new";

  private static final String NEW_REFERENCE_ACTION = "newReference";

  private static final String MODIFY_ACTION = "modify";

  private static final String INCREMENT_ACTION = "increment";

  private static final String DECREMENT_ACTION = "decrement";

  private static Parameter[] NEW_PARAMS = { new Parameter(ACTION, NEW_ACTION) };

  private static Parameter[] NEW_REFERENCE_PARAMS = { new Parameter(ACTION,
      NEW_REFERENCE_ACTION) };

  private static Parameter[] EDIT_PARAMS = { new Parameter(ACTION, EDIT_ACTION) };

  private static Parameter[] DELETE_PARAMS = { new Parameter(ACTION,
      DELETE_ACTION) };

  private static Parameter[] MODIFY_PARAMS = { new Parameter(ACTION,
      MODIFY_ACTION) };

  private static Parameter[] INCREMENT_PARAMS = { new Parameter(ACTION,
      INCREMENT_ACTION) };

  private static Parameter[] DECREMENT_PARAMS = { new Parameter(ACTION,
      DECREMENT_ACTION) };

  Map configs_;

  public UIContentConfig() {
    setRendererType("VelocityRenderer");
    setId("UIContentConfig");    
    addActionListener(DeleteActionListener.class, DELETE_ACTION);
    addActionListener(EditActionListener.class, EDIT_ACTION);
    addActionListener(ModifyActionListener.class, MODIFY_ACTION);
    addActionListener(IncrementActionListener.class, INCREMENT_ACTION);
    addActionListener(DecrementActionListener.class, DECREMENT_ACTION);
    addActionListener(NewActionListener.class, NEW_ACTION);
    addActionListener(NewReferenceActionListener.class, NEW_REFERENCE_ACTION);
  }

  public void updateConfigs(Session session) {
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    PortletRequest request = (PortletRequest) eContext.getRequest();
    PortletPreferences prefs = request.getPreferences();
    Enumeration e = prefs.getNames();
    configs_ = new HashMap();
    while (e.hasMoreElements()) {
      String name = (String) e.nextElement();
      if (!UIContentPortlet.LAYOUT.equals(name)) {
        String[] values = prefs.getValues(name, DEFAULT_VALUES);
        ContentConfig contentConfig = new ContentConfig(name, values);
        if (isContentAvailable(session, contentConfig))
          configs_.put(name, contentConfig);
      }
    }
  }
  
  public boolean isContentAvailable(Session session, ContentConfig contentConfig) {
    try {
      String path = ContentUtil.resolveLocalisedViewPath(contentConfig.getUri());
      session.getItem(path);
      return true;
    } catch (Exception e) {      
      return false;
    }
  }  

  public Parameter[] getNewParams() {
    return NEW_PARAMS;
  }

  public Parameter[] getNewReferenceParams() {
    return NEW_REFERENCE_PARAMS;
  }

  public Parameter[] getEditParams() {
    return EDIT_PARAMS;
  }

  public Parameter[] getDeleteParams() {
    return DELETE_PARAMS;
  }

  public Parameter[] getModifyParams() {
    return MODIFY_PARAMS;
  }

  public Parameter[] getIncrementParams() {
    return INCREMENT_PARAMS;
  }

  public Parameter[] getDecrementParams() {
    return DECREMENT_PARAMS;
  }

  public Collection getConfigs() {
    Collection nonSorted = configs_.values();
    Comparator comparator = new ContentConfigComparator();
    List nonSortedList = new ArrayList(nonSorted);
    Collections.sort(nonSortedList, comparator);
    return nonSortedList;
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      String name = event.getParameter(OBJECTID);

      Session session = ((UIAdminContent)uiConfig.getAncestorOfType(UIAdminContent.class)).getSession();      
      
      ContentConfig contentConfig = (ContentConfig) uiConfig.configs_.get(name);
      
      String uri = ContentUtil.getLocalisedAbsoluteSitePath() + contentConfig.getUri();
      try {
        Node node = (Node) session.getItem(uri);
        Node parent = node.getParent();
        node.remove();        
        parent.save();
      } catch (PathNotFoundException e) {
      }
      ContentConfigManipulationsUtils.removeContentConfig(name);
      uiConfig.updateConfigs(session);
    }
    
  }

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      String name = event.getParameter(OBJECTID);
      ContentConfig config = (ContentConfig) uiConfig.configs_.get(name);

      UIContentConfigForm form = (UIContentConfigForm) uiConfig
          .getSibling(UIContentConfigForm.class);
      form.setContentConfig(config);
      uiConfig.setRenderedSibling(UIContentConfigForm.class);
    }
  }

  static public class NewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      UIContentConfigForm uiConfigForm = (UIContentConfigForm) uiConfig
          .getSibling(UIContentConfigForm.class);
      uiConfigForm.setContentConfig(null);
      uiConfig.setRenderedSibling(UIContentConfigForm.class);
    }
  }

  static public class NewReferenceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      UIReferenceForm uiReferenceForm = (UIReferenceForm) uiConfig
          .getSibling(UIReferenceForm.class);
      uiReferenceForm.setContentConfig(null);
      uiConfig.setRenderedSibling(UIReferenceForm.class);
    }
  }

  static public class ModifyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      String name = event.getParameter(OBJECTID);
      ContentConfig config = (ContentConfig) uiConfig.configs_.get(name);
      if (!config.isEntry()) {
        UIReferenceForm uiReferenceForm = (UIReferenceForm) uiConfig
            .getSibling(UIReferenceForm.class);
        uiReferenceForm.setContentConfig(config);
        uiConfig.setRenderedSibling(UIReferenceForm.class);
      } else {
        UIContentEditor uiEditor = (UIContentEditor) uiConfig
            .getSibling(UIContentEditor.class);
        uiEditor.setContentConfig(config);
        uiEditor.setIsNew(false);
        uiConfig.setRenderedSibling(UIContentEditor.class);
      }

    }
  }

  static public class IncrementActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      String name = event.getParameter(OBJECTID);
      ContentConfigManipulationsUtils.incrementConfigPosition(name);
      Session session = ((UIAdminContent)uiConfig.getAncestorOfType(UIAdminContent.class)).getSession();
      uiConfig.updateConfigs(session);
    }
  }

  static public class DecrementActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent();
      String name = event.getParameter(OBJECTID);
      ContentConfigManipulationsUtils.decrementConfigPosition(name);
      Session session = ((UIAdminContent)uiConfig.getAncestorOfType(UIAdminContent.class)).getSession();
      uiConfig.updateConfigs(session);
    }
  }

}