/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.display.dynamic.component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.portlets.content.display.model.ContentConfigComparator;
import org.exoplatform.services.cms.CmsService;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentPortlet.java,v 1.2 2004/07/26 02:27:18 tuan08 Exp $
 */
public class UIContentPortlet extends UIPortlet implements
    ContentConfigConstants {

  private Session session;

  public UIContentPortlet(CmsService cmsService) throws Exception {
    setClazz("UIContentPortlet");
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    session = cmsService.getSession(null);
    PortletRequest request = (PortletRequest) eContext.getRequest();
    PortletPreferences prefs = request.getPreferences();
    Enumeration names = prefs.getNames();
    List children = getChildren();
    List nonSortedConfigs = new ArrayList();
    boolean select = true;
    while (names.hasMoreElements()) {
      String name = (String) names.nextElement();
      if (!LAYOUT.equals(name)) {
        String[] values = prefs.getValues(name, DEFAULT_VALUES);
        ContentConfig contentConfig = new ContentConfig(name, values);
        nonSortedConfigs.add(contentConfig);
      }
    }
    Collections.sort(nonSortedConfigs, new ContentConfigComparator());

    for (Iterator iter = nonSortedConfigs.iterator(); iter.hasNext();) {
      ContentConfig contentConfig = (ContentConfig) iter.next();
      UIContentTab uiContentTab = null;
      try {
        if (contentConfig.isEntry()) {
          String path = ContentUtil.resolveLocalisedViewPath(contentConfig
              .getUri());
          Node viewNode = (Node) session.getItem(path);
          uiContentTab = new UIContentTab(contentConfig.getName(), viewNode);
          uiContentTab.setId("id" + Integer.toString(uiContentTab.hashCode()));
          uiContentTab.setRendered(select);
          children.add(uiContentTab);
          select = false;
        } else {// it is a reference
          NodeIterator iterNode = getReferencedNodes(contentConfig.getUri());
          while (iterNode.hasNext()) {
            Node node = (Node) iterNode.nextNode();
            String name = node.getName();
            if (name.contains("."))
              name = name.substring(0, name.lastIndexOf("."));
            uiContentTab = new UIContentTab(name, node);
            uiContentTab
                .setId("id" + Integer.toString(uiContentTab.hashCode()));
            uiContentTab.setRendered(select);
            children.add(uiContentTab);
            select = false;
          }
        }
      } catch (Exception ex) {
        ex.printStackTrace();
        uiContentTab = new UIContentTab("Referenced uri unreachable : " + contentConfig.getUri());
        uiContentTab.setId("id" + Integer.toString(uiContentTab.hashCode()));
        uiContentTab.setRendered(select);
        children.add(uiContentTab);
        select = false;
      }
    }
  }

  public NodeIterator getReferencedNodes(String uri) throws Exception {
    String path = ContentUtil.getLocalisedAbsoluteSitePath() + uri;
    Node referencingNode = (Node) ((Node) session.getItem(path)).getPrimaryItem();
    String referencedNodeUri = referencingNode.getProperty("jcr:data")
        .getString();
    Node referencedNode = (Node) session.getItem(referencedNodeUri);
    return referencedNode.getNodes();
  }

  public String getRendererType() {
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    PortletRequest request = (PortletRequest) eContext.getRequest();
    PortletPreferences prefs = request.getPreferences();
    String layout = prefs.getValue(LAYOUT, TABS_LAYOUT);
    if (TABS_LAYOUT.equals(layout))
      return "PyramidTabBarRenderer";
    else if (LIST_LAYOUT.equals(layout))
      return "ListNodesRenderer";
    return "PyramidTabBarRenderer";
  }

}