package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.JCRUtil;
import org.exoplatform.portlets.content.jcr.TypeNodeComparator;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.security.SecurityService;

public class UIDocumentInfo extends UIExoCommand implements ChangeNodeListener {

  public static final String[] NON_EDITABLE_NODETYPES = { "nt:unstructured",
      "nt:folder", "nt:resource" };
  public static final String[] PASTE_TYPES = { "nt:unstructured", "nt:folder" };

  final static public String ACTION_NAME = "actionName".intern();
  
  final static public String EDIT_DOCUMENT = "editDocument".intern();
  final static public String RENAME = "rename".intern();
  public static final String COPY_ACTION = "copy".intern();
  public static final String CUT_ACTION = "cut".intern();
  final static public String SAVE_NODE = "saveNode".intern();
  final static public String REMOVE_NODE_ACTION = "removeNode".intern();
  final static public String CHECKIN_NODE_ACTION = "checkin".intern();
  final static public String CHECKOUT_NODE_ACTION = "checkout".intern();
  final static public String LOCK_NODE_ACTION = "lock".intern();
  final static public String UNLOCK_NODE_ACTION = "unlock".intern();  
  final static public String CUSTOM_ACTION = "custom".intern();

  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION,
      UIJCRExplorer.CHANGE_NODE_ACTION) };
  public static Parameter[] EDIT_DOCUMENT_PARAMS = { new Parameter(ACTION,
      EDIT_DOCUMENT) };
  public static Parameter[] RENAME_PARAMS = { new Parameter(ACTION, RENAME) };
  public static Parameter[] COPY_NODE_PARAMS = { new Parameter(ACTION,
      COPY_ACTION) };
  public static Parameter[] CUT_NODE_PARAMS = { new Parameter(ACTION,
      CUT_ACTION) };
  public static Parameter[] SAVE_NODE_PARAMS = { new Parameter(ACTION,
      SAVE_NODE) };
  public static Parameter[] REMOVE_NODE = { new Parameter(ACTION,
      REMOVE_NODE_ACTION) };
  public static Parameter[] CHECKIN_NODE_PARAMS = { new Parameter(ACTION,
      CHECKIN_NODE_ACTION) };
  public static Parameter[] CHECKOUT_NODE_PARAMS = { new Parameter(ACTION,
      CHECKOUT_NODE_ACTION) };
  public static Parameter[] LOCK_NODE_PARAMS = { new Parameter(ACTION,
      LOCK_NODE_ACTION) };
  public static Parameter[] UNLOCK_NODE_PARAMS = { new Parameter(ACTION,
      UNLOCK_NODE_ACTION) };    

  public static final int NB_PER_ROW = 4;
  public static final int MAC_VIEW_NB_COLUMNS = 3;

  protected Node node_;
  private CMSDelegate delegate;
  protected ActionServiceContainer actionService_;
  private SecurityService securityService_;
  private TemplateService templateService_;

  public UIDocumentInfo(TemplateService templateService, SecurityService securityService, 
      ActionServiceContainer actionService) {
    setId("UIDocumentInfo");
    setRendererType("VelocityRenderer");

    templateService_ = templateService;
    securityService_ = securityService;
    actionService_ = actionService;

    addActionListener(ChangeNodeActionListener.class,
        UIJCRExplorer.CHANGE_NODE_ACTION);
    addActionListener(EditDocumentActionListener.class, EDIT_DOCUMENT);
    addActionListener(CopyActionListener.class, COPY_ACTION);
    addActionListener(CutActionListener.class, CUT_ACTION);
    addActionListener(SaveNodeActionListener.class, SAVE_NODE);
    addActionListener(RemoveNodeActionListener.class, REMOVE_NODE_ACTION);
    addActionListener(CheckinNodeActionListener.class, CHECKIN_NODE_ACTION);
    addActionListener(CheckoutNodeActionListener.class, CHECKOUT_NODE_ACTION);
    addActionListener(LockActionListener.class, LOCK_NODE_ACTION);
    addActionListener(UnlockActionListener.class, UNLOCK_NODE_ACTION);
    addActionListener(CustomActionListener.class, CUSTOM_ACTION);
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    if (delegate == null)
      delegate = new CMSDelegate(node);
    else
      delegate.setNode(node);
    setRendered(true);
  }

  public boolean isJcrViewEnable() {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    return pref.isJcrEnable();
  }

  public boolean isReadAuthorized(ExtendedNode node) throws RepositoryException {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    return explorer.isReadAuthorized(node);
  }

  public boolean isReadAuthorized(String name) throws RepositoryException {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    ExtendedNode node = (ExtendedNode) node_.getNode(name);
    return explorer.isReadAuthorized(node);
  }

  public String getNodeOwner() {
    return ((ExtendedNode) node_).getChildrenACL().getOwner();
  }

  public boolean isListView() {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    String displayType = pref.getDisplayType();
    if (Preference.LIST_TYPE.equals(displayType))
      return true;
    return false;
  }

  public boolean hasChildren() {
    return hasChildren(null);
  }

  public boolean hasChildren(Node pNode) {
    try {
      if (pNode == null)
        return node_.hasNodes();
      else
        return pNode.hasNodes();
    } catch (RepositoryException e) {
      return false;
    }
  }

  public Parameter[] getEditDocumentParams() {
    return EDIT_DOCUMENT_PARAMS;
  }

  public Parameter[] getRenameParams() {
    return RENAME_PARAMS;
  }

  public Parameter[] getChangeNodeParams() {
    return CHANGE_NODE_PARAMS;
  }

  public Parameter[] getCopyParams() {
    return COPY_NODE_PARAMS;
  }

  public Parameter[] getCutParams() {
    return CUT_NODE_PARAMS;
  }

  public Parameter[] getSaveParams() {
    return SAVE_NODE_PARAMS;
  }

  public Parameter[] getDeleteParams() {
    return REMOVE_NODE;
  }

  public Parameter[] getCheckinParams() {
    return CHECKIN_NODE_PARAMS;
  }

  public Parameter[] getCheckoutParams() {
    return CHECKOUT_NODE_PARAMS;
  }

  public Parameter[] getLockParams() {
    return LOCK_NODE_PARAMS;
  }
  
  public Parameter[] getUnlockParams() {
    return UNLOCK_NODE_PARAMS;
  }  

  public Parameter[] getCustomActionParams(String actionName) {
    Parameter[] CUSTOM_ACTION_PARAMS = { new Parameter(ACTION,
        CUSTOM_ACTION),  new Parameter(ACTION_NAME, actionName)};      
    return CUSTOM_ACTION_PARAMS;
  }    
  
  public String getViewToParse() throws Exception {
    String nodetype;
    try {
      nodetype = getNodeType();
    } catch (Exception e) {
      return "";
    }
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    String displayType = pref.getDisplayType();

    if (isJcrViewEnable()) {
      return new StringBuffer(
          "org/exoplatform/portlets/content/jcr/renderer/html/").append(
          displayType).append("View.vm").toString();
    } else {
      if (isNodeTypeSupported()) {
        return "jcr:" + getTemplatePath();
      } else {
        return new StringBuffer(
            "org/exoplatform/portlets/content/jcr/renderer/html/").append(
            displayType).append("View.vm").toString();
      }
    }
  }

  public Collection getNodeIteratorList() throws Exception {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = uiExplorer.getPreference();
    String displayType = pref.getDisplayType();
    Node currentNode = uiExplorer.getSelectNode();
    Node selectedNode;
    int nbColumns = 1;
    int start;
    int columnMax;
    boolean rootFirst = false;
    List iteratorList;
    String currentPath = currentNode.getPath();

    if (Preference.MAC_TYPE.equals(displayType)) {
      nbColumns = MAC_VIEW_NB_COLUMNS;
    }
    iteratorList = new ArrayList(nbColumns);
    columnMax = nbColumns;

    if (hasChildren(currentNode)) {
      columnMax--;
    }
    selectedNode = currentNode;
    for (int i = 1; i < columnMax; i++) {
      try {
        selectedNode = selectedNode.getParent();
      } catch (Exception e) {
        rootFirst = true;
        break;
      }
    }
    if ("/".equals(selectedNode.getPath())) {
      rootFirst = true;
    }
    if (rootFirst) {
      Map m = new TreeMap();
      m.put("/", selectedNode);
      iteratorList.add(m.values().iterator());
      start = 2;
    } else {
      start = 1;
      try {
        selectedNode = selectedNode.getParent();
      } catch (Exception e) {
        return null;
      }
    }
    if ("/".equals(currentNode.getPath()))
      nbColumns = 2;
    for (int i = start; i <= nbColumns; i++) {
      if (hasChildren(selectedNode)) {
        Map nodeMap = getChildrenMap(selectedNode);
        iteratorList.add(nodeMap.values().iterator());
        if (i < nbColumns) {
          Iterator iter = nodeMap.values().iterator();
          while (iter.hasNext()) {
            Node node = (Node) iter.next();
            String nodePath = node.getPath();
            if (currentPath.indexOf(nodePath) > -1) {
              selectedNode = node;
              break;
            }
          }
        }
      } else
        break;
    }

    return iteratorList;
  }

  public Map getChildrenMap(Node pNode) throws Exception {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    String order = pref.getOrder();
    String sort = pref.getSort();

    NodeIterator i;
    if (pNode == null)
      i = node_.getNodes();
    else
      i = pNode.getNodes();
    Map nodesMap;
    int j = 0;
    String nodeType = "";
    if (Preference.ALPHABETICAL_SORT.equals(sort)) {
      nodesMap = new TreeMap(new AlphaNodeComparator(order));
      while (i.hasNext()) {
        Node childNode = i.nextNode();
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";

        } catch (Exception ex) {
        }
        if ("nt:folder".equals(nodeType))
          nodeType = "1";
        else
          nodeType = "2";
        String key = new StringBuffer(nodeType).append(childNode.getName())
            .append(j).toString().toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    } else {
      // Sort by node type
      nodesMap = new TreeMap(new TypeNodeComparator(order));
      String contentType = "";
      String mimeType = "";
      Node jcrContentNode;
      StringTokenizer strTk;
      while (i.hasNext()) {
        Node childNode = i.nextNode();
        jcrContentNode = null;
        nodeType = "default";
        contentType = "default";
        mimeType = "default";
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";
          if ("nt:file".equals(nodeType))
            jcrContentNode = childNode.getNode("jcr:content");
          else if ("nt:resource".equals(nodeType))
            jcrContentNode = childNode;
          if (jcrContentNode != null) {
            if (jcrContentNode.hasProperty("jcr:mimeType"))
              mimeType = jcrContentNode.getProperty("jcr:mimeType").getString();
          }
        } catch (Exception ex) {
        }
        // 2 node types for sorting : folder or file
        if ("nt:folder".equals(nodeType))
          nodeType = "folder";
        else
          nodeType = "file";
        // mime type if available. Ex : pdf for application/pdf
        if (!"default".equals(mimeType)) {
          strTk = new StringTokenizer(mimeType, "/");
          if (strTk.countTokens() == 2) {
            strTk.nextToken();
            mimeType = strTk.nextToken();
          }
        }
        String key = new StringBuffer(nodeType).append("//").append(mimeType)
            .append("//").append(childNode.getName()).append(j).toString()
            .toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    }

    return nodesMap;
  }

  public Iterator getChildrenIterator() throws Exception {
    return getChildrenIterator(null);
  }

  public Iterator getChildrenIterator(Node pNode) throws Exception {

    return getChildrenMap(pNode).values().iterator();
  }

  public Collection getChildrenPerRow() throws Exception {
    return Utils.getChildrenPerRow(NB_PER_ROW, getChildrenIterator());
  }

  public String getThumbnailName(Node node) throws RepositoryException {
    String icon = getLargeNodeTypeIcon(node);
    return icon + "<br clear=all/>" + node.getName();
  }

  public String getNodeFlag(Node node) {
    return JCRUtil.getNodeFlag(node);
  }

  public String getSmallNodeTypeIcon(Node node) {
    return JCRUtil.getSmallNodeTypeIcon(node);
  }

  public String getLargeNodeTypeIcon(Node node) {
    return JCRUtil.getLargeNodeTypeIcon(node);
  }

  public boolean isEditPossible(String name) throws Exception {
    boolean isEdit = true;
    Node childNode = node_.getNode(name);

    String nodeType = childNode.getPrimaryNodeType().getName();
    for (int i = 0; i < NON_EDITABLE_NODETYPES.length; i++) {
      String nonEditableType = NON_EDITABLE_NODETYPES[i];
      if (nonEditableType.equals(nodeType))
        return false;
    }
    return isEdit;
  }

  public Node getNode() throws Exception {
    return delegate.getNode();
  }

  public void setNode(Node node) {
    delegate.setNode(node);
  }

  public String getPortalName() {
    return delegate.getPortalName();
  }

  public String getWorkspaceName() throws Exception {
    return delegate.getWorkspaceName();
  }

  public boolean isNodeTypeSupported() {
    return delegate.isNodeTypeSupported();
  }

  public WikiContext getWikiContext() {
    return delegate.getWikiContext();
  }

  public WikiEngineService getWikiService() {
    return delegate.getWikiService();
  }
  
  public String getNodeType() throws Exception {
    return delegate.getNodeType();
  }
  
  public String getTemplatePath() throws Exception {
    return delegate.getTemplatePath();
  }  

  public boolean isVersionableOrAncestor(Node node) throws RepositoryException {
    if (isVersionable(node))
      return true;
    if (isAncestorVersionable(node))
      return true;
    return false;
  }

  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
        && !node.isNodeType("nt:frozenNode");
  }

  public boolean isAncestorVersionable(Node node) throws RepositoryException {
    int depth = node.getDepth();
    int i = depth - 1;
    while (i != 0) {
      Node parent = (Node) node.getAncestor(i);
      if (parent.isNodeType("mix:versionable"))
        return true;
      i--;
    }
    return false;
  }
  
  public String getVersionNumber(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "-";
    return node.getBaseVersion().getName();
  }
  
  public List<Node> getCustomActions(Node node) throws Exception {
    List<Node> safeActions = new ArrayList<Node>();
    
    try {
      String user = SessionContainer.getInstance().getRemoteUser();
      List<Node> unsafeActions = actionService_.getActions(node, ActionServiceContainer.READ_PHASE);
      for (Iterator iter = unsafeActions.iterator(); iter.hasNext();) {
        Node actionNode = (Node) iter.next();
        Value[] roles = actionNode.getProperty("exo:roles").getValues();
        for (int i = 0; i < roles.length; i++) {
          String role = roles[i].getString();
          if(securityService_.hasMembershipInGroup(user, role))
            safeActions.add(actionNode);
        }
      }      
    }
    catch(AccessDeniedException ignore) {
    }

    return safeActions;
  }

  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp
          .getAncestorOfType(UIJCRExplorer.class);
      String uri = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        if ("../".equals(uri)) {
          if (!"/".equals(uiExplorer.getSelectNode().getPath())) {
            uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
          }
        }
        uiExplorer.setSelectNode(uri);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class EditDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node selectedNode = uiExplorer.getSelectNode();
        String name = event.getParameter(OBJECTID);
        selectedNode = selectedNode.getNode(name);
        String nodeType = selectedNode.getPrimaryNodeType().getName();
        if ("nt:file".equals(nodeType)) {
          Node contentNode = selectedNode.getNode("jcr:content");
          String contentNodeType = contentNode.getPrimaryNodeType().getName();
          if ("nt:resource".equals(contentNodeType)) {
            UIResourceForm form = (UIResourceForm) uicomp.getSibling(UIResourceForm.class);
            form.setContentNode(contentNode);
            uicomp.setRenderedSibling(UIResourceForm.class);
            return;
          }
        }
        UIDocumentFormController uiNewNodeController = (UIDocumentFormController) uicomp
            .getSibling(UIDocumentFormController.class);
        uiNewNodeController.showEditDocumentForm(name);
        uicomp.setRenderedSibling(UIDocumentFormController.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class SaveNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp
          .getAncestorOfType(UIJCRExplorer.class);
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node parentNode = uiExplorer.getSelectNode();
        Node node = parentNode.getNode(name);
        node.save();
        Object[] args = { node.getPath() };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.save-node-success"), args));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CopyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      ResourceBundle res = getApplicationResourceBundle() ;
      
      String name = event.getParameter(OBJECTID);
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node parentNode = uiExplorer.getSelectNode();
        Node node = parentNode.getNode(name);
        uiExplorer.getClipBoard().setType(ClipboardCommand.COPY);
        uiExplorer.getClipBoard().setSrcPath(node.getPath());
        Object[] args = { node.getPath() };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.node-copied"), args));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CutActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String name = event.getParameter(OBJECTID);
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      Node parentNode = uiExplorer.getSelectNode();
      try {
        Node node = parentNode.getNode(name);
        uiExplorer.getClipBoard().setType(ClipboardCommand.CUT);
        uiExplorer.getClipBoard().setSrcPath(node.getPath());
        Object[] args = { node.getPath() };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.node-cut"), args));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }

  }

  static public class RemoveNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node parentNode = uiExplorer.getSelectNode();
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = parentNode.getNode(name);
        if ("/".equals(node.getPath())) {
          iprovider.addMessage(new ExoFacesMessage(
              res.getString("UIDocumentInfo.msg.remove-root")));
          return;
        }
        node.remove();
        // uicomp.node_ = parentNode;
        // uiExplorer.setSelectNode(parentNode);
        Object[] args = { node.getPath() };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.node-remove-success"), args));
        // uiExplorer.broadcastOnChange(parentNode);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CheckinNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(name);
        node.checkin();
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CheckoutNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(name);
        node.checkout();
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class LockActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(name);
        if(!node.isNodeType("mix:lockable")) {
          node.addMixin("mix:lockable");
          node.save();
        }
        node.lock(true, true);        
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }  

  static public class UnlockActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String name = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(name);
        node.unlock();        
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }    

  static public class CustomActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String nodeName = event.getParameter(OBJECTID);
      String actionName = event.getParameter(ACTION_NAME);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(nodeName);
        String userId = SessionContainer.getInstance().getRemoteUser();
        uicomp.actionService_.executeAction(userId, node, actionName, new HashMap());
        Object[] args = { actionName };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.custom-action-success"), args));        
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }      
  
}
