/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.renderer.html;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.TreeMap;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.portlets.content.jcr.JCRUtil;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.portlets.content.jcr.component.UIViewController;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.log.LogUtil;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: ExplorerRenderer.java,v 1.3 2004/09/05 22:06:15 tuan08 Exp $
 */
public class JCRExplorerRenderer extends HtmlBasicRenderer {
  private static Parameter CHANGE_NODE = new Parameter(ACTION,
      UIJCRExplorer.CHANGE_NODE_ACTION);

  public void encodeChildren(FacesContext context, UIComponent component)
      throws IOException {
    try {
      UIJCRExplorer uiExplorer = (UIJCRExplorer) component;
      Preference pref = uiExplorer.getPreference();
      String displayType = pref.getDisplayType();
      ResourceBundle res = getApplicationResourceBundle(context
          .getExternalContext());
      String currentUri = uiExplorer.getSelectNode().getPath();
      String[] path = StringUtils.split(currentUri, "/");
      if (path.length == 0) {
        path = new String[0];
      }
      String currentPath = "";
      Parameter uriParam = new Parameter("uri", "/");
      Parameter[] changeNodeParams = { CHANGE_NODE, uriParam };
      ResponseWriter w = context.getResponseWriter();
      // Toolbar : Preferences, Save, Select workspace
      w.write("<table class='UIExplorer'>");
      w.write("<tr>");
      w.write("<td colspan='2'>");
      UIToolbar toolbar = (UIToolbar) uiExplorer
          .findComponentById("MainToolbar");
      toolbar.encodeBegin(context);
      toolbar.encodeChildren(context);
      toolbar.encodeEnd(context);
      w.write("</td>");
      w.write("</tr>");
      // current url
      w.write("<tr>");
      w.write("<td class='header' colspan='2'>");
      linkRenderer_.render(w, uiExplorer, "/", changeNodeParams);
      String pathSeparator = res.getString("UIExplorer.img.path-separator");
      for (int i = 0; i < path.length; i++) {
        currentPath += "/" + path[i];
        uriParam.setValue(currentPath);
        w.write(pathSeparator);
        linkRenderer_.render(w, uiExplorer, path[i], changeNodeParams);
      }
      w.write("</td>");
      w.write("</tr>");
      // Left panel : tree
      w.write("<tr>");
      if (!Preference.MAC_TYPE.equals(displayType)) {
        w.write("<td class='tree'>");
        renderNodeTree(w, uiExplorer);
        w.write("</td>");
      }
      w.write("<td>");
      UIViewController viewController = (UIViewController) uiExplorer
          .getChildComponentOfType(UIViewController.class);
      viewController.encodeBegin(context);
      viewController.encodeChildren(context);
      viewController.encodeEnd(context);
      w.write("</td>");
      w.write("</tr>");
      w.write("</table>");
    } catch (Exception ex) {
      LogUtil.getLog(getClass()).error("Cannot remder jcr tree", ex);
    }
  }

  final public void renderNodeTree(ResponseWriter w, UIJCRExplorer uiExplorer)
      throws IOException {
    Preference pref = uiExplorer.getPreference();
    String order = pref.getOrder();
    try {
      Parameter uriParam = new Parameter("uri", "../");
      Parameter[] changeNodeParams = { CHANGE_NODE, uriParam };
      w.write("<div>");
      linkRenderer_.render(w, uiExplorer, "[..]", changeNodeParams);
      w.write("</div>");
      Node node = uiExplorer.getSelectNode();
      if (!"/".equals(node.getPath())) {
        String cpath = node.getPath();
        Node parent = node.getParent();
        NodeIterator i = parent.getNodes();

        Map nodesMap = new TreeMap(new AlphaNodeComparator(order));
        int j = 0;
        String nodeType = "";
        while (i.hasNext()) {
          Node childNode = i.nextNode();
          try {
            if (childNode.hasProperty("jcr:primaryType")) {
              nodeType = childNode.getProperty("jcr:primaryType").getString();
              // unstructured node considered as folder node for sorting
              if ("nt:unstructured".equals(nodeType))
                nodeType = "nt:folder";
            }
          } catch (Exception ex) {
          }
          if ("nt:folder".equals(nodeType))
            nodeType = "1";
          else
            nodeType = "2";
          String key = new StringBuffer(nodeType).append(childNode.getName())
              .append(j).toString().toLowerCase();
          nodesMap.put(key, childNode);
          j++;
        }
        Iterator iter = nodesMap.values().iterator();
        while (iter.hasNext()) {
          Node childNode = (Node) iter.next();
          uriParam.setValue(childNode.getPath());
          w.write("<div class='node");
          if (childNode.getPath().equals(cpath))
            w.write(" select-node");
          w.write("'>");
          w.write(JCRUtil.getSmallNodeTypeIcon(childNode));
          if(uiExplorer.isReadAuthorized((ExtendedNode)childNode)) {
            linkRenderer_.render(w, uiExplorer, childNode.getName(),
                changeNodeParams);
          } else {
            w.write(childNode.getName());  
          }
          w.write("</div>");
        }

      }
    } catch (Exception ex) {
      LogUtil.getLog(getClass()).error("Cannot remder jcr tree", ex);
    }
  }
}