package org.exoplatform.services.cms.queries;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.jcr.RepositoryService;

public class QueryServiceImpl implements QueryService{
  
  private String relativePath_;
  private CmsConfigurationService cmsConfig_;
  private String workspace_;
  private RepositoryService repositoryService_;

  public QueryServiceImpl(RepositoryService repositoryService, CmsConfigurationService cmsConf, 
      InitParams params) throws Exception {
    relativePath_ = params.getValueParam("relativePath").getValue();
    workspace_ = params.getValueParam("workspace").getValue();
    cmsConfig_ = cmsConf;
    repositoryService_ = repositoryService;
  }
  
  public String getRelativePath() {
    return relativePath_;
  }
  
  public List<Query> getQueries() throws Exception {
    List<Query> queries = new ArrayList<Query>();        
    String user = getRemoteUser();
    if(user == null)
      return queries;
    
    Session session = null;
    try {
      session = repositoryService_.getRepository().getSystemSession(workspace_);
    } catch (RepositoryException re) {
      return queries;
    }    
    QueryManager manager = session.getWorkspace().getQueryManager();
    
    Node usersHome = (Node) session.getItem(cmsConfig_.getJcrPath(BasePath.CMS_USERS_PATH));
    Node userHome = usersHome.getNode(user);
    Node queriesHome = userHome.getNode(relativePath_); 
    
    NodeIterator iter = queriesHome.getNodes();
    while (iter.hasNext()) {
      Node node = iter.nextNode();
      if("nt:query".equals(node.getPrimaryNodeType().getName()))
        queries.add(manager.getQuery(node));
    }    
    return queries;
  }

  public Query getQuery(String queryName) throws Exception {
    String user = getRemoteUser();
    if(user == null)
      return null;
    
    Session session = null;
    try {
      session = repositoryService_.getRepository().getSystemSession(workspace_);
    } catch (RepositoryException re) {
      return null;
    }    
    QueryManager manager = session.getWorkspace().getQueryManager();
    Node usersHome = (Node) session.getItem(cmsConfig_.getJcrPath(BasePath.CMS_USERS_PATH));
    Node userHome = usersHome.getNode(user);
    Node queriesHome = userHome.getNode(relativePath_); 
    return manager.getQuery(queriesHome.getNode(queryName));
  }

  public void addQuery(String queryName, String statement, String language) throws Exception {
    String user = getRemoteUser();
    if(user == null)
      return;
    
    Session session = null;
    try {
      session = repositoryService_.getRepository().getSystemSession(workspace_);
    } catch (RepositoryException re) {
      return;
    }    
    QueryManager manager = session.getWorkspace().getQueryManager();    
    Query query = manager.createQuery(statement, language);
    String usersHome = cmsConfig_.getJcrPath(BasePath.CMS_USERS_PATH);
    String absPath = usersHome + "/" + user + "/" + relativePath_ + "/" + queryName;
    query.storeAsNode(absPath);
    session.getItem(usersHome).save();  
  }

  public void removeQuery(String queryName) throws Exception {
    String user = getRemoteUser();
    if(user == null)
      return;
    
    Session session = null;
    try {
      session = repositoryService_.getRepository().getSystemSession(workspace_);
    } catch (RepositoryException re) {
      return;
    }    
    Node usersHome = (Node) session.getItem(cmsConfig_.getJcrPath(BasePath.CMS_USERS_PATH));
    Node userHome = usersHome.getNode(user);
    Node queriesHome = userHome.getNode(relativePath_); 
    queriesHome.getNode(queryName).remove();
    queriesHome.save();    
  }
  
  private String getRemoteUser() {
    SessionContainer sessionContainer = SessionContainer.getInstance();
    if (sessionContainer != null) {
      return SessionContainer.getInstance().getRemoteUser();      
    }
    return null;    
  }

}
