/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.component ;


import java.util.ArrayList ;
import java.util.Collection;
import java.util.GregorianCalendar ;
import java.util.Calendar ;
import java.util.List ;

import javax.faces.component.UIComponent;

import org.exoplatform.faces.core.component.UIExoCommand ;
import org.exoplatform.faces.core.event.ExoActionEvent ;
import org.exoplatform.faces.core.event.ExoActionListener ;
import org.exoplatform.portlets.calendar.Utils;
import org.exoplatform.services.calendar.CalendarService ;


public class UICalendar extends UIExoCommand {

	private CalendarService calendarService_ ;

	public UICalendar(CalendarService cService) throws Exception {
		setRendererType("CalendarRenderer") ;
		setId("UICalendar") ;
		setClazz("UICalendar") ;
		calendarService_ = cService ;

		addChild(UILeftPanel.class) ;

		addActionListener(ViewDayListener.class, UICalendarPortlet.VIEW_DAY_ACTION) ;
		addActionListener(ViewWeekListener.class, UICalendarPortlet.VIEW_WEEK_ACTION) ;
		addActionListener(ViewMonthListener.class, UICalendarPortlet.VIEW_MONTH_ACTION) ;
		
		addActionListener(ViewEventListener.class, VIEW_ACTION) ;
//		addActionListener(EditEventListener.class, EDIT_EVENT_ACTION) ;

		addActionListener(MovePreviousListener.class, UICalendarPortlet.MOVE_PREVIOUS_ACTION) ;
		addActionListener(MoveNextListener.class, UICalendarPortlet.MOVE_NEXT_ACTION) ;
		
		addActionListener(displayLeftPanelListener.class, UICalendarPortlet.LEFT_PANEL_DISPLAY_ACTION) ;
	}

	
	public CalendarService getCalendarService() {
		return calendarService_ ;
	}
	
	public Collection getPublicCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPublicCalendars(uiCalendarPortlet.getCurrentUser()) ;
	}

	public Collection getPrivateCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPrivateCalendars(uiCalendarPortlet.getCurrentUser()) ;
	}
	
	public List findCalendarEventByUserAndDate(String user, GregorianCalendar selectDate) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		List l = new ArrayList() ;
		try {
			l =	(List) calendarService_.getCalendarEventByUserAndDate(user, uiCalendarPortlet.getCurrentCalendarType(), 
					uiCalendarPortlet.getCurrentCalendar(), selectDate) ;
		} catch (Exception e) {
			e.printStackTrace() ;
		}
		return l ;
	}

	public List findCalendarEventByUserAndDate(GregorianCalendar selectDate) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		List l = new ArrayList() ;
		try {
			l =	(List) calendarService_.getCalendarEventByUserAndDate(uiCalendarPortlet.getCurrentUser(), 
					uiCalendarPortlet.getCurrentCalendarType(), uiCalendarPortlet.getCurrentCalendar(), selectDate) ;
		} catch (Exception e) {
			e.printStackTrace() ;
		}
		return l ;
	}
	
	public List findCalendarEventByUserAndDate() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		List l = new ArrayList() ;
		GregorianCalendar gCal = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone() ;
		try {
			l =	(List) calendarService_.getCalendarEventByUserAndDate(uiCalendarPortlet.getCurrentUser(), 
					uiCalendarPortlet.getCurrentCalendarType(), uiCalendarPortlet.getCurrentCalendar(), gCal) ;
		} catch (Exception e) {
			e.printStackTrace() ;
		}
		return l ;
	}

	public List findCalendarEventByUserAndDateOfMonth(int currentDay) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		List l = new ArrayList() ;
		GregorianCalendar gCal = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone() ;
		try {
			gCal.set(Calendar.DAY_OF_MONTH, currentDay) ;
			l =	(List) calendarService_.getCalendarEventByUserAndDate(uiCalendarPortlet.getCurrentUser(), 
					uiCalendarPortlet.getCurrentCalendarType(), uiCalendarPortlet.getCurrentCalendar(), gCal) ;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace() ;
		}
		
		return l ;
	}

	public List findCalendarEventByUserAndDateOfWeek(int currentDay) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		List l = new ArrayList() ;
		GregorianCalendar gCal = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone() ;
		try {
			gCal.set(Calendar.DAY_OF_WEEK, currentDay) ;
			l =	(List) calendarService_.getCalendarEventByUserAndDate(uiCalendarPortlet.getCurrentUser(), 
					uiCalendarPortlet.getCurrentCalendarType(), uiCalendarPortlet.getCurrentCalendar(), gCal) ;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace() ;
		}
		
		return l ;
	}

	public String getFamily() {
		return "org.exoplatform.portlets.calendar.component.UICalendar" ;
	}
	
	static public class ViewDayListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			int day = new Integer(event.getParameter(UICalendarPortlet.DAY_PARAM)).intValue() ;
			int month = new Integer(event.getParameter(UICalendarPortlet.MONTH_PARAM)).intValue() ;
			int year = new Integer(event.getParameter(UICalendarPortlet.YEAR_PARAM)).intValue() ;
			uiCalendarPortlet.setSelectDate(year, month, day) ;
			uiCalendarPortlet.setViewMode(UICalendarPortlet.DAY_VIEW_MODE) ;
		}
	}

	static public class ViewWeekListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			int day = new Integer(event.getParameter(UICalendarPortlet.DAY_PARAM)).intValue() ;
			int month = new Integer(event.getParameter(UICalendarPortlet.MONTH_PARAM)).intValue() ;
			int year = new Integer(event.getParameter(UICalendarPortlet.YEAR_PARAM)).intValue() ;
			uiCalendarPortlet.setSelectDate(year, month, day) ;
			uiCalendarPortlet.setViewMode(UICalendarPortlet.WEEK_VIEW_MODE) ;
		}
	}

	static public class ViewMonthListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			int day = new Integer(event.getParameter(UICalendarPortlet.DAY_PARAM)).intValue() ;
			int month = new Integer(event.getParameter(UICalendarPortlet.MONTH_PARAM)).intValue() ;
			int year = new Integer(event.getParameter(UICalendarPortlet.YEAR_PARAM)).intValue() ;
			if (month != uiCalendarPortlet.getSelectMonth()) {
				uiCalendarPortlet.setSelectDate(year, month, day) ;
			}
		}
	}
	
//	static public class EditEventListener extends ExoActionListener {
//		public void execute(ExoActionEvent event) throws Exception {
//			UICalendar uiCalendar = (UICalendar) event.getComponent() ;
//			String eventId = event.getParameter(UICalendarPortlet.EVENT_ID_PARAM) ;
//			
//			UICalendarPortlet uiCalendarPortlet =
//				(UICalendarPortlet) uiCalendar.getAncestorOfType(
//					UICalendarPortlet.class) ;
//			UICalendarEventForm uiEventForm =
//				(UICalendarEventForm) uiCalendarPortlet.getChildComponentOfType(
//					UICalendarEventForm.class) ;
////			uiEventForm.setEditEventInfo(uiCalendar.calendarService_.findCalendarEventById(eventId)) ;
//			uiCalendarPortlet.setRenderedComponent(UICalendarEventForm.class) ;
//		}
//	}

	static public class ViewEventListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendar uiCalendar = (UICalendar) event.getComponent() ;
			UICalendarPortlet calendarPortlet =
				(UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;

			UICalendarEventForm uiCalendarEventForm = 
				(UICalendarEventForm) calendarPortlet.getChildComponentOfType(UICalendarEventForm.class) ;
			uiCalendarEventForm.clean() ;
			uiCalendarEventForm.setEditMode(false) ;
			String eventId = event.getParameter(UICalendarPortlet.EVENT_ID_PARAM) ;
			uiCalendarEventForm.setEventId(eventId) ;
			calendarPortlet.setRenderedComponent(UICalendarEventForm.class) ;
		}
	}

	static public class MovePreviousListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			String periodMoved = event.getParameter(UICalendarPortlet.PERIOD_MOVED) ;
			GregorianCalendar selectDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone() ;
			if (periodMoved.equals(UICalendarPortlet.MOVE_YEAR)) {
				selectDate.add(Calendar.YEAR, -1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_MONTH)) {
				selectDate.add(Calendar.MONTH, -1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_WEEK)) {
				selectDate.add(Calendar.WEEK_OF_YEAR, -1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_DAY)) {
				uiCalendarPortlet.getSelectDate().add(Calendar.DATE, -1) ;
			}
			uiCalendarPortlet.setSelectDate(selectDate) ;
			uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}
	
	static public class MoveNextListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UIMiniCalendar uiMiniCalendar = (UIMiniCalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiMiniCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			String periodMoved = event.getParameter(UICalendarPortlet.PERIOD_MOVED) ;
			GregorianCalendar selectDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone() ;
			if (periodMoved.equals(UICalendarPortlet.MOVE_YEAR)) {
				selectDate.add(Calendar.YEAR, 1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_MONTH)) {
				selectDate.add(Calendar.MONTH, 1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_WEEK)) {
				selectDate.add(Calendar.WEEK_OF_YEAR, 1) ;
			}
			else if (periodMoved.equals(UICalendarPortlet.MOVE_DAY)) {
				selectDate.add(Calendar.DATE, 1) ;
			}
			uiCalendarPortlet.setSelectDate(selectDate) ;
			uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}

	static public class displayLeftPanelListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet ;
			if (uiComponent instanceof UICalendar) {
				UICalendar uiCalendar = (UICalendar) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
			}
			else {
				UILeftPanel uiLeftPanel = (UILeftPanel) uiComponent ;
				uiCalendarPortlet = (UICalendarPortlet) uiLeftPanel.getAncestorOfType(UICalendarPortlet.class) ;
			}
			String leftPanelDisplaySelection = event.getParameter(UICalendarPortlet.LEFT_PANEL_DISPLAY_SELECTION) ;
			if (leftPanelDisplaySelection.equals(UICalendarPortlet.SHOW_LEFT_PANEL)) 
				uiCalendarPortlet.setDisplayLeftPanel(true) ;
			else uiCalendarPortlet.setDisplayLeftPanel(false) ;
			uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}
}
