package org.exoplatform.portlets.calendar.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.calendar.Utils;
import org.exoplatform.services.calendar.CalendarService;

public class UICalendarManagement extends UIExoCommand {
	
	public final static String SELECT_CALENDAR_ACTION = "SELECT_CALENDAR_ACTION" ;
	public final static String RENAME_CALENDAR_ACTION = "RENAME_CALENDAR_ACTION" ;
	public final static String DELETE_CALENDAR_ACTION = "DELETE_CALENDAR_ACTION" ;
	public final static String SHARE_CALENDAR_ACTION = "SHARE_CALENDAR_ACTION" ;
	public final static String UNSHARE_CALENDAR_ACTION = "UNSHARE_CALENDAR_ACTION" ;
	public final static String NEW_CALENDAR_ACTION = "NEW_CALENDAR_ACTION" ;
	
	public final static String CALENDAR_NAME_PARAM = "CALENDAR_NAME" ;
	public final static String CALENDAR_TYPE_PARAM = "CALENDAR_TYPE" ;
	
	public final static Parameter SELECT_CALENDAR_PARAM = new Parameter(ACTION, SELECT_CALENDAR_ACTION) ;
	public final static Parameter RENAME_CALENDAR_PARAM = new Parameter(ACTION, RENAME_CALENDAR_ACTION) ;
	public final static Parameter DELETE_CALENDAR_PARAM = new Parameter(ACTION, DELETE_CALENDAR_ACTION) ;
	public final static Parameter SHARE_CALENDAR_PARAM = new Parameter(ACTION, SHARE_CALENDAR_ACTION) ;
	public final static Parameter UNSHARE_CALENDAR_PARAM = new Parameter(ACTION, UNSHARE_CALENDAR_ACTION) ;
	public final static Parameter NEW_CALENDAR_PARAM = new Parameter(ACTION, NEW_CALENDAR_ACTION) ;
	public final static Parameter CANCEL_PARAM = new Parameter(ACTION, CANCEL_ACTION) ;
	
	private Parameter calendarNameParam_ = new Parameter(CALENDAR_NAME_PARAM, "") ;
	private Parameter calendarTypeParam_ = new Parameter(CALENDAR_TYPE_PARAM, "") ;
	
	private Parameter[] selectCalendarParams_ = {SELECT_CALENDAR_PARAM, calendarNameParam_, calendarTypeParam_} ;
	private Parameter[] renameCalendarParams_ = {RENAME_CALENDAR_PARAM, calendarNameParam_, calendarTypeParam_} ;
	private Parameter[] deleteCalendarParams_ = {DELETE_CALENDAR_PARAM, calendarNameParam_, calendarTypeParam_} ;
	private Parameter[] shareCalendarParams_ = {SHARE_CALENDAR_PARAM, calendarNameParam_, calendarTypeParam_} ;
	private Parameter[] unshareCalendarParams_ = {UNSHARE_CALENDAR_PARAM, calendarNameParam_, calendarTypeParam_} ;
	private Parameter[] newCalendarParams_ = {NEW_CALENDAR_PARAM} ;
	private Parameter[] cancelParams_ = {CANCEL_PARAM} ;
	
	public UICalendarManagement() throws Exception {
		setId("UICalendarManagement") ;
		setRendererType("VelocityRenderer") ;
		setClazz("UICalendarManagement") ;

		addActionListener(SelectCalendarActionListener.class, SELECT_CALENDAR_ACTION) ;
		addActionListener(RenameCalendarActionListener.class, RENAME_CALENDAR_ACTION) ;
		addActionListener(ShareCalendarActionListener.class, SHARE_CALENDAR_ACTION) ;
		addActionListener(UnshareCalendarActionListener.class, UNSHARE_CALENDAR_ACTION) ;
		
		addActionListener(NewCalendarActionListener.class, NEW_CALENDAR_ACTION) ;
		addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
}
	
	public boolean isCurrentCalendar(String type, String name) {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return uiCalendarPortlet.isCurrentCalendar(type, name) ;
	}
	
	public boolean isDefaultCalendar(String name) {
		if (name == null || "".equals(name)) return false ;
		if (name.equals(CalendarService.DEFAULT_CALENDAR)) return true ;
		return false ;
	}
	
	public String getPrivateType() {
		return CalendarService.PRIVATE_CALENDAR_TYPE ;
	}
	
	public String getPublicType() {
		return CalendarService.PUBLIC_CALENDAR_TYPE ;
	}

	public Collection getPublicCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPublicCalendarsName(uiCalendarPortlet.getCurrentUser()) ;
	}

	public Collection getPrivateCalendarsName() {
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) getAncestorOfType(UICalendarPortlet.class) ;
		return Utils.getPrivateCalendarsName(uiCalendarPortlet.getCurrentUser()) ;
	}

	public Parameter[] getSelectCalendarParams() {
		return selectCalendarParams_;
	}

	public Parameter[] getRenameCalendarParams() {
		return renameCalendarParams_;
	}

	public Parameter[] getDeleteCalendarParams() {
		return deleteCalendarParams_;
	}

	public Parameter[] getShareCalendarParams() {
		return shareCalendarParams_;
	}

	public Parameter[] getUnshareCalendarParams() {
		return unshareCalendarParams_;
	}

	public Parameter[] getNewCalendarParams() {
		return newCalendarParams_;
	}

	public Parameter[] getCancelParams() {
		return cancelParams_;
	}
	
	public void setCalendarNameParam(String calendarNameParam) {
		this.calendarNameParam_.setValue(calendarNameParam) ;
	}

	public void setCalendarTypeParam(String calendarTypeParam) {
		this.calendarTypeParam_.setValue(calendarTypeParam) ;
	}
	
	static public class SelectCalendarActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIComponent uiComponent = event.getComponent() ;
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) uiComponent ;
			String calandarName = event.getParameter(CALENDAR_NAME_PARAM) ;
			String calandarType = event.getParameter(CALENDAR_TYPE_PARAM) ;
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			uiCalendarPortlet.setCurrentCalendar(calandarName) ;
			uiCalendarPortlet.setCurrentCalendarType(calandarType) ;
			uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}
	
	static public class RenameCalendarActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) event.getComponent() ;
			String calandarName = event.getParameter(CALENDAR_NAME_PARAM) ;
			String calandarType = event.getParameter(CALENDAR_TYPE_PARAM) ;
			UICalendarPortlet uiCalendarPortlet =
				(UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			UICalendarForm uiCalendarForm = (UICalendarForm) uiCalendarPortlet.getChildComponentOfType(UICalendarForm.class) ;
			uiCalendarForm.setNode(uiCalendarPortlet.getCalendarService().
					getCalendar(uiCalendarPortlet.getCurrentUser(), calandarType, calandarName)) ;
			uiCalendarForm.setEditMode(true) ;

			uiCalendarPortlet.setRenderedComponent(UICalendarForm.class) ;
		}
	}

	static public class ShareCalendarActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) event.getComponent() ;
			String calandarName = event.getParameter(CALENDAR_NAME_PARAM) ;
			UICalendarPortlet uiCalendarPortlet =
				(UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			List l;
			l = new ArrayList(Utils.getPublicCalendarsName(uiCalendarPortlet.getCurrentUser())) ;
			for (int i = 0; i < l.size(); i++) {
				if (calandarName.equals(l.get(i))) {
					// TODO : display error message in uicalendarManagement
					return ;
				}
			}
			uiCalendarPortlet.getCalendarService().renameCalendar(uiCalendarPortlet.getCurrentUser(), 
					CalendarService.PRIVATE_CALENDAR_TYPE, calandarName, CalendarService.PUBLIC_CALENDAR_TYPE, calandarName) ;
		}
	}

	static public class UnshareCalendarActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) event.getComponent() ;
			String calandarName = event.getParameter(CALENDAR_NAME_PARAM) ;
			UICalendarPortlet uiCalendarPortlet =
				(UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			List l;
			l = new ArrayList(Utils.getPrivateCalendarsName(uiCalendarPortlet.getCurrentUser())) ;
			for (int i = 0; i < l.size(); i++) {
				if (calandarName.equals(l.get(i))) {
					// TODO : display error message in uicalendarManagement
					return ;
				}
			}
			uiCalendarPortlet.getCalendarService().renameCalendar(uiCalendarPortlet.getCurrentUser(), 
					CalendarService.PUBLIC_CALENDAR_TYPE, calandarName, CalendarService.PRIVATE_CALENDAR_TYPE, calandarName) ;
		}
	}

	static public class NewCalendarActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet =
				(UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			UICalendarForm uiCalendarForm = (UICalendarForm) uiCalendarPortlet.getChildComponentOfType(UICalendarForm.class) ;
			uiCalendarForm.setNode(null) ;
			uiCalendarForm.setEditMode(true) ;

			uiCalendarPortlet.setRenderedComponent(UICalendarForm.class) ;
		}
	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarManagement uiCalendarManagement = (UICalendarManagement) event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet =
				(UICalendarPortlet) uiCalendarManagement.getAncestorOfType(UICalendarPortlet.class) ;
			uiCalendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) uiCalendarPortlet.getChildComponentOfType(UIToolbarPanel.class)).setRendered(true) ;
		}
	}
}
