/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.component;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.faces.context.FacesContext;

import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.services.calendar.CalendarService;

public class UICalendarPortlet extends UIPortlet {
	// CALENDAR_EVENT ACTION
	public final static String NEW_EVENT_ACTION = "NEW_EVENT_ACTION" ;
//	public final static String VIEW_EVENT_ACTION = "VIEW_EVENT_ACTION" ;
//	public final static String EDIT_EVENT_ACTION = "EDIT_EVENT_ACTION" ;
	
	// CHANGE VIEW ACTION
	public final static String VIEW_DAY_ACTION = "VIEW_DAY_ACTION" ;
	public final static String VIEW_WEEK_ACTION = "VIEW_WEEK_ACTION" ;
	public final static String VIEW_MONTH_ACTION = "VIEW_MONTH_ACTION" ;

	// MOVE VIEW ACTION
	public final static String MOVE_HOME_ACTION = "MOVE_HOME_ACTION" ;
	public final static String MOVE_PREVIOUS_ACTION = "MOVE_PREVIOUS_ACTION" ;
	public final static String MOVE_NEXT_ACTION = "MOVE_NEXT_ACTION" ;
	
	public final static String LEFT_PANEL_DISPLAY_ACTION = "LEFT_PANEL_DISPLAY_ACTION" ;
	
	public final static String DAY_CHANGE = "DAY_CHANGE" ;
	
	public final static String MOVE_DAY = "MOVE_DAY" ;
	public final static String MOVE_WEEK = "MOVE_WEEK";
	public final static String MOVE_MONTH = "MOVE_MONTH" ;
	public final static String MOVE_YEAR = "MOVE_YEAR" ;
	public final static String PERIOD_MOVED = "PERIOD_MOVED" ;

	public final static String SHOW_LEFT_PANEL = "SHOW_LEFT_PANEL" ;
	public final static String HIDE_LEFT_PANEL = "HIDE_LEFT_PANEL" ;
	public final static String LEFT_PANEL_DISPLAY_SELECTION = "LEFT_PANEL_DISPLAY_SELECTION" ;
	
	public final static String MONTH_VIEW_MODE = "MONTH_VIEW_MODE" ;
	public final static String WEEK_VIEW_MODE = "WEEK_VIEW_MODE" ;
	public final static String DAY_VIEW_MODE = "DAY_VIEW_MODE" ;
	
	public final static String DAY_PARAM = "DAY" ;
	public final static String DAY_OF_WEEK_PARAM = "DAY" ;
	public final static String WEEK_PARAM = "WEEK" ;
	public final static String MONTH_PARAM = "MONTH" ;
	public final static String YEAR_PARAM = "YEAR" ;
	public final static String EVENT_ID_PARAM = "EVENT_ID" ;

	// Event action parameters
	public final static Parameter VIEW_DAY_PARAM = new Parameter(ACTION, VIEW_DAY_ACTION);
	public final static Parameter VIEW_WEEK_PARAM = new Parameter(ACTION, VIEW_WEEK_ACTION);
	public final static Parameter VIEW_MONTH_PARAM = new Parameter(ACTION, VIEW_MONTH_ACTION);
	public final static Parameter VIEW_EVENT_PARAM = new Parameter(ACTION, VIEW_ACTION);
	public final static Parameter EDIT_EVENT_PARAM = new Parameter(ACTION, EDIT_ACTION);

	public final static Parameter MOVE_PREVIOUS_PARAM = new Parameter(ACTION, MOVE_PREVIOUS_ACTION);
	public final static Parameter MOVE_NEXT_PARAM = new Parameter(ACTION, MOVE_NEXT_ACTION);
	
	public final static Parameter LEFT_PANEL_DISPLAY_PARAM = new Parameter(ACTION, LEFT_PANEL_DISPLAY_ACTION);
	
	
	private String currentUser_ ;
	private String currentCalendar_  ;
	private String currentCalendarType_  ;
	private GregorianCalendar selectDate_ ;
	private int selectMonth_ ;
	private int selectDayOfWeek_ ;
	private CalendarService calendarService_ ;

	private String viewMode_ = UICalendarPortlet.MONTH_VIEW_MODE ;
	private boolean displayLeftPanel_ = true ;

	public UICalendarPortlet(CalendarService calendarService)throws Exception {
		setId("UICalendarPortlet") ;
		setClazz("UICalendarPortlet") ;
		setRendererType("ChildrenRenderer") ;
		
		currentUser_ = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
		if (currentUser_ == null || "".equals(currentUser_)) currentUser_ = "anonymous" ;
		calendarService.createUserCalendarIfNeeded(currentUser_) ;
    currentCalendar_ = CalendarService.DEFAULT_CALENDAR  ;
    currentCalendarType_ = CalendarService.DEFAULT_CALENDAR_TYPE  ;
		
		selectDate_ = new GregorianCalendar() ;
		selectMonth_ = selectDate_.get(Calendar.MONTH) ;
		selectDayOfWeek_ = selectDate_.get(Calendar.DAY_OF_WEEK) ;
		calendarService_ = calendarService ;

		addChild(UIToolbarPanel.class) ;
		addChild(UICalendar.class) ;
		addChild(UICalendarEventForm.class).setRendered(false) ;
		addChild(UICalendarManagement.class).setRendered(false) ;
		addChild(UICalendarForm.class).setRendered(false) ;
	}
	
	public CalendarService getCalendarService() {
		return this.calendarService_ ;
	}

	public boolean isLeftPanelDisplayed() {
		return displayLeftPanel_ ;
	}

	public void setDisplayLeftPanel(boolean displayLeftPanel) {
		this.displayLeftPanel_ = displayLeftPanel ;
	}

	public GregorianCalendar getSelectDate() {
		return selectDate_ ;
	}
	
	public void setSelectDate(GregorianCalendar date) {
		selectDate_ = date ;
		update() ;
	}

	public void setSelectDate(int year, int month, int day) {
		selectDate_.set(year, month, day) ;
		update() ;
	}
	
	private void update() {
		selectMonth_ = selectDate_.get(Calendar.MONTH) ;
		selectDayOfWeek_ = selectDate_.get(Calendar.DAY_OF_WEEK) ;
	}

	public int getSelectMonth() {
		return selectMonth_ ;
	}
	
	private void setSelectMonth(int month) {
		this.selectMonth_ = month ;
	}
	
	public int getSelectDayOfWeek() {
		return selectDayOfWeek_ ;
	}

	private void setSelectDayOfWeek(int selectDayOfWeek) {
		this.selectDayOfWeek_ = selectDayOfWeek ;
	}

	public String getViewMode() {
		return viewMode_ ;
	}

	public void setViewMode(String viewMode) {
		this.viewMode_ = viewMode ;
	}

	public String getCurrentCalendarType() {
		return currentCalendarType_;
	}

	public void setCurrentCalendarType(String currentCalendarShareType) {
		this.currentCalendarType_ = currentCalendarShareType;
	}

	public String getCurrentCalendar() {
		return currentCalendar_;
	}

	public boolean isCurrentCalendar(String type, String name) {
		if (this.getCurrentCalendarType().equals(type) && 
				this.getCurrentCalendar().equals(name))
			return true ;
		return false ;
	}

	public void setCurrentCalendar(String currentCalendar) {
		this.currentCalendar_ = currentCalendar;
	}

	public String getCurrentUser() {
		return currentUser_;
	}
}
