/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.renderer.html;

import java.io.IOException;
import java.lang.Math;
import java.util.*;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.jcr.Node;
import javax.jcr.RepositoryException;


import org.exoplatform.commons.utils.CalendarUtil;
import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.renderer.html.HtmlBasicRenderer;
import org.exoplatform.portlets.calendar.component.*;

import java.text.SimpleDateFormat;


public class CalendarRenderer extends HtmlBasicRenderer {
	
	// Effective ui parameters
	private Parameter eventIdParam_ = new Parameter(UICalendarPortlet.EVENT_ID_PARAM, "");
	private Parameter dayParam_ = new Parameter(UICalendarPortlet.DAY_PARAM, "");
	private Parameter monthParam_ = new Parameter(UICalendarPortlet.MONTH_PARAM, "");
	private Parameter yearParam_ = new Parameter(UICalendarPortlet.YEAR_PARAM, "");
	private Parameter periodMovedParam_ = new Parameter(UICalendarPortlet.PERIOD_MOVED, "");

	private Parameter leftPanelDisplaySelectionParam_ = new Parameter(UICalendarPortlet.LEFT_PANEL_DISPLAY_SELECTION, "");
	
	// Rendered ui parameters
	private Parameter[] viewDayParams_ = { UICalendarPortlet.VIEW_DAY_PARAM, dayParam_, monthParam_, yearParam_ };
	private Parameter[] viewWeekParams_ = { UICalendarPortlet.VIEW_WEEK_PARAM, dayParam_, monthParam_, yearParam_ };
//	private Parameter[] viewMonthParams_ = { UICalendarPortlet.VIEW_MONTH_PARAM, dayParam_, monthParam_, yearParam_ };
	private Parameter[] viewEventParams_ = { UICalendarPortlet.VIEW_EVENT_PARAM, eventIdParam_ };
//	private Parameter[] editEventParams_ = { UICalendarPortlet.EDIT_EVENT_PARAM, eventIdParam_ };

	private Parameter[] movePreviousParams_ = { UICalendarPortlet.MOVE_PREVIOUS_PARAM, periodMovedParam_ };
	private Parameter[] moveNextParams_ = { UICalendarPortlet.MOVE_NEXT_PARAM, periodMovedParam_ };

	private Parameter[] leftPanelDisplayParams_ = { UICalendarPortlet.LEFT_PANEL_DISPLAY_PARAM, leftPanelDisplaySelectionParam_ };
	
	private SimpleDateFormat timeFormat_ = new SimpleDateFormat("HH:mm");

	final public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
		ResourceBundle res = getApplicationResourceBundle(context.getExternalContext());
		ResponseWriter w = context.getResponseWriter();
		UICalendar uiCalendar = (UICalendar) component;
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;

    w.write("<div class='" + uiCalendar.getClazz() + "'>");
		w.write("  <table>");
		w.write("    <tr>");
    if (uiCalendarPortlet.isLeftPanelDisplayed()) {
			w.write("      <td>") ;
    	renderChildren(context, uiCalendar) ;
			w.write("      </td>") ;
		}
		w.write("      <td>") ;
		w.write("        <table>");
    if (!uiCalendarPortlet.isLeftPanelDisplayed()) {
  		w.write("          <tr>");
			w.write("            <td class='showLeftPanel'>");
			leftPanelDisplaySelectionParam_.setValue(UICalendarPortlet.SHOW_LEFT_PANEL);
			buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.showLeftPanel"), leftPanelDisplayParams_);
			w.write("            </td>");
			w.write("          </tr>");
			w.write("          <tr>");
			w.write("            <td>") ;
			w.write(new StringBuffer(uiCalendarPortlet.getCurrentCalendar()).append(" (").append(uiCalendarPortlet.getCurrentCalendarType())
					.append(")").toString());
			w.write("            </td>") ;
			w.write("          </tr>");
		}
		w.write("          <tr>");
		w.write("            <td class='period_title'>");
    if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.MONTH_VIEW_MODE)) {
  		// Title
  		periodMovedParam_.setValue(UICalendarPortlet.MOVE_MONTH);
  		buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.previous"), movePreviousParams_);
  		SimpleDateFormat dateFormat = new SimpleDateFormat("MMMMMMMMMM");
  		w.write(dateFormat.format(uiCalendarPortlet.getSelectDate().getTime()));
  		buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.next"), moveNextParams_);
  		w.write("</td>");
  		w.write("</tr>");
  		w.write("</table>");
			addCalendarByMonth(w, res, uiCalendar);
		} else if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.WEEK_VIEW_MODE)) {
			// Title
			periodMovedParam_.setValue(UICalendarPortlet.MOVE_WEEK);
			buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.previous"), movePreviousParams_);
			SimpleDateFormat dateFormat = new SimpleDateFormat("d MMMMMMMMMM");
			GregorianCalendar currentDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
			if (currentDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) currentDate.add(Calendar.DATE, -6);
			else currentDate.add(Calendar.DATE, Calendar.MONDAY - currentDate.get(Calendar.DAY_OF_WEEK));
			GregorianCalendar endDate = (GregorianCalendar) currentDate.clone();
			endDate.add(Calendar.DATE, 6);
			w.write(dateFormat.format(currentDate.getTime()) + " - " + dateFormat.format(endDate.getTime()));  
			buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.next"), moveNextParams_);
			w.write("</td>");
			w.write("</tr>");
			w.write("</table>");
			addCalendarByWeek(w, res, uiCalendar);
		} else if (uiCalendarPortlet.getViewMode().equals(UICalendarPortlet.DAY_VIEW_MODE)) {
			// Title
			periodMovedParam_.setValue(UICalendarPortlet.MOVE_DAY);
			buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.previous"), movePreviousParams_);
			SimpleDateFormat dateFormat = new SimpleDateFormat("EEEEEEEEEE d MMMMMMMMMM");
			GregorianCalendar currentDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
			w.write(dateFormat.format(currentDate.getTime()));  
			buttonRenderer_.render(w, uiCalendar, res.getString("UICalendarPortlet.button.next"), moveNextParams_);
			w.write("</td>");
			w.write("</tr>");
			w.write("</table>");
			addCalendarByDay(w, res, uiCalendar);
		}
		w.write("      </td>") ;
		w.write("    </tr>");
		w.write("  </table>");
    w.write("</div>");
	}

	private void fillEventList(List events, List allDayEvents, List otherEvents, GregorianCalendar currentDate) {
		GregorianCalendar dtstart, dtend;
		Iterator iter = events.iterator();
		while (iter.hasNext()) {
			Node node = (Node) iter.next();
			try {
				dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
				dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
				if (CalendarUtil.compareDate(dtstart, currentDate) < 0 | 
						CalendarUtil.compareDate(dtend, currentDate) > 0) allDayEvents.add(node);
				else otherEvents.add(node);
			} 
			catch (RepositoryException e) {
				e.printStackTrace();
			}
		}        
	}
	
	private void addCalendarByMonth(ResponseWriter w, ResourceBundle res, UICalendar uiCalendar) throws IOException {
		StringBuffer strBuf;
		String str;
		
		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;

		// Calendar
		w.write("<table class='UICalendarByMonth'>");
		w.write("<tr>");
		w.write("<th class='week'>");
		w.write("</th>");

		SimpleDateFormat dateFormat = new SimpleDateFormat("EEEEEEEEEE");		
		GregorianCalendar gCal = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		if (gCal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) gCal.add(Calendar.DATE, -6);
		else gCal.add(Calendar.DATE, Calendar.MONDAY - gCal.get(Calendar.DAY_OF_WEEK));
		for (int i = 0; i < 7; i++) {
			w.write("<th>");
			w.write(dateFormat.format(gCal.getTime()));
			gCal.add(Calendar.DATE, 1);
			w.write("</th>");
		}
		w.write("</tr>");
		
		GregorianCalendar startDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		startDate.set(Calendar.DATE, startDate.getActualMinimum(Calendar.DATE));
		if (startDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) startDate.add(Calendar.DATE, -6); 
		else startDate.add(Calendar.DATE, Calendar.MONDAY - startDate.get(Calendar.DAY_OF_WEEK));

		GregorianCalendar endDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		endDate.set(Calendar.DATE, endDate.getActualMaximum(Calendar.DATE));
		if (endDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) endDate.add(Calendar.DATE, 1); 
		else endDate.add(Calendar.DATE, Calendar.SATURDAY - endDate.get(Calendar.DAY_OF_WEEK) + Calendar.SUNDAY);
		
		GregorianCalendar currentDate = (GregorianCalendar) startDate.clone();
		GregorianCalendar today = new GregorianCalendar();
		GregorianCalendar selectDate = uiCalendarPortlet.getSelectDate();
		
		dateFormat.applyPattern("d");
		boolean lastDay = false;
		GregorianCalendar dateInWeek;
		while (!lastDay) {
			w.write("<tr>");
			//week column
			w.write("<td  class='week'>");
			dateInWeek = (GregorianCalendar) currentDate.clone();
			while (dateInWeek.get(Calendar.DAY_OF_WEEK) != uiCalendarPortlet.getSelectDayOfWeek()) {
				dateInWeek.add(Calendar.DATE, 1);
			}
			dayParam_.setValue(new Integer(dateInWeek.get(Calendar.DATE)).toString());
			monthParam_.setValue(new Integer(dateInWeek.get(Calendar.MONTH)).toString());
			yearParam_.setValue(new Integer(dateInWeek.get(Calendar.YEAR)).toString());
			linkRenderer_.render(w, uiCalendar, new Integer(dateInWeek.get(Calendar.WEEK_OF_YEAR)).toString(), viewWeekParams_);
			w.write("</td>");
			
			// 7 day columns
			for (int i = 0; i < 7; i++) {
				if (currentDate.get(Calendar.MONTH) != selectDate.get(Calendar.MONTH)) {
					w.write("<td class='outOfMonth'>");
				} 
				else if (currentDate.get(Calendar.YEAR) == today.get(Calendar.YEAR)
						&& currentDate.get(Calendar.MONTH) == today.get(Calendar.MONTH)
						&& currentDate.get(Calendar.DATE) == today.get(Calendar.DATE)) {
					w.write("<td class='today'>");
				} 
				else w.write("<td  class='normal'>");
				dayParam_.setValue(new Integer(currentDate.get(Calendar.DATE)).toString());
				monthParam_.setValue(new Integer(currentDate.get(Calendar.MONTH)).toString());
				yearParam_.setValue(new Integer(currentDate.get(Calendar.YEAR)).toString());
				w.write("<div class='day'>");
				linkRenderer_.render(w, uiCalendar, dateFormat.format(currentDate.getTime()), viewDayParams_);
				w.write("</div>");
				
				// Events
				List l = uiCalendar.findCalendarEventByUserAndDate(currentDate);
				if (l != null) {
		      int nbEvents = 0;
					SimpleDateFormat df = new SimpleDateFormat("dd/MM");
					boolean displayEvent;
					GregorianCalendar dtstart, dtend;
					List allDayEvents = new Vector();
					List otherEvents = new Vector();
					fillEventList(l, allDayEvents, otherEvents, currentDate);
					
					// All-day events
					Iterator iter = allDayEvents.iterator();
					while (iter.hasNext()) {
						Node node = (Node) iter.next();
						displayEvent = true;
						strBuf = new StringBuffer();
						str = "";
						try {
							eventIdParam_.setValue(node.getPath());
							dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
							dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());

							if (CalendarUtil.compareDate(dtstart, currentDate) < 0) {
								if (CalendarUtil.compareDate(dtend, currentDate) == 0) {
									strBuf.append(timeFormat_.format(dtstart.getTime()));
									strBuf.append(" (");
									strBuf.append(df.format(dtstart.getTime()));
									strBuf.append(") - ");
									strBuf.append(timeFormat_.format(dtend.getTime()));
									strBuf.append("<br/>");
								}
							}
							else {
								strBuf.append(timeFormat_.format(dtstart.getTime()));
								strBuf.append(" ");
							}
							
							str = strBuf.append(node.getProperty("exo:summary").getValue().getString()).toString();
							str = str.substring(0, 30) + "...";
						} 
						catch (IndexOutOfBoundsException e) {
						}
						catch (RepositoryException e) {
							e.printStackTrace();
							displayEvent = false;
						}
						if (displayEvent) {
							nbEvents++;
							if (nbEvents == 1) w.write("<div class='firstAllDayEvent'>");
							else w.write("<div class='allDayEvent'>");
							linkRenderer_.render(w, uiCalendar, str, viewEventParams_);
							w.write("</div>");
						}
					}        

					// other events
					iter = otherEvents.iterator();
					while (iter.hasNext()) {
						Node node = (Node) iter.next();
						displayEvent = true;
						strBuf = new StringBuffer();
						str = "";
						try {
							eventIdParam_.setValue(node.getPath());
							dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
							dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
							
							strBuf.append(timeFormat_.format(dtstart.getTime()));
							strBuf.append(" ");
							str = strBuf.append(node.getProperty("exo:summary").getValue().getString()).toString();
							str = str.substring(0, 30) + "...";
						} 
						catch (IndexOutOfBoundsException e) {
						}
						catch (RepositoryException e) {
							e.printStackTrace();
							displayEvent = false;
						}
						if (displayEvent) {
							nbEvents++;
							if (nbEvents == 1) w.write("<div class='firstEvent'>");
							else w.write("<div class='event'>");
							linkRenderer_.render(w, uiCalendar, str, viewEventParams_);
							w.write("</div>");
						}
					}        
				}
				w.write("</td>");
				if (currentDate.get(Calendar.YEAR) == endDate.get(Calendar.YEAR)
						&& currentDate.get(Calendar.MONTH) == endDate.get(Calendar.MONTH)
						&& currentDate.get(Calendar.DATE) == endDate.get(Calendar.DATE)) {
					lastDay = true;
				}
				currentDate.add(Calendar.DATE, 1);
			}
			w.write("</tr>") ;
		}
		w.write("</table>") ;
	}

	private void addCalendarByWeek(ResponseWriter w, ResourceBundle res, UICalendar uiCalendar) throws IOException {
		StringBuffer strBuf;
		String str;
		String summary;
		SimpleDateFormat dateFormat = new SimpleDateFormat("EEE");

		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
		GregorianCalendar currentDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		if (currentDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) currentDate.add(Calendar.DATE, -6);
		else currentDate.add(Calendar.DATE, Calendar.MONDAY - currentDate.get(Calendar.DAY_OF_WEEK));
		
		w.write("<table class='UICalendarByWeek'>");
		for (int i = 0; i < 7; i++) {
            int nbEvents = 0;
			w.write("<tr>");
			w.write("<td class='day'>");
			dayParam_.setValue(new Integer(currentDate.get(Calendar.DATE)).toString());
			monthParam_.setValue(new Integer(currentDate.get(Calendar.MONTH)).toString());
			yearParam_.setValue(new Integer(currentDate.get(Calendar.YEAR)).toString());
			linkRenderer_.render(w, uiCalendar, dateFormat.format(currentDate.getTime())+ "<br/>" 
					+ new Integer(currentDate.get(Calendar.DATE)).toString(), viewDayParams_);
			w.write("</td>");
			
			w.write("<td class='eventList'>");
			List l = uiCalendar.findCalendarEventByUserAndDate(currentDate);
			if (l != null) {
				SimpleDateFormat df = new SimpleDateFormat("dd/MM");
				boolean displayEvent;
				GregorianCalendar dtstart, dtend;
				List allDayEvents = new Vector();
				List otherEvents = new Vector();
				fillEventList(l, allDayEvents, otherEvents, currentDate);
				
				// All-day events
				Iterator iterAllDayEvents = allDayEvents.iterator();
				Iterator iterOtherEvents = otherEvents.iterator();
				while (iterAllDayEvents.hasNext()) {
					Node node = (Node) iterAllDayEvents.next();
					displayEvent = true;
					strBuf = new StringBuffer();
					str = "";
					summary = "";
					try {
						eventIdParam_.setValue(node.getPath());
						dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
						dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
						
						strBuf.append(df.format(dtstart.getTime()));
						strBuf.append(" (");
						strBuf.append(timeFormat_.format(dtstart.getTime()));
						strBuf.append(") - <br/>");
						strBuf.append(df.format(dtend.getTime()));
						strBuf.append(" (");
						strBuf.append(timeFormat_.format(dtend.getTime()));
						strBuf.append(") - ");
						str = strBuf.toString();
						summary = node.getProperty("exo:summary").getValue().getString();
						summary = summary.substring(0, 40) + "...";
					} 
					catch (IndexOutOfBoundsException e) {
					}
					catch (RepositoryException e) {
						e.printStackTrace();
						displayEvent = false;
					}
					if (displayEvent) {
						nbEvents++;
						if (nbEvents < 3) w.write("<div class='allDayEvent'>");
						else if (iterAllDayEvents.hasNext() || iterOtherEvents.hasNext()) w.write("<div class='allDayEvent'>");
						else w.write("<div class='allDayEventLast'>");
						w.write(str);
						linkRenderer_.render(w, uiCalendar, summary, viewEventParams_);
						w.write("</div>");
					}
				}        

				// other events
				while (iterOtherEvents.hasNext()) {
					Node node = (Node) iterOtherEvents.next();
					displayEvent = true;
					strBuf = new StringBuffer();
					str = "";
					summary = "";
					try {
						eventIdParam_.setValue(node.getPath());
						dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
						dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
						
						strBuf.append(timeFormat_.format(dtstart.getTime()));
						strBuf.append(" - ");
						strBuf.append(timeFormat_.format(dtend.getTime()));
						strBuf.append(" - ");
						str = strBuf.toString();
						summary = node.getProperty("exo:summary").getValue().getString();
						summary = summary.substring(0, 50) + "...";
					} 
					catch (IndexOutOfBoundsException e) {
					}
					catch (RepositoryException e) {
						e.printStackTrace();
						displayEvent = false;
					}
					if (displayEvent) {
						nbEvents++;
						if (nbEvents < 3) w.write("<div class='event'>");
						else if (iterOtherEvents.hasNext()) w.write("<div class='event'>");
						else w.write("<div class='eventLast'>");
						w.write(str);
						linkRenderer_.render(w, uiCalendar, summary, viewEventParams_);
						w.write("</div>");
					}
				}        
			}
			for (int j = nbEvents; j < 3; j++) {
				if (j < 2) w.write("<div class='noEvent'>");
				else w.write("<div class='noEventLast'>");
				w.write("</div>");
			}
			w.write("</td>");
			w.write("</tr>");
			w.write("</table>") ;
			currentDate.add(Calendar.DATE, 1);
		}
	}
	
	private void addCalendarByDay(ResponseWriter w, ResourceBundle res, UICalendar uiCalendar) throws IOException {
		StringBuffer strBuf;
		String str;
		String summary;
		List allDayEvents = new Vector();
		List otherEvents = new Vector();
        int nbEvents;
        int hourMin = 8, hourMax = 20;
		GregorianCalendar dtstart, dtend;
		SimpleDateFormat df = new SimpleDateFormat("dd/MM");
		boolean displayEvent;

		UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendar.getAncestorOfType(UICalendarPortlet.class) ;
		GregorianCalendar currentDate = (GregorianCalendar) uiCalendarPortlet.getSelectDate().clone();
		
		List l = uiCalendar.findCalendarEventByUserAndDate(currentDate);
		fillEventList(l, allDayEvents, otherEvents, currentDate);

		w.write("<table class='UICalendarByDay'>");
		w.write("<tr>");
		w.write("<td class='hour'>");
		w.write(res.getString("UICalendarPortlet.label.allDay"));
		w.write("</td>");
		
		// All-day events
		w.write("<td class='eventList'>");
        Iterator iter = allDayEvents.iterator();
        if (l == null || !iter.hasNext()) {
			w.write("<div class='noEvent'>");
			w.write("&nbsp");
			w.write("</div>");
        }
		while (iter.hasNext()) {
			Node node = (Node) iter.next();
			displayEvent = true;
			strBuf = new StringBuffer();
			str = "";
			summary = "";
			try {
				eventIdParam_.setValue(node.getPath());
				dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
				dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
				
				strBuf.append(df.format(dtstart.getTime()));
				strBuf.append(" (");
				strBuf.append(timeFormat_.format(dtstart.getTime()));
				strBuf.append(") - <br/>");
				strBuf.append(df.format(dtend.getTime()));
				strBuf.append(" (");
				strBuf.append(timeFormat_.format(dtend.getTime()));
				strBuf.append(") - ");
				str = strBuf.toString();
				summary = node.getProperty("exo:summary").getValue().getString();
				summary = summary.substring(0, 40) + "...";
			} 
			catch (IndexOutOfBoundsException e) {
			}
			catch (RepositoryException e) {
				e.printStackTrace();
				displayEvent = false;
			}
			if (displayEvent) {
				if (iter.hasNext()) w.write("<div class='allDayEvent'>");
				else w.write("<div class='allDayEventLast'>");
				w.write(str);
				linkRenderer_.render(w, uiCalendar, summary, viewEventParams_);
				w.write("</div>");
			}
		}        
		w.write("</td>");
		w.write("</tr>");

		// other events
        iter = otherEvents.iterator();
		while (iter.hasNext()) {
			Node node = (Node) iter.next();
			try {
				dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
				dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
				hourMin = Math.min(dtstart.get(GregorianCalendar.HOUR_OF_DAY), hourMin);
				hourMax = Math.max(dtend.get(GregorianCalendar.HOUR_OF_DAY), hourMax);
			}
			catch (RepositoryException e) {
				e.printStackTrace();
			}
		}        
		hourMax++;
		
		for (int i = hourMin; i < hourMax; i++) {
			nbEvents = 0;
			w.write("<tr>");
			w.write("<td class='hour'>");
			w.write(new Integer(i).toString());
			w.write("</td>");
			
			w.write("<td class='eventList'>");
			iter = otherEvents.iterator();
			Node nextNode = null;
			while (iter.hasNext() || nextNode != null) {
				Node node;
				if (nextNode != null) {
					node = nextNode;
					nextNode = null;
				}
				else node = (Node) iter.next();
				displayEvent = true;
				strBuf = new StringBuffer();
				str = "";
				summary = "";
				try {
					dtstart = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtstart").getValue().getString());
					if (dtstart.get(GregorianCalendar.HOUR_OF_DAY) == i) {
						dtend = (GregorianCalendar) ISO8601.parse(node.getProperty("exo:dtend").getValue().getString());
						eventIdParam_.setValue(node.getPath());
						
						strBuf.append(timeFormat_.format(dtstart.getTime()));
						strBuf.append(" - ");
						strBuf.append(timeFormat_.format(dtend.getTime()));
						strBuf.append(" ");
						str = strBuf.toString();
						summary = node.getProperty("exo:summary").getValue().getString();
						summary = summary.substring(0, 50) + "...";
					}
					else displayEvent = false;
				} 
				catch (IndexOutOfBoundsException e) {
				}
				catch (RepositoryException e) {
					e.printStackTrace();
					displayEvent = false;
				}
				if (displayEvent) {
					nbEvents++;
					if (iter.hasNext()) {
						nextNode = (Node) iter.next();
						try {
							dtstart = (GregorianCalendar) ISO8601.parse(nextNode.getProperty("exo:dtstart").getValue().getString());
							if (dtstart.get(GregorianCalendar.HOUR_OF_DAY) != i) displayEvent = false;
						}
						catch (RepositoryException e) {
							e.printStackTrace();
						}
						if (displayEvent) w.write("<div class='event'>");
						else w.write("<div class='eventLast'>");
					}
					else w.write("<div class='eventLast'>");
					w.write(str);
					linkRenderer_.render(w, uiCalendar, summary, viewEventParams_);
					w.write("</div>");
				}
			}        
			if (nbEvents == 0) {
				w.write("<div class='noEvent'>");
				w.write("&nbsp");
				w.write("</div>");
			}
			w.write("</td>");
			w.write("</tr>");
		}
		w.write("</table>") ;
	}
	
	final public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
	}
}