/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.calendar.impl ;


import java.util.* ;

import javax.faces.context.FacesContext ;
import javax.jcr.query.Query ;
import javax.jcr.query.QueryResult ;
import javax.jcr.query.RowIterator ;
import javax.jcr.query.Row ;
import javax.jcr.Node ;
import javax.jcr.NodeIterator ;
import javax.jcr.PathNotFoundException ;
import javax.jcr.Repository ;
import javax.jcr.RepositoryException ;
import javax.jcr.Session ;
import javax.jcr.Value ;

import org.apache.commons.logging.Log;
import org.exoplatform.services.idgenerator.IDGeneratorService ;
import org.exoplatform.commons.utils.ISO8601 ;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.xml.InitParams ;
import org.exoplatform.services.calendar.CalendarService ;
import org.exoplatform.services.calendar.CalendarConfig ;
import org.exoplatform.services.cms.BasePath ;
import org.exoplatform.services.cms.CmsConfigurationService ;
import org.exoplatform.services.cms.CmsService ;
import org.exoplatform.services.cms.JcrInputProperty ;
import org.exoplatform.services.cms.categories.CategoriesService;
import org.exoplatform.services.jcr.RepositoryService ;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException ;
import org.exoplatform.services.jcr.core.ManageableRepository ;
import org.exoplatform.services.log.LogService;
import org.picocontainer.Startable;

public class CalendarServiceImpl implements CalendarService, Startable {
	private CmsService cmsService_ ;
  private CmsConfigurationService cmsConfigService_ ;
  private RepositoryService jcrService_  ;
  private IDGeneratorService idService_ ;
  private CategoriesService categoriesService_ ;
  private Log log_ ;
  private String workspace_  ;
  private Session session_ ;

  public CalendarServiceImpl(CmsService cmsService,
                         CmsConfigurationService cmsConfigService,
                         RepositoryService jcrService,
                         LogService logService,
                         CategoriesService categoriesService,
                         InitParams params,
                         IDGeneratorService idService) throws Exception {
    this.cmsService_ = cmsService ;
    this.cmsConfigService_ = cmsConfigService ;
    this.jcrService_ = jcrService  ;
    this.idService_ = idService ;
    this.categoriesService_ = categoriesService ;
    this.log_ = logService.getLog(this.getClass()) ;

    List configs = params.getObjectParamValues(CalendarConfig.class)  ;
    this.workspace_ = ""  ;
    for(int i = 0  ; i  < configs.size() ; i++) {
      CalendarConfig config = (CalendarConfig)configs.get(i)  ;
      if (this.workspace_ == ""){
      	this.workspace_ = config.getWorkspace()  ;
      }
    }
    this.session_ = null ;
//    init() ;
  }

  public void start() {
    Session session = null ;
    try {
      ManageableRepository jcrRepository = this.jcrService_.getRepository() ;
      session = jcrRepository.getSystemSession(this.workspace_) ;
    } catch (RepositoryConfigurationException re) {
    	log_.error("RepositoryConfigurationException during calendar service initialization : getSystemSession(" 
    				+ this.workspace_ + ")", re) ;
      return ;
    } catch (RepositoryException re) {
      // no production and draft workspace configured for the portal = no cms for the portal instance
    	log_.error("RepositoryException during calendar service initialization : getSystemSession(" 
    			+ this.workspace_ + ")", re) ;
      return ;
    } 
    try {
	    Node root = session.getRootNode() ;                  
	    String usersPath = new String(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) ; 
	    if(usersPath.startsWith("/")) usersPath = usersPath.substring(1)  ;
	    if(!root.hasNode(usersPath)){
	      Utils.makePath(root, usersPath, "nt:unstructured") ;
	      session.save() ;
	    }
    } catch (Exception e) {
    	log_.error("RepositoryException during calendar service initialization : users path creation", e) ;
      return ;
    }
  }

  public void stop() {

  }

  //used the first time when portal be initialized 
  private void init() throws Exception{
    ManageableRepository jcrRepository = this.jcrService_.getRepository() ;
    Session session = null ;
    try {
      session = jcrRepository.getSystemSession(this.workspace_) ;
    } catch (RepositoryException re) {
      // no production and draft workspace configured for the portal = no cms for the portal instance
    	log_.error("RepositoryException while initializing calendar service : getSystemSession(" + this.workspace_ + ")", re) ;
      return ;
    }
    Node root = session.getRootNode() ;                  
    String usersPath = new String(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) ; 
    if(usersPath.startsWith("/")) usersPath = usersPath.substring(1)  ;
    if(!root.hasNode(usersPath)){
      Utils.makePath(root, usersPath, "nt:unstructured") ;
      session.save() ;
    }
  }
  
  public Session getSession()  {
  	if (session_ == null || (session_ != null && !session_.isLive()) ) {
  		session_ = null ;
      try {
        Repository jcrRepository = this.jcrService_.getRepository() ;
        session_ = jcrRepository.login(this.workspace_) ;
      } 
      catch (RepositoryException e) {
        // no production and draft workspace configured for the portal = no cms for the portal instance
      	log_.error("RepositoryException while getting session", e) ;
      }
      catch (RepositoryConfigurationException e) {
      	log_.error("RepositoryConfigurationException while getting session", e) ;
      }
  	}
  	return session_ ; 
  }

  public Session getSystemSession()  {
    try {
      ManageableRepository jcrRepository = this.jcrService_.getRepository() ;
      return jcrRepository.getSystemSession(this.workspace_) ;
    } 
    catch (RepositoryException e) {
      // no production and draft workspace configured for the portal = no cms for the portal instance
    	log_.error("RepositoryException while getting system session", e) ;
      return null ;
    }
    catch (RepositoryConfigurationException e) {
    	log_.error("RepositoryConfigurationException while getting system session", e) ;
      return null ;
    }
  }

  public void createUserCalendarIfNeeded(String user) throws Exception {
    Session session = this.getSession() ;
    String userCalendarPath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
    		.append("/").append(user).
    		append("/calendar").toString() ;
    if(userCalendarPath.startsWith("/")) userCalendarPath = userCalendarPath.substring(1) ;
    Node root = session.getRootNode() ;                  
    if(!root.hasNode(userCalendarPath)) {
      this.createUserCalendar(user) ;
    }
  }
  
  // Used when new user created
  public void createUserCalendar(String user) throws Exception{
  	Session session = this.getSystemSession() ;
    Node root = session.getRootNode() ;
    String userPath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
    		.append("/").append(user).toString() ;
    if(userPath.startsWith("/")) userPath = userPath.substring(1)  ;
    Node usersHome = (Node) session.getItem(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) ;
    Node userHome ;
    if(!root.hasNode(userPath)){
        userHome = usersHome.addNode(user, "nt:unstructured") ;
    }
    else {
    	String rootPath = root.getPath()  ;
      userHome = (Node) session.getItem("/" + userPath) ;
    }
    // create "/calendar" node if not exists
    Node calendarHome ;
    if (!usersHome.hasNode(user + "/calendar")) 
      calendarHome = userHome.addNode("calendar", "nt:unstructured") ;
    else calendarHome = usersHome.getNode(user + "/calendar") ;
    
    // create "/calendar/private" node if not exists
    Node privateCalendar ;
    if (!userHome.hasNode("calendar/private")) 
      privateCalendar = calendarHome.addNode("private", "nt:unstructured") ;
    else privateCalendar = userHome.getNode("calendar/private") ;
    
    // create default private calendar if not exists
    Node defaultCalendar ;
    if (!calendarHome.hasNode("private/default"))
      defaultCalendar = privateCalendar.addNode("default", "exo:calendar") ;
    else defaultCalendar = calendarHome.getNode("private/default") ;
    defaultCalendar.setProperty("exo:shared", false) ;
    // create private node if not exists
    Node eventShareCategoryNode ;
    String path = new StringBuffer("default/").
    					append(CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY).toString()  ;
    if (!privateCalendar.hasNode(path)) {
    	eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY, 
    			"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY) ;
    }
    // create free-busy node if not exists
    path = new StringBuffer("default/").
		append(CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY).toString() ;    	
    if (!privateCalendar.hasNode(path)) {
    	eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY, 
    			"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY) ;
    }
    // create detail node if not exists
    path = new StringBuffer("default/").
    	append(CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY).toString() ;    	
    if (!privateCalendar.hasNode(path)) {
    	eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY, 
    			"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY) ;
    }

    // create "/calendar/favorites" node if not exists
    if (!userHome.hasNode("calendar/favourites")) 
      calendarHome.addNode("favourites", "nt:unstructured") ;
    
    // create "/calendar/userLists" node if not exists
    if (!userHome.hasNode("calendar/userLists")) 
      calendarHome.addNode("userLists", "nt:unstructured") ;

    // create "/calendar/sharedWithMe" node if not exists
    if (!userHome.hasNode("calendar/sharedWithMe")) 
      calendarHome.addNode("sharedWithMe", "nt:unstructured") ;
    

    // create "/calendar/public" node if not exists
    Node publicCalendar ;
    if (!userHome.hasNode("calendar/public")) 
      publicCalendar = calendarHome.addNode("public", "nt:unstructured") ;
    else publicCalendar = userHome.getNode("calendar/public") ;

    // create default public calendar if not exists
    if (!calendarHome.hasNode("public/default"))
      defaultCalendar = publicCalendar.addNode("default", "exo:calendar") ;
    else defaultCalendar = calendarHome.getNode("public/default") ;
    defaultCalendar.setProperty("exo:shared", true) ;
    // create private node if not exists
    path = new StringBuffer("default/").
		append(CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY).toString() ;
		if (!publicCalendar.hasNode(path)) {
			eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY, 
					"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY) ;
		}
    // create free-busy node if not exists
		path = new StringBuffer("default/").
			append(CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY).toString() ;    	
		if (!publicCalendar.hasNode(path)) {
			eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY, 
					"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY) ;
		}
    // create detail node if not exists
		path = new StringBuffer("default/").
			append(CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY).toString() ;    	
		if (!publicCalendar.hasNode(path)) {
			eventShareCategoryNode = defaultCalendar.addNode(CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY, 
					"exo:calendarEventShareCategory") ;
    	eventShareCategoryNode.setProperty("exo:sharetype", CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY) ;
		}
	
		session.save() ;
	}

  // Used when new user deleted
  public void removeUserCalendar(String user) {
  	Session session = this.getSystemSession() ;
  	if (session == null) return ;
    try {
      Node usersHome = (Node) session.getItem(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) ;
      usersHome.getNode(user).remove() ;
      usersHome.save() ;
    } catch (PathNotFoundException ex) {
      log_.info("Can not delete calendar home dir of user " + user) ;
    } catch (RepositoryException e) {
      log_.error("RepositoryException while trying to delete a user calendar home dir", e) ;
    } 
  }

  public void createCalendar(String user, String calendarType, String calendarName) {
  	if (calendarType == null || "".equals(calendarType)) return ;
  	if (calendarName == null || "".equals(calendarName)) return ;
  	
  	if (user == null || "".equals(user)) {
  		user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
      if (user == null || "".equals(user)) return  ;
  	}
  	try {
  		this.createUserCalendarIfNeeded(user) ;
  	} catch (Exception e) {
  		log_.error("Exception while trying to create calendar nodes for user", e) ;
  		return ;
  	}
  	Node node ;
    try {
    	Session session = this.getSession() ;
      node = (Node) session.getItem(
      		new StringBuffer(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
          .append("/").append(user)
          .append("/calendar")
          .append("/").append(calendarType)
          .toString() ) ;
      Node calendarNode= node.addNode(calendarName, "exo:calendar") ;
      if (calendarType.equals(CalendarService.PUBLIC_CALENDAR_TYPE))
      	calendarNode.setProperty("exo:shared", true) ;
      else
      	calendarNode.setProperty("exo:shared", false) ;
     	calendarNode.addNode(CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY, 
      			"exo:calendarEventShareCategory") ;
      calendarNode.addNode(CalendarEventShareCategoryType.FREEBUSY_SHARE_CATEGORY, 
      			"exo:calendarEventShareCategory") ;
      calendarNode.addNode(CalendarEventShareCategoryType.DETAIL_SHARE_CATEGORY, 
      			"exo:calendarEventShareCategory") ;
      session.save() ;
    } catch (Exception e) {
  		log_.error("Exception while trying to create calendar", e) ;
  	}
  }
  
  public void renameCalendar(String user, String oldCalendarType, String oldCalendarName, 
  																					String newCalendarType, String newCalendarName) {
  	if (oldCalendarName == null || "".equals(oldCalendarName)) return ;
  	if (oldCalendarType == null || "".equals(oldCalendarType)) return ;
  	if (newCalendarName == null || "".equals(newCalendarType)) return ;
  	if (newCalendarType == null || "".equals(newCalendarType)) return ;
  	
  	if (user == null || "".equals(user)) {
  		user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
      if (user == null || "".equals(user)) return ;
  	}
  	
  	String oldStorePath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
        .append("/").append(user)
        .append("/calendar")
        .append("/").append(oldCalendarType)
        .append("/").append(oldCalendarName).toString() ;
  	
  	String newStorePath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
        .append("/").append(user)
        .append("/calendar")
        .append("/").append(newCalendarType)
        .append("/").append(newCalendarName).toString() ;
  	this.cmsService_.moveNode(oldStorePath, this.workspace_, this.workspace_, newStorePath) ;
  }
  
	public void removeCalendar(String user, String calendarType, String calendarName) {
		this.removeCalendar(this.getCalendar(user, calendarType, calendarName)) ;
	}
	
	public void removeCalendar(Node calendarNode) {
  	try {
  		if (CalendarService.DEFAULT_CALENDAR.equals(calendarNode.getName())) return ;
  		calendarNode.remove() ;
  		this.getSession().save() ;
  	} catch (Exception e) {
  		log_.error("Exception while trying to remove calendar", e) ;
  	}
	}

	public String saveCalendarEvent(String user, Map rawInputs) throws Exception {
  	return this.saveCalendarEvent(CalendarService.DEFAULT_CALENDAR_TYPE, CalendarService.DEFAULT_CALENDAR, user, rawInputs) ;
  }
  
  public String saveCalendarEvent(String user, String calendarType, String calendarName, Map rawInputs)  {
  	boolean newCalendarEvent ;
  	
  	if (calendarType == null || "".equals(calendarType)) {
  		calendarType = CalendarService.DEFAULT_CALENDAR_TYPE ;
  	}
  	if (calendarName == null || "".equals(calendarName)) {
  		calendarName = CalendarService.DEFAULT_CALENDAR ;
  	}
  	
  	if (user == null || "".equals(user)) {
  		user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
      if (user == null || "".equals(user)) return null ;
  	}

  	try {
  		this.createUserCalendarIfNeeded(user) ;
  	} catch (Exception e) {
  		log_.error("Exception while trying to create calendar nodes for user", e) ;
  		return null;
  	}
	
    JcrInputProperty property  =  (JcrInputProperty) rawInputs.get("/node") ;
    //TODO : set an unique id (out of portail) in order to export events as ical events
    if ("".equals(property.getValue())) {
      property.setValue(this.idService_.generateStringID("calendarEvent"))  ; 
      rawInputs.put("/node", property) ;
      newCalendarEvent = true ;
    }
    else newCalendarEvent = false ;
    property  =  (JcrInputProperty) rawInputs.get("/node/exo:sharetype") ;
    String shareType = (String) property.getValue() ;
    if (shareType == null || "".equals(shareType)) 
    	shareType = CalendarEventShareCategoryType.PRIVATE_SHARE_CATEGORY ;
    property  =  (JcrInputProperty) rawInputs.get("/node/exo:category") ;
    String categoryPath = new StringBuffer(cmsConfigService_.getJcrPath(BasePath.CALENDAR_CATEGORIES_PATH))
    													.append("/").append((String) property.getValue()).toString() ;
    String storePath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
        .append("/").append(user)
        .append("/calendar")
        .append("/").append(calendarType)
        .append("/").append(calendarName)
        .append("/").append(shareType).toString() ;
    String nodePath = null ;
  	try {
  		if (newCalendarEvent) {
  			// set category if new event as category is mandatory
  			property.setValue(new Value[] {getSystemSession().getValueFactory().createValue(categoryPath)}) ;
  			rawInputs.put("/node/exo:category", property) ;
  		}
  		nodePath = this.cmsService_.storeNode(this.workspace_, "exo:calendarEvent", storePath, rawInputs) ;
			if (!newCalendarEvent && nodePath != null && !"".equals(nodePath)) {
				Node node = (Node) this.getSystemSession().getItem(nodePath) ;
				categoriesService_.addCategory(node, categoryPath, true) ;
			}
  	} catch (Exception e) {
  		log_.error("Exception while trying to save calendar event node", e) ;
  		return null;
  	}
    return nodePath ;
  }
  
  public Node removeCalendarEvent(String id) throws Exception {
  	return this.removeCalendarEvent("", id) ;
  }

	public Node removeCalendarEvent(String user, String id) throws Exception {
		Node eventNode = this.getCalendarEventByUserAndId(user, id) ;
		if (eventNode == null) return null ;
		return this.removeCalendarEvent(eventNode) ;
	}
	
	public Node removeCalendarEvent(Node eventNode) throws Exception {
		eventNode.remove() ;
		Session session = this.getSession() ;
		session.save() ;
		return eventNode ;
	}

	public Collection getCalendars(String user, String type) {
	  Session session = this.getSession() ;
    
    if (type == null || "".equals(type) || user == null || "".equals(user)) return null;
    
    StringBuffer strQuery = new StringBuffer("SELECT * FROM exo:calendar WHERE ").append("jcr:path LIKE ")
			.append("'").append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
			.append("/").append(user)			// user
			.append("/calendar")
			.append("/").append(type)			// calendareShareType 
			.append("/%").append("'") ;		// calendarName
    
    Query query = null ;
    QueryResult queryResult = null ;
    try {
      query = session.getWorkspace().getQueryManager().createQuery(strQuery.toString(), Query.SQL)  ;
  	  queryResult = query.execute() ;
    } catch (Exception e) {
  		log_.error("Exception while trying to retrieve calandars (query)", e) ;
  		return null ;
    }
    
	  List calendars = new Vector() ;
	  NodeIterator nodeIterator = null ;
    try {
  	  nodeIterator  = queryResult.getNodes() ;
    } catch (Exception e) {
  		log_.error("Exception while trying to retrieve calandars (retrieve nodes from query)", e) ;
  		return null ;
    }
	  while (nodeIterator.hasNext()) {
	    calendars.add(nodeIterator.nextNode()) ;
	  }
	  
	  return calendars ;
	}
	
	public Collection getPrivateCalendars(String user) {
	  return getCalendars(user, CalendarService.PRIVATE_CALENDAR_TYPE) ;
	}
	
	public Collection getPublicCalendars(String user) {
	  return getCalendars(user, CalendarService.PUBLIC_CALENDAR_TYPE) ;
	}

	public Node getCalendar(String user, String calendarType, String calendarName) {
		if (user == null || "".equals(user)) return null ;
		if (calendarType == null || "".equals(calendarType)) return null ;
		if (calendarName == null || "".equals(calendarName)) return null ;
		
  	String calendarPath = new StringBuffer(
    		this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH))
        .append("/").append(user)
        .append("/calendar")
        .append("/").append(calendarType)
        .append("/").append(calendarName).toString() ;
  	Node node = null ;
  	try {
    	node = (Node) this.getSession().getItem(calendarPath) ; 
  	} catch (RepositoryException re) {
  		log_.error("Exception while trying to retrieve a calendar", re) ;
  	}
  	return node ;
	}
	
  public Node getCalendarEventByPath(String nodePath) throws Exception {
    if (nodePath == null || "".equals(nodePath)) return null ;
    
    Session session = this.getSession() ;
    Node root = session.getRootNode() ;
        
    String relNodePath ;
    if(nodePath.startsWith("/")) relNodePath = nodePath.substring(1) ;
    else relNodePath = nodePath ;
    if(root.hasNode(relNodePath)){
      return (Node) session.getItem(nodePath) ;
    }
        
    return null ;
  }
  
  public Node getCalendarEventById(String id) throws Exception {
  	return this.getCalendarEventByUserAndId("", id) ;
  }

  public Node getCalendarEventByUserAndId(String user, String id) throws Exception {
    Session session = this.getSession() ;
    StringBuffer strQuery ;
    
    if (user == null || "".equals(user)) {
    	strQuery = new StringBuffer("SELECT * FROM exo:calendarEvent WHERE ").append("jcr:path LIKE ")
			.append("'").append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
			.append("/*")			// user
			.append("/calendar")
			.append("/*")			// calendareShareType 
			.append("/*")			// calendarName
			.append("/").append(id).append("'") ;
    }
    else {
    	strQuery = new StringBuffer("SELECT * FROM exo:calendarEvent WHERE ").append("jcr:path LIKE ")
				.append("'").append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
				.append("/").append(user)
				.append("/calendar")
				.append("/*")			// calendareShareType 
				.append("/*")			// calendarName
				.append("/").append(id).append("'") ;
		}
    
    
//    String XPath ;
//    if (user == null || "".equals(user)) {
//    	XPath = new StringBuffer("/jcr:root")
//  		.append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
//  		.append("/*/calendar/*/*/*/").append(id).toString() ;
//    }
//    else {
//    	strQuery = new StringBuffer("SELECT * FROM exo:calendarEvent WHERE ").append("jcr:path LIKE ")
//			.append("'").append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
//			.append("/").append(username)
//			.append("/calendar")
//			.append("/").append(calendareShareType) 
//			.append("/").append(calendarName)
//			.append("/%'")    }
//
//	  Query query = session.getWorkspace().getQueryManager().createQuery(XPath, Query.XPATH)  ;
    Query query = session.getWorkspace().getQueryManager().createQuery(strQuery.toString(), Query.SQL)  ;
	  QueryResult queryResult = query.execute() ;
	  NodeIterator iter = queryResult.getNodes() ;
	  if (iter.hasNext()) return iter.nextNode() ;
	  return null ;
  }
  
  public Collection getCalendarEventByUserAndDate(String user, String calendareShareType, String calendarName,
  				Calendar date) throws Exception {
  	
    GregorianCalendar startDate = new GregorianCalendar(date.get(Calendar.YEAR), date.get(Calendar.MONTH),
        date.get(Calendar.DATE), 0, 0, 0)  ;
    GregorianCalendar endDate = new GregorianCalendar(date.get(Calendar.YEAR), date.get(Calendar.MONTH),
        date.get(Calendar.DATE), 23, 59, 59)  ;
    List <Node>eventsFoundList = new ArrayList<Node>()  ;
      
    Session session = this.getSession() ;
    
    StringBuffer strQuery = new StringBuffer("SELECT * FROM exo:calendarEvent WHERE ").append("jcr:path LIKE ")
    					.append("'").append(this.cmsConfigService_.getJcrPath(BasePath.CMS_USERS_PATH)) 
							.append("/").append(user)
							.append("/calendar")
							.append("/").append(calendareShareType) 
							.append("/").append(calendarName)
							.append("/%'")
							.append(" AND exo:dtstart >= TIMESTAMP '").append(ISO8601.format(startDate)).append("'")
							.append(" AND exo:dtend <= TIMESTAMP '").append(ISO8601.format(endDate)).append("'") ;
    Query query = session.getWorkspace().getQueryManager().createQuery(strQuery.toString(), Query.SQL) ;
    QueryResult queryResult = query.execute() ;
    if (queryResult == null) return null ;
    NodeIterator iter = queryResult.getNodes()  ;
    while (iter.hasNext()) {
    	Node node = iter.nextNode() ;
    	eventsFoundList.add(node) ;
    }

    if (eventsFoundList.size() > 0) {
        CalendarEventNodeComparator comp = new CalendarEventNodeComparator() ;
        Collections.sort(eventsFoundList, comp) ;
    	return eventsFoundList ;
    }
    else return null ;
  }

  public class CalendarEventNodeComparator implements Comparator {
    public int compare(Object o1, Object o2) throws ClassCastException {
      Node calendarEventNode1 = (Node) o1 ;
      Node calendarEventNode2 = (Node) o2 ;
      GregorianCalendar dtstart1, dtstart2 ;
      try {
        dtstart1 =  (GregorianCalendar) ISO8601.parse(calendarEventNode1.getProperty("exo:dtstart")
              .getValue().getString()) ;
        dtstart2 = (GregorianCalendar) ISO8601.parse(calendarEventNode2.getProperty("exo:dtstart")
              .getValue().getString()) ;
        if (dtstart1.before(dtstart2)) return -1 ;
        else if (dtstart1.after(dtstart2)) return 1 ;
        else return 0 ;
      } catch (Exception e) {
        e.printStackTrace() ;
        throw new ClassCastException() ;
      }
    }
  }
}
