/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved   *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.hibernate;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.forum.ForumEventListener;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.ForumServiceContainer;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.services.indexing.IndexingService;
import org.hibernate.Query;
import org.hibernate.Session;
/**
 * Created by The eXo Platform SARL        .
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Date: Jun 14, 2003
 * Time: 1:12:22 PM
 */
public class ForumServiceContainerImpl implements ForumServiceContainer {
  public static String queryForumOwners =
    "select distinct c.owner from org.exoplatform.services.forum.hibernate.CategoryImpl as c" ;

  private HibernateService hservice_ ;
  private IDGeneratorService idService_ ;
  private ForumIndexerPluginImpl indexer_ ;
  private ExoCache forumServices_ ;
  private List<ForumEventListener>  eventlisteners_ ;
  private IndexingService iservice_ ;
  
  public ForumServiceContainerImpl(HibernateService hservice,
                                   IDGeneratorService idService,
                                   CacheService cService,
                                   IndexingService iservice) throws Exception {
    hservice_ = hservice ;
    idService_ = idService ;    
    eventlisteners_  = new ArrayList<ForumEventListener>() ;
    forumServices_ = cService.getCacheInstance(getClass().getName()) ;
    iservice_ = iservice ;
    indexer_ = (ForumIndexerPluginImpl)iservice_.getIndexerPlugin(ForumIndexerPluginImpl.IDENTIFIER) ;
  }

  public ForumService findForumService(String owner) throws Exception {
    ForumService service = (ForumService) forumServices_.get(owner) ;
    if(service == null) {
      synchronized(forumServices_) {
        service = createForumService(owner) ;
        forumServices_.put(owner, service) ;
      }
    }
    return service  ;
  }

  public ForumService createForumService(String owner) throws Exception {
    ForumServiceImpl service = 
      new ForumServiceImpl(hservice_,iservice_,idService_, eventlisteners_, owner) ;
    return service ;
  }
  
  public List getForumOwners() throws Exception {
    Session session = hservice_.openSession();
    Query q = session.createQuery(queryForumOwners);
    return q.list() ;
  }
  
  public void addForumEventListener(ComponentPlugin listener)  {
    eventlisteners_.add((ForumEventListener)listener) ;
  }
}