/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.test;

import java.io.File;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.ForumServiceContainer;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.Watcher;

/*
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestHibernateForumServiceNoCache.java,v 1.6 2004/07/29 14:09:47 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestForumImportExport extends ForumServiceTestCase {
  
  private  ImportExportService bservice_ ;
  private ForumService communityForum_ ;  
  
  public TestForumImportExport(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    PortalContainer manager  = PortalContainer.getInstance();    
      bservice_ = 
        (ImportExportService) manager.getComponentInstanceOfType(ImportExportService.class) ;
      container_ = 
        (ForumServiceContainer) manager.getComponentInstanceOfType(ForumServiceContainer.class) ;
      service_ = container_.findForumService("exo") ;      
  }
  public void testForumImportExport() throws Exception {            
    Date date = new Date() ;
    Category cat = createCategory("category", "this is a test") ;
    assertEquals("check category name: ", "category", cat.getCategoryName()) ;    
    
    Forum forum = createForum(cat, "forum", "this is a test") ;
    assertEquals("check forum name: ", "forum", forum.getForumName()) ;
    
    Watcher forumWatcher = service_.createWatcher(forum) ;
    forumWatcher.setUserName("guest") ;
    forumWatcher.setMessageProtocol("smtp") ;
    forumWatcher.setAddress("exo.platform@laposte.net") ;
    service_.saveWatcher(forumWatcher) ;
    forumWatcher = service_.getWatcher(forum, "guest") ;
    assertTrue("expect forum watcher is found", forumWatcher != null) ;
    
    Topic topic = createTopic(forum, "topic", "this is a test") ;
    assertEquals("check topic : ", "topic", topic.getTopic()) ;
    
    Watcher topicWatcher = service_.createWatcher(topic) ;
    topicWatcher.setUserName("guest") ;
    topicWatcher.setMessageProtocol("smtp") ;
    topicWatcher.setAddress("exo.platform@laposte.net") ;
    service_.saveWatcher(topicWatcher) ;
    topicWatcher = service_.getWatcher(topic, "guest") ;
    assertTrue("expect topic watcher is found", topicWatcher != null) ;
        
    
    Post post = createPost(topic, "post", "this is a test","exo",date) ;
    assertEquals("check post subject : ", "post", post.getSubject()) ;    
    forum = service_.getForum(forum.getId()) ;// with no cache forum has been updated in db
    topic = service_.getTopic(topic.getId()) ;// with no cache topic has been updated in db
    assertEquals("check forum topic count: ", 1 , forum.getTopicCount()) ;
    assertEquals("check forum post count: ", 1 , forum.getPostCount()) ;
    assertEquals("check topic post  count: ", 1 , topic.getPostCount()) ;
    
    
    Post post2 = createPost(topic, "post2", "this is a test","exo",date) ;
    assertEquals("check post subject : ", "post2", post2.getSubject()) ;
    assertEquals("check post message : ", "this is a test", post.getMessage()) ;
    
    topic = service_.getTopic(topic.getId()) ;// with no cache topic has been updated in db
    forum = service_.getForum(forum.getId()) ;// with no cache forum has been updated in db
    assertEquals("check topic post  count: ", 2 , topic.getPostCount()) ;
    assertEquals("check forum topic count: ", 1 , forum.getTopicCount()) ;
    assertEquals("check forum post count: ", 2 , forum.getPostCount()) ;
    
    /*---------create an forum with owner is 'community' with data------*/
    communityForum_ = container_.findForumService("community") ;        
    Category ccat = communityForum_.createCategoryInstance();            
    ccat.setCategoryName("community category") ;
    ccat.setDescription("this is community catefory forum") ;
    communityForum_.addCategory(ccat) ;
    
    Forum cforum = communityForum_.createForumInstance() ;
    cforum.setForumName("community forum") ;
    cforum.setDescription("this is a forum of community portal") ;
    communityForum_.addForum(ccat,cforum) ;
    
    Topic ctopic = communityForum_.createTopicInstance() ;
    ctopic.setTopic("community topic") ;
    ctopic.setOwner("community") ;
    ctopic.setDescription("this is community topic") ;
    communityForum_.addTopic(cforum,ctopic) ;
    
    Post cpost = communityForum_.createPostInstance() ;
    cpost.setSubject("community post") ;
    cpost.setOwner("community") ;
    cpost.setMessage("this is the community post") ;
    communityForum_.addPost(ctopic,cpost) ;
    
    System.out.println("==============Export forum service data============") ;
    bservice_.exportServiceData() ;
    /*expect to found entrys:'exo/{name}.category','exo/{name}.forum','exo/{name}.topic'
     *and: 'community/{name}.category','community/{name}.forum', 'community/{name}.topic'*/
    String[] forumPortals = {"exo","community"} ;
    String[] entryNames = {".category",".forum",".topic"} ;    
    ZipFile zfile = getZipServiceData("forum") ;
    assertTrue("expect found a zipfile with name is 'forum'",zfile!=null) ;
    assertTrue("expect found all entrys name with 'exo'",hasEntry(zfile,entryNames,forumPortals)) ;    
    /*--------clean database before import-------*/    
    service_.removeCategory(cat.getId()) ;
    communityForum_.removeCategory(ccat.getId()) ;
    assertTrue("expect all forum data of 'exo' is removed",service_.getCategories().size()==0) ;
    assertTrue("expect all forum fata of 'community' is removed",
                communityForum_.getCategories().size()==0) ;
    System.out.println("==============Import forum service data============") ;
    bservice_.importServiceData() ;
    /*-----check  all data of 'exo' portal */
    assertEquals("expect found 1 category of 'exo' forum",1,service_.getCategories().size()) ;
    Category catAfter = (Category)service_.getCategories().get(0) ;
    assertObject(cat,catAfter,new CategoryComparator()) ;
    
    assertEquals("expect found 1 forum in this category",1,service_.getForums(catAfter.getId()).size()) ;
    Forum forumAfter = (Forum)service_.getForums(catAfter.getId()).get(0) ;
    assertObject(forum,forumAfter,new ForumComparator()) ;
    Watcher forumWatcherAfter = service_.getWatcher(forumAfter,"guest") ;    
    assertTrue("expect found watche this forum",forumWatcherAfter!=null) ;
    assertObject(forumWatcher,forumWatcherAfter, new WatcherComparator()) ;
    
    assertEquals("expect found 1 topic in this forum",
                  1,service_.getTopics(forumAfter.getId()).getAll().size()) ;
    Topic topicAfter = (Topic)service_.getTopics(forumAfter.getId()).getAll().get(0) ;    
    assertObject(topic,topicAfter,new TopicComparator()) ;
    Watcher topicWatcherAfter = service_.getWatcher(topicAfter,"guest") ;
    assertTrue("expect found watche this forum",topicWatcherAfter!=null) ;
    assertObject(topicWatcher,topicWatcherAfter, new WatcherComparator()) ;

    List<Post> postAfters = service_.getPosts(topicAfter.getId()).getAll() ; 
    assertEquals("expect 2 post is found in this topic",2,postAfters.size())  ;
    hasObjectInCollection(post,postAfters,new PostComparator()) ;
    hasObjectInCollection(post2,postAfters,new PostComparator()) ;
    /*-----check  all forum data of 'community' portal */
    assertEquals("expect found 1 category of 'exo' forum",1,communityForum_.getCategories().size()) ;
    catAfter = (Category)communityForum_.getCategories().get(0) ;
    assertObject(ccat,catAfter,new CategoryComparator()) ;
    
    assertEquals("expect found 1 forum in this category",1,service_.getForums(catAfter.getId()).size()) ;
    forumAfter = (Forum)communityForum_.getForums(catAfter.getId()).get(0) ;
    assertObject(cforum,forumAfter,new ForumComparator()) ;
    
    assertEquals("expect found 1 topic in this forum",
        1,service_.getTopics(forumAfter.getId()).getAll().size()) ;
    topicAfter = (Topic)communityForum_.getTopics(forumAfter.getId()).getAll().get(0) ;    
    assertObject(ctopic,topicAfter,new TopicComparator()) ;
    
    postAfters = communityForum_.getPosts(topicAfter.getId()).getAll() ; 
    assertEquals("expect 1 post is found in this topic",1,postAfters.size())  ;
    assertObject(cpost,postAfters.get(0),new PostComparator()) ;    
    /*--------clean database-------*/       
    service_.removeCategory(cat.getId()) ;
    service_.removeCategory(ccat.getId()) ;
  }
  
  private ZipFile getZipServiceData(String serviceName) throws Exception{
    String zipFileName = bservice_.getServiceDataDirectory() + serviceName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
  
  private boolean hasEntry(ZipFile zfile, String[] entryNames,String [] owners) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;      
      for(int i = 0;i<owners.length;i++) {        
        for(int j=0;j<entryNames.length;j++) {
          if((entry.getName().startsWith(owners[i]))&&
             (entry.getName().endsWith(entryNames[j]))&&
             (entry.getSize()>0)) {return true ; }
        }        
      }            
     }    
    return false ;
  } 
  
  protected String getDescription() {
    return "Test Forum Import-Export " ;
  }
}
