/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.message.impl;

import java.util.Iterator;
import java.util.List;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.database.XResources;
import org.exoplatform.services.message.Account;
import org.exoplatform.services.message.MessageService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserEventListener;
import org.hibernate.Session;
/**
 * Jul 20, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UserEventAccountListener.java,v 1.1 2004/10/30 02:24:13 tuan08 Exp $
 */
public class UserEventAccountListener extends UserEventListener {
  private MessageServiceImpl mservice_ ;
  private List accounts_ ;
  
	public UserEventAccountListener(InitParams params,
                                  MessageServiceImpl mservice) throws Exception {
    accounts_ =  params.getObjectParamValues(AccountListenerConfig.class);
    mservice_ = mservice ;
  }
  
	public void postSave(User user, boolean isNew, XResources xresources) throws Exception {
	  Session session = (Session) xresources.getResource(Session.class) ;
    Iterator i = accounts_.iterator() ;
    while(i.hasNext()) {
      AccountListenerConfig config = (AccountListenerConfig) i.next() ;
      if(MessageService.STANDALONE_PROTOCOL.equals(config.getProtocol())) {
        Account account = createStandaloneAccount(user, config) ;
        mservice_.createAccount(session, account) ;
      } else {
        Account account = createMailAccount(user, config) ;
        mservice_.createAccount(session, account) ;
      }
    }
	}
  
   public void postDelete(User user,  XResources xresources) throws Exception {
     List accounts = mservice_.getAccounts(user.getUserName()) ;
     for(int i = 0; i < accounts.size(); i++) {
       Account account = (Account) accounts.get(i) ;
       mservice_.removeAccount(account) ;
     }
   }
  
	private Account createMailAccount(User user, AccountListenerConfig config){
	  Account account = mservice_.createAccountInstance() ;
	  account.setAccountName(config.getAccountName());
	  account.setOwner(user.getUserName()) ;
    account.setOwnerName(user.getFirstName() + " " + user.getLastName()) ;
    account.setReplyToAddress(user.getUserName() +"@" +config.getAccountName()) ;
	  account.setProtocol(config.getProtocol());
	  account.setProperty(Account.SERVER_SETTING_HOSTNAME, config.getServer());
	  account.setProperty(Account.SERVER_SETTING_USERNAME, user.getUserName());
	  account.setProperty(Account.SERVER_SETTING_PASSWORD, user.getPassword());
	  return account;
	}
  
  private Account createStandaloneAccount(User user, AccountListenerConfig config){
    Account account = mservice_.createAccountInstance() ;
    account.setAccountName(config.getAccountName());
    account.setOwner(user.getUserName()) ;
    account.setOwnerName(user.getFirstName() + " " + user.getLastName()) ;
    account.setReplyToAddress(user.getUserName() +"#" + config.getAccountName()) ;
    account.setProtocol(config.getProtocol());
    return account;
  }
}
