/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.spider.htmlparser.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.exoplatform.services.spider.core.text.HTMLCharset;

/**
 * Created by The eXo Platform SARL
 * Author : Thuannd
 *         nhudinhthuan@yahoo.com
 * Jan 3, 2006
 */
class DocumentParser {
  
  private Pattern sPattern ;   
  
  DocumentParser(){
    sPattern = Pattern.compile("\\p{Space}+", Pattern.CASE_INSENSITIVE);
  }
  
  List<Token> getToken(byte[] data, String charset) throws Exception {    
    if( charset == null || charset.trim().equals(""))
      charset = (new HTMLCharset()).getCharset(data);    
    return this.getToken( new String( data, charset));   
  }
  
  List<Token> getToken( String html){     
    Pattern pattern = Pattern.compile( "[<>]", Pattern.CASE_INSENSITIVE);
    Matcher matcher = pattern.matcher(html);
    
    List<Token> list = new ArrayList<Token>();
    
    if( !matcher.find()) return list;
    
    int end = matcher.start(); 
    int start = 0;    
    
    while( end <= html.length()){       
      if( html.charAt( end) == '<'){
        createToken( html.substring( start, end).trim(), Token.CONTENT, list);        
        start = end;
      }else if( html.charAt( end) == '>'){          
        createToken( html.substring( start+1, end).trim(), Token.TAG, list);
        start = end+1;
      }
      if( start+1>= html.length()) break;
      if( matcher.find( start+1)) 
        end = matcher.start();
      else
        break;     
    }   
    if( end < html.length()){      
      createToken( html.substring( end+1, html.length()).trim(), Token.CONTENT, list);      
    }
    return list;
  }
  
  private void createToken( String value, int type, List<Token> list){   
    if( value == null ||  value.equals("")) return ;          
    Token token = new Token( value, type);
    if( type == Token.TAG ){
      Matcher matcher = sPattern.matcher( value);
      value = matcher.replaceAll(" ");  
      if( value.startsWith("/")) token.setType( Token.CLOSE);     
    }
    list.add( token);    
  } 
}