/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.spider.htmlparser.impl;

import java.util.List;

import org.exoplatform.services.spider.core.htmlparser.Node;

/**
 * Created by The eXo Platform SARL
 * Author : Thuannd
 *         nhudinhthuan@yahoo.com
 * Jan 3, 2006
 */
class NodeParser {
  
  private Node nodeDocument; 
  
  private int idx = 0;  
  
  private String [][] nodeOptionClose = { {"tr","table",null},
      {"li","ul","ol"}, {"dt","dl",null},{"option","select",null}, {"colgroup", "table",null},
      {"td", "tr", null} ,{"dd", "dt", null}, {"th", "tr", null}};
  
  NodeParser ( List<Token> tokens) {    
    nodeDocument = new NodeImpl( new Token("document", Token.TAG));  
    idx = 0;
    Node current = nodeDocument; 
    Token temp;
    
    while( idx < tokens.size()){
      temp = tokens.get( idx);      
      //content     
      if( temp.getType() == Token.CLOSE){       
        current = find(  temp.getValue().substring( 1, temp.getValue().length()).trim(), current); 
      } else {
        NodeImpl node = new NodeImpl( temp);
        if( node.getType() == -1 || node.getType() == 0){
          node.setParent( current);   
        } else         
          current = checkFormatToken( node, current);
      }       
      idx++; 
    }    
  }  
  
  private Node find( String name, Node n){   
    NodeImpl node = (NodeImpl)n;   
    while( node.getParent() != null){      
      if( !node.isClose() && (node.getName().equals( name))){        
        closeAll( node);        
        return node.getParent(); 
      }
      node = (NodeImpl) node.getParent();  
    }   
    return n;
  }
  
  public Node checkFormatToken( Node node, Node current){
    if( current == null || current.getParent() == null){
      node.setParent( current);
      return node;
    }
    Node temp ;
    for( int i=0; i<  nodeOptionClose.length; i++){
      temp = complete(
          nodeOptionClose[i][0], nodeOptionClose[i][1], nodeOptionClose[i][2], node, current);
      if( temp != null) return temp;
    }
    node.setParent( current);
    return node;
  }  
  
  private Node complete(String n, String p, String o, Node node, Node current){    
    if( !node.getName().equals( n)) return null;      
    Node pre = findNearest( n, (NodeImpl)current);
    Node parent = findNearest( p, (NodeImpl)current);
    if( parent == null && o != null) parent = findNearest( o, (NodeImpl)current);
    if( pre == null && parent != null) return null; 
    if( pre == null && parent == null){
      parent = createNode( p, current);
      ((NodeImpl)parent).setClose( true);
    }
    node.setParent( parent);
    return node;    
  }
  
  private NodeImpl findNearest( String name, NodeImpl node ){
    NodeImpl parent =  node;    
    while( true){ 
      if( parent == null) break;      
      if( !parent.isClose() && (parent.getName().equals( name))) return parent;        
      parent = (NodeImpl) parent.getParent();    
    }   
    return null;
  }
  
  private void closeAll(NodeImpl node){    
    if( node.isClose()) return;
    for( int i = 0;  i<node.getChildrenSize(); i++) {
      ((NodeImpl)node.getChild(i)).setClose( true);
    }
    node.setClose( true);
  }
  
  public Node getNode(){
    return this.nodeDocument;
  }  
  
  private NodeImpl createNode( String v, Node parent){    
    return new NodeImpl( new Token( v, Token.TAG), parent);
  }
  
}
