/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.access;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: AccessControlList.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class AccessControlList {

  private String owner;

  private Map permissions;

  public AccessControlList(boolean initDefault) {
    this.owner = SystemIdentity.SYSTEM;
    this.permissions = new HashMap();
    if(initDefault)
      permissions.put(SystemIdentity.ANY, PermissionType.ALL);
  }

  public AccessControlList(String owner, Map permissions) {
    this.owner = owner;
    this.permissions = permissions;
  }

  public AccessControlList(PropertyData ownerProp, PropertyData permissionsProp)
      throws RepositoryException {

    if(ownerProp != null) // if property already initialized in storage
      this.owner = new String(((ValueData) ownerProp.getValues().get(0))
        .getAsByteArray());
    
    if (permissionsProp != null) {
      permissions = new HashMap();
      List permValues = permissionsProp.getValues();

      for (int i = 0; i < permValues.size(); i++) {
        String p = new String(((ValueData) permValues.get(i)).getAsByteArray());
        // TODO
        StringTokenizer parser = new StringTokenizer(p);
        addPermission(parser.nextToken(), parser.nextToken());
        //////
      }
    }
  }

  private void addPermission(String identity, String perm) {
    if (permissions.containsKey(identity)) {
      String[] oldPerm = (String[]) permissions.get(identity);
      String[] newPerm = new String[oldPerm.length + 1];
      for (int i = 0; i < oldPerm.length; i++)
        newPerm[i] = oldPerm[i];
      newPerm[newPerm.length - 1] = perm;
      permissions.put(identity, newPerm);
    } else {
      permissions.put(identity, new String[] { perm });
    }
  }

  /**
   * @return Returns the owner.
   */
  public String getOwner() {
    return owner;
  }

  /**
   * Returns map key=(String)identity; value=(String[])permissions
   * 
   * @return Returns the permissions.
   */
  public Map getPermissions() {
    return permissions;
  }

  
  public String dump() {
    String res = "OWNER: "+owner+"\n";
    Iterator ids = permissions.keySet().iterator();
    while(ids.hasNext()) {
      String id = (String)ids.next();
      String[] perm = (String[])permissions.get(id);
      String permStr = "";
      for(int i=0; i<perm.length; i++)
        permStr+=perm[i]+" ";
      res+=" "+id +" --->" +permStr+"\n";
    }
    return res;
  }
}