/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl;

import java.net.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.io.File;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.configuration.ConfigurationManagerImpl;
import org.exoplatform.container.jmx.MX4JComponentAdapterFactory;
import org.exoplatform.container.util.ContainerUtil;
import org.picocontainer.ComponentAdapter;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: StandaloneContainer.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class StandaloneContainer extends ExoContainer {

  private static StandaloneContainer container;

  private static URL configurationURL = null;

  private ConfigurationManagerImpl configurationManager;

  private StandaloneContainer() {
    super(new MX4JComponentAdapterFactory(), null);
    configurationManager = new ConfigurationManagerImpl(null);
    this.registerComponentInstance(ConfigurationManager.class,
        configurationManager);
  }

  public static StandaloneContainer getInstance() throws Exception {
    if (container == null) {
      container = new StandaloneContainer();
      List configs = new ArrayList();
      if(configurationURL == null) {
        configs.add(Thread.currentThread().getContextClassLoader().getResource("conf/standalone/configuration.xml"));
      }
      else {
        configs.add(configurationURL);
      }

      container.populate(configs);
      Iterator c = container.getComponentAdapters().iterator();
      while (c.hasNext()) {
        System.out.println("STANDALONE component >>>> "
            + ((ComponentAdapter) c.next()).getComponentImplementation());
      }

      //container.initContainer();
      container.start();
    }
    return container;
  }

  public static void setConfigurationURL(String url) throws MalformedURLException {
    if((url == null) || (url.length() == 0))
      return;
    configurationURL = new URL(url);
  }
  
  public static void setConfigurationPath(String url) throws MalformedURLException {
    if((url == null) || (url.length() == 0))
      return;
    configurationURL = (new File(url)).getAbsoluteFile().toURL();
  }
  
  public MBeanServer getMBeanServer() {
    return MBeanServerFactory.createMBeanServer("exojcrmx");
  }

  private void populate(Collection conf) throws Exception {
    configurationManager.addConfiguration(conf);
    configurationManager.processRemoveConfiguration();
    ContainerUtil.addComponents(this, configurationManager);
  }

  private void populate(URL conf) throws Exception {
    configurationManager.addConfiguration(conf);
    configurationManager.processRemoveConfiguration();
    ContainerUtil.addComponents(this, configurationManager);
  }

  private void populateGroovy(URL conf) throws Exception {
    configurationManager.addConfiguration(conf);
    configurationManager.processRemoveConfiguration();
  //ContainerUtil.populateGroovy(this, configurationManager);
  }

  public ConfigurationManagerImpl getConfigurationManager() {
    return configurationManager;
  }
}