/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: ChangedItemsStorage.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class ChangedItemsStorage {

  // uuid -> ItemImpl
  private List itemChanges;
  
  // path -> ChangeHistory
  //private Map changeHistories;
  
  protected ExoLog log;

  public ChangedItemsStorage(LogService logService) {
    this.itemChanges = new ArrayList();
    this.log = (ExoLog)logService.getLog(this.getClass());
    log.setLevel(LogService.DEBUG);
    //this.changeHistories = new HashMap(); 
  }
  
  /**
   * Adds item to session storage
   * @param item
   */
  synchronized void put(ItemImpl item) {
    log.debug("put "+item.getPath()+" "+item.getInternalUUID()+" "+item.getState());
    itemChanges.add(item);
  }
  
  /**
   * Removes item from session storage
   * @param item
   */
  synchronized void remove(ItemImpl item) {
    itemChanges.remove(item);
    //changeHistories.remove(item.getPath());
  }
  

  /**
   * @param uuid
   * @return item by uuid or null
   */
  synchronized ItemImpl get(String uuid) {
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl item = (ItemImpl)it.next();
      if(item.getInternalUUID().equals(uuid))
        return item;
    }
    return null; 
  }
  
  /**
   * @param jcrPath
   * @return current (Note: there can be multiple items with the same path) 
   * item by path or null if there are no one in session   
   * @throws RepositoryException
   */
  synchronized ItemImpl get(JCRPath jcrPath) {
    ItemImpl item = null;
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl curItem = (ItemImpl)it.next();
      if (curItem.getLocation().equals(jcrPath))
        item = curItem;
    }
    return item;
  }

  synchronized List getChanges(JCRPath jcrPath, boolean childrenOnly) throws RepositoryException {

    List items = new ArrayList();
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl curItem = (ItemImpl)it.next();
      JCRPath testLocation = curItem.getLocation();
      if ((testLocation.equals(jcrPath) || testLocation.isDescendantOf(jcrPath, childrenOnly))) {
        items.add(curItem);
      }
    }
    return items;
  }
  
  synchronized List getChanges() {
    return itemChanges;
  }
  
}
