/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.io.InputStream;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.Item;
import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.ItemVisitor;
import javax.jcr.MergeException;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.Lock;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.nodetype.ItemDefinition;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.Version;
import javax.jcr.version.VersionException;
import javax.jcr.version.VersionHistory;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.access.SystemIdentity;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.itemfilters.ItemFilter;
import org.exoplatform.services.jcr.impl.core.itemfilters.NamePatternFilter;
import org.exoplatform.services.jcr.impl.core.itemvisitors.NodeDumpVisitor;
import org.exoplatform.services.jcr.impl.core.lock.LockImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.ItemDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitions;
import org.exoplatform.services.jcr.impl.core.value.PermissionValue;
import org.exoplatform.services.jcr.impl.core.version.MergeVisitor;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionImpl;
import org.exoplatform.services.jcr.impl.storage.TransientItemData;
import org.exoplatform.services.jcr.impl.storage.TransientNodeData;
import org.exoplatform.services.jcr.impl.storage.TransientPropertyData;
import org.exoplatform.services.jcr.impl.util.EntityCollection;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL.
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: NodeImpl.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class NodeImpl extends ItemImpl implements ExtendedNode {

  protected LocationFactory sysLocFactory;

  protected UUIDGenerator uuidGenerator;

  // protected LogService logService;

  // //////
  private List childNodes;

  private List childProperties;

  private JCRName primaryType;

  private JCRName[] mixinTypes;

  private AccessControlList childrenACL;

  private int orderNumber;

  private NodeDefinition definition;

  // //////

  public NodeImpl(NodeData data, NodeImpl parentNode, int state,
      SessionImpl session, LogService logService) throws PathNotFoundException,
      RepositoryException, ConstraintViolationException {

    super(data, parentNode, state, session, logService);
    // nodeType names should conform NamespaceRegistry's mapping
    sysLocFactory = session.getSystemLocationFactory();
    uuidGenerator = session.getUuidGenerator();
    // this.logService = logService;

    loadData(data);

  }

  void loadData(ItemData data) throws RepositoryException,
      InvalidItemStateException, ConstraintViolationException {

    if (data == null)
      throw new InvalidItemStateException(
          "Data is null for "
              + this.getPath()
              + " Probably was deleted by another session and can not be loaded from container ");

    if (!(data instanceof NodeData))
      throw new RepositoryException(
          "Load data: NodeData is expected. But have " + data);

    NodeData nodeData = (NodeData) data;
    this.primaryType = sysLocFactory.createJCRName(nodeData
        .getPrimaryTypeName());
    this.mixinTypes = new JCRName[nodeData.getMixinTypeNames().length];
    for (int i = 0; i < mixinTypes.length; i++)
      mixinTypes[i] = sysLocFactory
          .createJCRName(nodeData.getMixinTypeNames()[i]);
    this.orderNumber = nodeData.getOrderNumber();
    this.childrenACL = initACL(nodeData);
    this.childNodes = null;
    this.childProperties = null;
    
    //this.data = data;

    initDefinition();

  }

  private void initDefinition() throws RepositoryException,
      ConstraintViolationException {

    if (this.isRoot()) { // root - no parent
      this.definition = new NodeDefinitionImpl();
      return;
    }
    NodeImpl parent = (NodeImpl) getParent();
    NodeType[] nodetypes = parent.getAllNodeTypes();
    NodeDefinition residualDef = null;
    for (int i = 0; i < nodetypes.length; i++) {

      NodeDefinition def = ((NodeTypeImpl) nodetypes[i])
          .getChildNodeDefinition(getName());

      if (def != null) {
        if (def.getName() == ItemDefinitionImpl.RESIDUAL_SET)
          residualDef = def;
        else {
          log.debug("Node definition found " + def.getName());
          this.definition = def;
          return;
          // return def;
        }
      }
    }

    if (residualDef != null)
      this.definition = residualDef;
    else
      throw new ConstraintViolationException(
          "NodeImpl.getDefinition failed. Definition not found for "
              + getPath());

  }

  private AccessControlList initACL(NodeData nodeData) {
    AccessControlList dataACL = nodeData.getChildrenACL();
    if (dataACL == null) { // default one
      return new AccessControlList(true);
    } else {
      return dataACL;
    }
  }

  public TransientItemData getActualItemData() throws RepositoryException {

    return traverseData(this);
  }

  private synchronized TransientNodeData traverseData(NodeImpl node)
      throws RepositoryException {
    InternalQName[] mixinNames = new InternalQName[node.getMixinTypes().length];
    for (int i = 0; i < node.getMixinTypes().length; i++)
      mixinNames[i] = node.getMixinTypes()[i].getInternalName();

    TransientNodeData currentData = new TransientNodeData(node.getLocation()
        .getInternalPath(), node.storedNodeData().getUUID(), node
        .getPersistedVersion(), node.getPrimaryType().getInternalName(),
        mixinNames, node.getOrderNumber(), node.storedNodeData()
            .getParentUUID());

    if (node.isChildPropertiesInitialized()) {
      PropertyIterator props = node.getProperties();
      while (props.hasNext()) {
        PropertyImpl prop = (PropertyImpl) props.next();
        currentData.addProperty((TransientPropertyData) prop
            .getActualItemData());
      }
    }

    if (node.isChildNodesInitialized()) {
      NodeIterator nodes = node.getNodes();
      while (nodes.hasNext()) {
        NodeImpl childNode = (NodeImpl) nodes.next();
        // System.out.println("child "+childNode.getPath());
        if (childNode.getPath().equals(node.getPath()))
          throw new RepositoryException(
              "FATAL: internal error child node path equals parent "
                  + node.getPath());
        currentData.addNode(traverseData(childNode));
      }
    }

    return currentData;

  }

  public NodeData storedNodeData() {
    return (NodeData) data;
  }

  // /// ------------ Reading

  /**
   * Finds parent. It can be this node or this node's child.
   * 
   * @param itemPath
   * @return this node or find parent by path
   * @throws PathNotFoundException
   *           if not found
   */
  private NodeImpl findParent(JCRPath itemPath) throws RepositoryException,
      PathNotFoundException {

    JCRPath parentPath = itemPath.makeParentPath();
    if (getLocation().equals(parentPath))
      return this;
    else {
      ItemImpl parentNode = dataManager.getItem(parentPath);
      if (parentNode == null || !parentNode.isNode())
        throw new PathNotFoundException("Parent not found for "
            + itemPath.getAsString(false));
      else
        return (NodeImpl) parentNode;
    }
  }

  /**
   * @see javax.jcr.Node#getNode
   */
  public Node getNode(String relPath) throws PathNotFoundException,
      RepositoryException {
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);
    NodeImpl parent = findParent(itemPath);
    // log.debug("GET NODE "+parent.getPath()+" "+parent);

    for (NodeIterator nodes = parent.getNodes(itemPath.getName().getAsString()); nodes
        .hasNext();) {
      Node nextNode = nodes.nextNode();
      if (nextNode.getIndex() == itemPath.getIndex())
        return nextNode;
    }
    throw new PathNotFoundException("Node not found "
        + itemPath.getAsString(true));
  }

  /**
   * @see javax.jcr.Node#getNodes
   */
  public NodeIterator getNodes() throws RepositoryException {

    loadChildNodes();
    return new EntityCollection(childNodes);

  }

  public List loadChildNodes() throws RepositoryException,
      AccessDeniedException {
    // return if initialized
    if (childNodes == null) {

      List storedNodes = dataManager.getChildren(this, true);
      /*
       * childNodes = new CopyOnWriteArrayList();
       *  // add node according to orderNumber NodeImpl[] childArray = new
       * NodeImpl[storedNodes.size()]; for (int i = 0; i < storedNodes.size();
       * i++) { NodeImpl node = (NodeImpl) storedNodes.get(i);
       * childArray[node.getOrderNumber()] = node; System.out.println(">>>
       * getNodes >>" +this.getPath()+" "+childArray[node.getOrderNumber()]); }
       * for (int i = 0; i < childArray.length; i++) {
       * childNodes.add(childArray[i]); System.out.println(">>> getNodes >>"
       * +this.getPath()+" "+childArray[i]);
       *  }
       */
      Collections.sort(storedNodes, new NodesOrderComparator());
      childNodes = new CopyOnWriteArrayList(storedNodes);
    }

    return childNodes;

  }

  private static class NodesOrderComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      NodeImpl n1 = (NodeImpl) o1;
      NodeImpl n2 = (NodeImpl) o2;
      return n1.getOrderNumber() - n2.getOrderNumber();
    }
  }

  boolean areNodesInitialized() {
    return childNodes != null;
  }

  /**
   * @see javax.jcr.Node#getNodes
   */
  public NodeIterator getNodes(String namePattern) throws RepositoryException {
    ItemFilter filter = new NamePatternFilter(namePattern);
    ArrayList list = new ArrayList();
    NodeIterator children = getNodes();
    // System.out.println(">>> getNodes >>");
    while (children.hasNext()) {

      NodeImpl item = (NodeImpl) children.nextNode();
      // System.out.println(">>> getNodes >>" +this.getPath()+" "+item);

      if (filter.accept(item))
        list.add(item);
    }
    return new EntityCollection(list);
  }

  /**
   * @see javax.jcr.Node#getProperty
   */
  public Property getProperty(String relPath) throws PathNotFoundException,
      RepositoryException {
    // log.debug("getProp1 >>"+relPath+" parent "+parent);

    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);
    
    //  Search in node props
    PropertyIterator properties = null;
    /*
    properties = parent.getProperties(itemPath.getName()
        .getAsString());
    if (properties.hasNext()) {
      PropertyImpl prop = (PropertyImpl) properties.nextProperty();
      System.err.println("=!= prop: " + prop + ", prop.isNode(): " + prop.isNode());      
      return prop;
    }
    */
    
    // Search in parent props

    NodeImpl parent = findParent(itemPath);

    // log.debug("getProp2 >>"+relPath+" parent "+parent);
    
    properties = parent.getProperties(itemPath.getName()
        .getAsString());

    if (!properties.hasNext()) {
      throw new PathNotFoundException("Property not found "
          + itemPath.getAsString(false));
    } else {
      PropertyImpl prop = (PropertyImpl) properties.nextProperty();
      // log.debug("getProp2 >>"+relPath+" "+prop.getState());
      return prop;
    }
  }

  /**
   * @see javax.jcr.Node#getProperties
   */
  public PropertyIterator getProperties() throws RepositoryException {
    loadChildProperties();
    return new EntityCollection(childProperties);
  }

  public List loadChildProperties() throws RepositoryException,
      AccessDeniedException {

    // not yet initialized
    if (this.childProperties == null) {

      childProperties = new CopyOnWriteArrayList();

      List storedProps = dataManager.getChildren(this, false);
      // 0 - primaryType
      // 1 - mixinTypes
      // 2 - uuid
      // ....
      for (int i = 0; i < storedProps.size(); i++) {
        PropertyImpl prop = (PropertyImpl) storedProps.get(i);
        InternalQName qname = prop.getLocation().getName().getInternalName();
        if (qname.equals(Constants.JCR_PRIMARYTYPE) && i > 0) {
          childProperties.add(0, prop);
        } else if (qname.equals(Constants.JCR_MIXINTYPES) && i > 1) {
          childProperties.add(1, prop);
        } else if (qname.equals(Constants.JCR_UUID) && i > 2) {
          childProperties.add(2, prop);
        } else {
          childProperties.add(prop);
        }
      }
    }
    return childProperties;
  }

  boolean arePropertiesInitialized() {
    return childProperties != null;
  }

  /**
   * @see javax.jcr.Node#getProperties
   */
  public PropertyIterator getProperties(String namePattern)
      throws RepositoryException {
    ItemFilter filter = new NamePatternFilter(namePattern);
    ArrayList list = new ArrayList();
    PropertyIterator children = getProperties();
    while (children.hasNext()) {
      PropertyImpl item = (PropertyImpl) children.nextProperty();
      if (filter.accept(item))
        list.add(item);
    }

    return new EntityCollection(list);
  }

  /**
   * @see javax.jcr.Node#getIndex
   */
  public int getIndex() throws RepositoryException {
    return getLocation().getIndex();
  }

  /**
   * @see javax.jcr.Node#getPrimaryItem
   */
  public Item getPrimaryItem() throws ItemNotFoundException,
      RepositoryException {

    NodeType[] types = this.getAllNodeTypes();
    for (int i = 0; i < types.length; i++) {
      String itemName = types[i].getPrimaryItemName();
      if (itemName != null) {
        Item primaryItem = dataManager.getItem(locationFactory.createJCRPath(
            getLocation(), itemName));
        if (primaryItem != null)
          return primaryItem;
      }
    }

    throw new ItemNotFoundException("Primary item not found for " + getPath());
  }

  /**
   * @see javax.jcr.Node#getUUID
   */
  public String getUUID() throws UnsupportedRepositoryOperationException,
      RepositoryException {
    if (isNodeType("mix:referenceable")) {
      return this.internalUUID;
    } else {
      throw new UnsupportedRepositoryOperationException("Node " + getPath()
          + " is not referenceable");
    }

  }

  /**
   * Returns saved only references (allowed by specs)
   * 
   * @see javax.jcr.Node#getReferences
   */
  public PropertyIterator getReferences() throws RepositoryException {
    List propDatas = dataManager.getReferences(getUUID());
    /*
     * System.out.println("REF OF >>>> "+getPath()); byte[] bytes =
     * getUUID().getBytes(); for(int i=0;i<bytes.length;i++) {
     * System.out.print(bytes[i]); } for (int i = 0; i < propDatas.size(); i++) {
     * System.out.println("REF>>>>
     * "+((PropertyData)propDatas.get(i)).getQPath().getAsString()); }
     */
    EntityCollection refs = new EntityCollection();
    for (int i = 0; i < propDatas.size(); i++) {
      PropertyData data = (PropertyData) propDatas.get(i);
      refs.add(new PropertyImpl((PropertyData) propDatas.get(i), null,
          ItemChangeState.UNCHANGED, session, logService));
    }
    return refs;
  }

  /**
   * @see javax.jcr.Node#hasNode
   */
  public boolean hasNode(String relPath) throws RepositoryException {
    try {
      getNode(relPath);
    } catch (PathNotFoundException e) {
      return false;
    }
    return true;
  }

  /**
   * @see javax.jcr.Node#hasNodes
   */
  public boolean hasNodes() throws RepositoryException {
    return getNodes().hasNext();
  }

  /**
   * @see javax.jcr.Node#hasProperty
   */
  public boolean hasProperty(String relPath) throws RepositoryException {
    try {
      getProperty(relPath);
      return true;
    } catch (RepositoryException e) {
      return false;
    }
  }

  /**
   * @see javax.jcr.Node#hasProperties
   */
  public boolean hasProperties() throws RepositoryException {
    return getProperties().hasNext();
  }

  /**
   * @see javax.jcr.Item#isModified
   */
  public boolean isModified() {
    return (getState() == ItemChangeState.UPDATED)
        || (!isNew() && session.getTransientNodesManager()
            .hasAddedOrRemovedDescendant(getLocation(), true));
  }

  // // -------------- Writting

  /**
   * @see javax.jcr.Node#addNode
   */
  public Node addNode(String relPath) throws ItemExistsException,
      PathNotFoundException, ConstraintViolationException, RepositoryException,
      VersionException, LockException {

    // Parent can be not the same as this node
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);

    // Check if there no final index
    if (itemPath.getIndex() > 1)
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + itemPath.getAsString(false));

    NodeImpl parent = this.findParent(itemPath);
    // Check if parent exists
    if (parent == null)
      throw new PathNotFoundException("Parent not found for "
          + itemPath.getAsString(true));

    // find node type
    NodeType[] nodeTypes = parent.getAllNodeTypes();
    JCRName nodeTypeName = findNodeType(nodeTypes, itemPath.getName()
        .getAsString());

    // try to make new node
    return doAddNode(parent, itemPath.getName(), nodeTypeName, null, false,
        true);

  }

  private JCRName findNodeType(NodeType[] nodeTypes, String name)
      throws RepositoryException, ConstraintViolationException {
    String residualNodeTypeName = null;
    for (int j = 0; j < nodeTypes.length; j++) {
      NodeDefinition[] nodeDefs = nodeTypes[j].getChildNodeDefinitions();
      for (int i = 0; i < nodeDefs.length; i++) {
        NodeDefinition nodeDef = nodeDefs[i];
        if (nodeDef.getName().equals(name)) {
          return sysLocFactory.parseJCRName(nodeDef.getDefaultPrimaryType()
              .getName());
        } else if (nodeDef.getName().equals(NodeDefinitionImpl.RESIDUAL_SET)) {
          residualNodeTypeName = nodeDef.getDefaultPrimaryType().getName();
        }
      }
    }
    if (residualNodeTypeName == null)
      throw new ConstraintViolationException("Can not define node type for "
          + name);
    return sysLocFactory.parseJCRName(residualNodeTypeName);
  }

  /**
   * @see javax.jcr.Node#addNode
   */
  public Node addNode(String relPath, String nodeTypeName)
      throws ItemExistsException, PathNotFoundException,
      NoSuchNodeTypeException, ConstraintViolationException,
      RepositoryException, VersionException, LockException {

    // Parent can be not the same as this node
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);

    // Check if there no final index
    if (itemPath.getIndex() > 1)
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + itemPath.getAsString(false));

    NodeImpl parent = this.findParent(itemPath);

    // Check if parent exists
    if (parent == null)
      throw new PathNotFoundException("Parent not found for "
          + itemPath.getAsString(true));

    return doAddNode(parent, itemPath.getName(), sysLocFactory
        .parseJCRName(nodeTypeName), null, false, true);
  }

  /**
   * Creates child node without adding to session's transient storage
   * 
   * @param name
   * @param primaryTypeName
   * @param addAutoCreated
   *          if add auto-created items (false value is useful for node
   *          import/copy/move)
   * @return new child node
   * @throws ItemExistsException
   * @throws RepositoryException
   * @throws ConstraintViolationException
   */
  public NodeImpl createChildNode(String name, String primaryTypeName,
      boolean addAutoCreated) throws ItemExistsException, RepositoryException,
      ConstraintViolationException {
    return doAddNode(this, locationFactory.parseJCRName(name), sysLocFactory
        .parseJCRName(primaryTypeName), null, true, addAutoCreated);
  }

  public NodeImpl createChildNode(String name, String primaryTypeName,
      String[] mixinTypeNames, String uuid, boolean autoCreated,
      boolean addAutoCreated) throws ItemExistsException, RepositoryException,
      ConstraintViolationException {

    // log.debug(">>> clone mixin "+mixinTypeNames);
    NodeImpl node = doAddNode(this, locationFactory.parseJCRName(name),
        sysLocFactory.parseJCRName(primaryTypeName), uuid, autoCreated,
        addAutoCreated);
    
    for (int i = 0; i < mixinTypeNames.length; i++) {
      //node.doAddMixin(mixinTypeNames[i], autoCreated, addAutoCreated);
      doAddMixin(mixinTypeNames[i], true, !autoCreated, addAutoCreated);
    }    
    return node;
  }

  public NodeImpl createChildNode(String name, String primaryTypeName,
      boolean autoCreated, boolean addAutoCreated) throws ItemExistsException,
      RepositoryException, ConstraintViolationException {
    
    return doAddNode(this, locationFactory.parseJCRName(name), sysLocFactory
        .parseJCRName(primaryTypeName), null, autoCreated, addAutoCreated);
  }

  // with mixin
  public NodeImpl createChildNode(String name, String primaryTypeName,
      String[] mixinTypeNames, boolean addAutoCreated)
      throws ItemExistsException, RepositoryException,
      ConstraintViolationException {
    
    NodeImpl node = doAddNode(this, locationFactory.parseJCRName(name),
        sysLocFactory.parseJCRName(primaryTypeName), null, true, addAutoCreated);
    
    for (int i = 0; i < mixinTypeNames.length; i++) {
      //node.doAddMixin(mixinTypeNames[i], true, false);
      node.doAddMixin(mixinTypeNames[i], false, false, false);
    }
    
    return node;
  }

  // TODO fix it
  public NodeImpl importChildNode(String relPath, String primaryTypeName,
      String[] mixinTypeNames) throws ItemExistsException, RepositoryException,
      ConstraintViolationException {

    // Parent can be not the same as this node
    JCRPath itemPath = locationFactory.createJCRPath(getLocation(), relPath);

    // Check if there no final index
    if (itemPath.getIndex() > 1)
      throw new RepositoryException(
          "The relPath provided must not have an index on its final element. "
              + itemPath.getAsString(false));

    // NodeImpl parent = (NodeImpl)dataManager.getItem(path.makeParentPath(),
    // false);
    NodeImpl parent = this.findParent(itemPath);

    // Check if parent exists
    if (parent == null)
      throw new PathNotFoundException("Parent not found for "
          + itemPath.getAsString(true));

    NodeImpl node = doAddNode(parent, itemPath.getName(), sysLocFactory
        .parseJCRName(primaryTypeName), null, false, false);

    for (int i = 0; i < mixinTypeNames.length; i++)
      doAddMixin(mixinTypeNames[i], false, false, false);
      //doAddMixin(mixinTypeNames[i], true, false);
    return node;
  }

  public NodeImpl createChildNodeInmemory(InternalQName name,
      InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      String uuid, int index, boolean registerInDataManager,
      boolean doExternalValidation, boolean addAutoCreated, 
      boolean cacheInSession)
      throws ItemExistsException, RepositoryException,
      ConstraintViolationException, VersionException, LockException {

    return doAddNode(this, name, primaryTypeName, mixinTypeNames, uuid, index,
        registerInDataManager, doExternalValidation, addAutoCreated, cacheInSession);

  }
  
  public NodeImpl copyChildNodeInmemory(InternalQName qName, 
      NodeData srcNode,
      boolean keepUUID,
      boolean registerInDataManager,
      boolean doExternalValidation, boolean addAutoCreated,
      boolean cacheInSession)
      throws ItemExistsException, RepositoryException,
      ConstraintViolationException, VersionException, LockException {
    
    InternalQName ptQName = srcNode.getPrimaryTypeName();
    InternalQName[] mtQNames = srcNode.getMixinTypeNames();
    int index = srcNode.getQPath().getIndex();
    String uuid = (keepUUID)?srcNode.getUUID():null;
        
    NodeImpl nodeImpl = doAddNode(this, qName, ptQName, mtQNames, uuid, index,
        registerInDataManager, doExternalValidation, addAutoCreated, cacheInSession);
     
    return nodeImpl;

  }


  /**
   * Main method for node adding
   * 
   * @param parentNode
   * @param name
   * @param primaryTypeName
   * @param mixinTypeNames
   *          (null or empty if undefined)
   * @param uuid
   *          (null if undefined)
   * @param index
   *          (-1 if calculate)
   * @param registerInDataManager
   *          if register this node with transient session storage (as a rule:
   *          true for Session's method, false for Workspace's and for auto
   *          created)
   * @param doExternalValidation
   *          if do validation for externally added nodes (as a rule: true for
   *          manually added node (addNode(..)), false for auto created/imported
   *          etc)
   * @param addAutoCreated
   *          if auto-created items will be added as well (as a rule: true for
   *          just created node, false for copied/moved/imported etc)
   * @param cacheInSession
   *          if save this item in session's cache (as a rule: true for
   *          session methods returned node outside, false for workspace methods 
   *          like copy/clone/import etc)
   * 
   * @return new node
   * @throws ItemExistsException
   * @throws RepositoryException
   */
  private NodeImpl doAddNode(NodeImpl parentNode, InternalQName name,
      InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      String uuid, int index, boolean registerInDataManager,
      boolean doExternalValidation, boolean addAutoCreated,
      boolean cacheInSession)
      throws ItemExistsException, RepositoryException,
      ConstraintViolationException, VersionException, LockException {

    // Check if nodeType exists and not mixin
    String ptStr = sysLocFactory.createJCRName(primaryTypeName).getAsString();
    NodeTypeImpl type = (NodeTypeImpl) nodeType(ptStr);
    if (type.isMixin())
      throw new ConstraintViolationException("Add Node failed: Node Type <"
          + ptStr + "> is MIXIN type!");

    // Set mixin types to empty array if null
    if (mixinTypeNames == null)
      mixinTypeNames = new InternalQName[0];

    // Create new NodeData inmemory
    String nameStr = sysLocFactory.createJCRName(name).getAsString();
    JCRPath path = locationFactory.createJCRPath(parentNode.getLocation(),
        nameStr);

    // set index if any -- will be set later else
    if (index > 0)
      path.setIndex(index);

    // Generate UUID if not found
    if (uuid == null)
      uuid = uuidGenerator.generateUUID(path.getAsString(true)).getString();

    // Find Order number
    int orderNum = 0;
    try {
      orderNum = (int) getNodes().getSize();
    } catch (AccessDeniedException e) {
      log.warn("Access denied for read order num == 0 " + getPath());
    }

    // create new nodedata
    NodeData nodeData = new TransientNodeData(path.getInternalPath(), uuid, -1,
        primaryTypeName, mixinTypeNames, orderNum, parentNode.storedNodeData()
            .getUUID());

    // Create new Node
    int state = ItemChangeState.ADDED;
    NodeImpl node = new NodeImpl(nodeData, parentNode, state, session,
        logService);

    // Check if new node's node type is allowed by its parent definition
    NodeType[] types = parentNode.getAllNodeTypes();
    for (int i = 0; i < types.length; i++) {
      NodeTypeImpl t = (NodeTypeImpl) types[i];
      if (t.isChildNodePrimaryTypeAllowed(locationFactory.createJCRName(primaryTypeName).getAsString())) {
        break;
      } else if (i == types.length - 1) {
        throw new ConstraintViolationException("Can't add node "
            + name.getAsString() + " to " + parentNode.getPath()
            + " node type " + primaryTypeName.getAsString()
            + " is not allowed as child's node type for parent node type ");
      }
    }

    // Validation
    if (doExternalValidation) {

      // Check if node is not protected
      if (node.getDefinition().isProtected()) // nodeTypeValidator.isItemProtected(this))
        throw new ConstraintViolationException("Can't add protected node "
            + name.getAsString() + " to " + parentNode.getPath());

      // Check if versionable ancestor is not checked-in
      if (!parentNode.isCheckedOut())
        throw new VersionException("Node " + parentNode.getPath()
            + " or its nearest ancestor is checked-in");

      // Check locking
      if (!parentNode.checkLocking())
        throw new LockException("Node " + parentNode.getPath() + " is locked ");

    }

    // if samename exists check samename siblings and calculate index
    if (index == -1)
      node.getLocation().setIndex(calculateIndex(parentNode.getNodes(), node));


    // add node to parent
    parentNode.addChildNode(node);

    // Register node with dataManager
    if (registerInDataManager)
      dataManager.update(node);

    if (addAutoCreated) {
      // Add autocreated items
      node.addAutoCreatedItems(node.getPrimaryNodeType(), false);
      // Init some values
      // node.initAutoCreatedItems(node.getPrimaryNodeType());
    }
    // return node;

    System.out.println("DO ADD NODE >>>> new node : " + node.getPath() + " name: "
        + " primaryType: " + node.getPrimaryNodeType().getName() + " index: "
        + node.getIndex() + " state: " + node.getState() + " parent: "
        + parentNode);
    
    if(cacheInSession)
      return (NodeImpl) dataManager.cacheItem(node, true);
    else
      return node;
  }

  /**
   * @deprecated
   * 
   */
  private NodeImpl doAddNode(NodeImpl parentNode, JCRName name,
      JCRName primaryTypeName, String uuid, boolean autoCreated,
      boolean addAutoCreated) throws ItemExistsException, RepositoryException,
      ConstraintViolationException, VersionException, LockException {

    // Check if nodeType exists and not mixin
    NodeTypeImpl type = (NodeTypeImpl) nodeType(primaryTypeName.getAsString());
    if (type.isMixin())
      throw new ConstraintViolationException("Add Node failed: Node Type <"
          + primaryTypeName + "> is MIXIN type!");

    // Create new NodeData inmemory
    JCRPath path = locationFactory.createJCRPath(parentNode.getLocation(), name
        .getAsString());
    if (uuid == null)
      uuid = uuidGenerator.generateUUID(path.getAsString(true)).getString();
    int orderNum = 0;
    try {
      orderNum = (int) getNodes().getSize();
    } catch (AccessDeniedException e) {
      log.warn("Access denied for read order num == 0 " + getPath());
    }

    NodeData nodeData = new TransientNodeData(path.getInternalPath(), uuid, -1,
        primaryTypeName.getInternalName(), new InternalQName[0], orderNum,
        parentNode.storedNodeData().getUUID());
    // , new AccessControlList());
    
    
    // Create new Node
    int state = ItemChangeState.ADDED;
    NodeImpl node = new NodeImpl(nodeData, parentNode, state, session,
        logService);
    
    // Check if new node's node type is allowed by its parent definition
    NodeType[] types = parentNode.getAllNodeTypes();
    for (int i = 0; i < types.length; i++) {
      NodeTypeImpl t = (NodeTypeImpl) types[i];
      if (t.isChildNodePrimaryTypeAllowed(primaryTypeName.getAsString())) {
        break;
      } else if (i == types.length - 1) {
        throw new ConstraintViolationException("Can't add node "
            + name.getAsString() + " to " + parentNode.getPath()
            + " node type " + primaryTypeName.getAsString()
            + " is not allowed as child's node type for parent node type ");
      }
    }

    // For manually added
    if (!autoCreated) {

      // Check if node is not protected
      if (node.getDefinition().isProtected()) // nodeTypeValidator.isItemProtected(this))
        throw new ConstraintViolationException("Can't add protected node "
            + name.getAsString() + " to " + parentNode.getPath());

      // Check if versionable ancestor is not checked-in
      if (!parentNode.isCheckedOut())
        throw new VersionException("Node " + parentNode.getPath()
            + " or its nearest ancestor is checked-in");

      // Check locking
      if (!parentNode.checkLocking())
        throw new LockException("Node " + parentNode.getPath() + " is locked ");

    }

    // if samename exists check samename siblings and calculate index
    node.getLocation().setIndex(calculateIndex(parentNode.getNodes(), node));

    log.debug("DO ADD NODE >>>> new node : " + node.getPath() + " name: "
        + " primaryType: " + node.getPrimaryNodeType().getName() + " index: "
        + node.getIndex() + " state: " + node.getState() + " parent: "
        + parentNode);

    // add node to parent    
    parentNode.addChildNode(node);
    
    // ====== Check for TCK tests compability, Peter Nedonosko ======
    // test if node contains all mandatory properties of current type
    /*
    PropertyDefinition propDefs[] = type.getPropertyDefinitions();
    for (int i = 0; i < propDefs.length; i++) {
      PropertyDefinition propDef = propDefs[i];

      if (propDef.isMandatory()) {
        String pdName = propDef.getName();

        System.err.println("======= type: " + type.getName()
            + ", node: " + node.getName() + "(" + node + "), propDef: "
            + pdName);
        if (node.hasProperty(pdName)) {
          System.err.println("=== node.hasProperty: true, " 
              + "node: " + node.getName() + "(" + node + "), propDef: "
              + pdName);
        } else {
          System.err.println("=== node.hasProperty: false, " 
              + "node: " + node.getName() + "(" + node + "), propDef: "
              + pdName);
        }
      }
    }
    */

    // Register node with dataManager
    if (!autoCreated)
      dataManager.update(node);

    if (addAutoCreated) {
      // Add autocreated items
      node.addAutoCreatedItems(node.getPrimaryNodeType(), false);
      // Init some values
      // node.initAutoCreatedItems(node.getPrimaryNodeType());
    }

    // return node;
    return (NodeImpl) dataManager.cacheItem(node, true);

  }

  private int calculateIndex(NodeIterator children, NodeImpl childToAdd)
      throws RepositoryException, ItemExistsException {
    NodeDefinition def = childToAdd.getDefinition();
    int ind = 0;
    while (children.hasNext()) {
      NodeImpl child = (NodeImpl) children.nextNode();
      if (child.getState() != ItemChangeState.DELETED
          && child.getName().equals(childToAdd.getName())) {
        log.debug(">>> Calculate index for " + childToAdd.getPath() + " "
            + child.getPath() + " " + ((NodeImpl) child).getState());

        if (def.allowsSameNameSiblings())
          ind++;
        else
          throw new ItemExistsException("The node " + childToAdd.getName()
              + " already exists in " + getPath()
              + " and same name sibling is not allowed ");

      }
    }

    return ind + 1;

  }

  /**
   * Initializes children if neccessary and Adds child node
   * 
   * @param node
   *          to add
   * @throws RepositoryException
   */
  protected void addChildNode(NodeImpl node) throws RepositoryException {
    loadChildNodes();
    for (int i = 0; i < childNodes.size(); i++) {
      if (((Node) childNodes.get(i)).getPath().equals(node.getPath())) {
        childNodes.remove(i);
      }
    }
    childNodes.add(node);
  }

  void removeChildNode(ItemImpl node) throws RepositoryException {
    loadChildNodes();
    NodeImpl removedNode = null;
    log.debug("child to remove >> " + node.getPath() + " " + childNodes.size()
        + " from " + this);
    for (int i = 0; i < childNodes.size(); i++) {
      NodeImpl nextNode = (NodeImpl) childNodes.get(i);
      log.debug("looking for child to remove >> " + node.getPath() + " "
          + nextNode.getPath() + " " + nextNode.getInternalUUID() + " equals: "
          + node.getPath().equals(nextNode.getPath()));
      // if (nextNode.getPath().equals(node.getPath())) {
      if (nextNode.getInternalUUID().equals(node.getInternalUUID())) {

        childNodes.remove(i);
        removedNode = nextNode;
        log.debug("removed child " + node.getPath() + " "
            + ((NodeImpl) node).getOrderNumber());
      }
    }
    log.debug("after remove >> " + node.getPath() + " " + childNodes.size());

    // recalculate orderNum and index if neccessary

    if (removedNode != null) {
      for (int i = 0; i < childNodes.size(); i++) {
        NodeImpl children = (NodeImpl) childNodes.get(i);
        children.setOrderNumber(i);
        // same name sibs
        if (children.getName().equals(removedNode.getName())
            && children.getIndex() > removedNode.getIndex()) {
          log.debug("sameName child for pack " + children.getPath() + " -> "
              + children.getIndex());

          children.getLocation().setIndex(children.getIndex() - 1);

          // dataManager.changeIndex(children, children.getIndex() - 1);

          // dataManager.delete(children);
          // children.setState(ItemChangeState.ADDED);
          // dataManager.update(children);

          log.debug("sameName child renewed index " + children.getPath()
              + " -> " + children.getIndex());

        }

        children.setState(ItemChangeState.UPDATED);
        dataManager.update(children);
        log.debug("reordered child " + children.getPath() + " "
            + children.getOrderNumber());
      }
    }

  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value[] values, int type)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    return doSetProperty(name, values, type, true, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value[] values)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    int type = PropertyType.UNDEFINED;
    if (values != null && values.length > 0 && values[0] != null)
      type = values[0].getType();

    return doSetProperty(name, values, type, true, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String[] values)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value[] strValues;
    if (values == null)
      strValues = null;
    else {
      List valList = packStrValues(values);
      strValues = new Value[valList.size()];
      for (int i = 0; i < valList.size(); i++) {
        strValues[i] = valueFactory.createValue((String) valList.get(i));
      }
    }
    return doSetProperty(name, strValues, PropertyType.STRING, true, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String[] values, int type)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    Value[] strValues;
    if (values == null)
      strValues = null;
    else {
      List valList = packStrValues(values);
      strValues = new Value[valList.size()];
      for (int i = 0; i < valList.size(); i++) {
        strValues[i] = valueFactory.createValue((String) valList.get(i));
      }
    }

    return doSetProperty(name, strValues, type, true, true, true);
  }

  private List packStrValues(String values[]) {
    ArrayList valList = new ArrayList();
    for (int i = 0; i < values.length; i++) {
      if (values[i] != null)
        valList.add(values[i]);
    }
    return valList;
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {
    int valType;
    Value[] values;
    if (value == null) {
      valType = PropertyType.UNDEFINED;
      values = null;
    } else {
      valType = value.getType();
      values = new Value[] { value };
    }

    return doSetProperty(name, values, valType, false, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Value value, int type)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value[] newValues = null;
    if (value != null)
      newValues = valueFactory.convert(new Value[] { value }, type, false);
    return doSetProperty(name, newValues, type, false, true, true);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String value, int type)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value, type);
    return setProperty(name, val, type);

  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, String value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.STRING);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, InputStream value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.BINARY);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, boolean value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value),
        PropertyType.BOOLEAN);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Calendar value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.DATE);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, double value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value),
        PropertyType.DOUBLE);
  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, long value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    return setProperty(name, valueFactory.createValue(value), PropertyType.LONG);

  }

  /**
   * @see javax.jcr.Node#setProperty
   */
  public Property setProperty(String name, Node value)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    Value val = (value == null) ? null : valueFactory.createValue(value);
    return setProperty(name, val, PropertyType.REFERENCE);
  }

  public PropertyImpl createChildProperty(String name, Value[] values, int type)
      throws ValueFormatException, VersionException, LockException,
      ConstraintViolationException, RepositoryException {

    PropertyDefinitions def = null;
    PropertyDefinition resDef = null;
    NodeType[] allNT = getAllNodeTypes();
    for (int i = 0; i < allNT.length; i++) {
      NodeTypeImpl nodeType = (NodeTypeImpl) allNT[i];
      def = nodeType.getPropertyDefinitions(name);
            
      // log.debug(" DEF check "+nodeType.getName()+" "+name+"
      // "+def.getAnyDefinition().getName()+this.getMixinNodeTypes().length);

      PropertyDefinition anyDef = def.getAnyDefinition();
      
      if (anyDef != null) {
        if (anyDef.getName().equals(name)) {
            break;
        } else {
            resDef = anyDef;
        }        
      }
    }
    
    PropertyDefinition anyDef = def.getAnyDefinition();
        
    if (anyDef != null) {
      return doSetProperty(name, values, type, anyDef.isMultiple(), 
          false, false);
    } else if (resDef != null) {  
      return doSetProperty(name, values, type, resDef.isMultiple(), 
          false, false);
    } else {
      throw new ConstraintViolationException("Property " + name
          + " is not defined for " + getPath());
    }
  }

  // registerInDataManager,
  // doExternalValidation
  protected PropertyImpl doSetProperty(String name, Value[] values, int type,
      boolean multiValued, boolean registerInDataManager,
      boolean doExternalValidation) throws ValueFormatException,
      VersionException, LockException, ConstraintViolationException,
      RepositoryException {

    PropertyImpl property;
    int state;
    
    try {
      property = (PropertyImpl) getProperty(name);
      state = ItemChangeState.UPDATED;

    } catch (PathNotFoundException e) {
      // create new property
      JCRPath path = locationFactory.createJCRPath(getLocation(), name);
      String uuid = uuidGenerator.generateUUID(path.getAsString(true))
          .getString();
      PropertyData data = new TransientPropertyData(path.getInternalPath(),
          uuid, -1, type, internalUUID);
      state = ItemChangeState.ADDED;
      // state = autoCreated?ItemChangeState.AUTO_ADDED:ItemChangeState.ADDED;
      property = new PropertyImpl(data, this, state, session, logService);

    }

    // System.out.println("SET VALUES >>>>>>> "+property.getType());

    // set new values
    property.doSetValue(values, multiValued, type, state,
        registerInDataManager, doExternalValidation);

    //System.out.println("SET VALUES >>>>>>> "+property.getPath());

    if (property.getState() != ItemChangeState.DELETED)
      addChildProperty(property);

    //System.out.println("DO SeT property: "+name+" "+this.getPath()+" prop "+property.getPath()+" "+property.getState());

    
    return (PropertyImpl) dataManager.cacheItem(property, true);

    // return property;

  }

  void addChildProperty(PropertyImpl property) throws RepositoryException {

    loadChildProperties();
    for (int i = 0; i < childProperties.size(); i++) {
      if (((Property) childProperties.get(i)).getPath().equals(property.getPath())) {
        childProperties.remove(i);
      }
    }
    childProperties.add(property);
  }

  void removeChildProperty(ItemImpl property) throws RepositoryException {
    loadChildProperties();
    for (int i = 0; i < childProperties.size(); i++) {
      PropertyImpl nextProp = (PropertyImpl) childProperties.get(i);
      // if (((ItemImpl) childProperties.get(i)).getPath().equals(
      // property.getPath())) {
      if (nextProp.getInternalUUID().equals(property.getInternalUUID())) {
        childProperties.remove(i);
        log.debug("removed child property " + property + " from " + this
            + " size: " + childProperties.size());
      }
    }
  }

  /**
   * @see javax.jcr.Node#getPrimaryNodeType
   */
  public NodeType getPrimaryNodeType() throws RepositoryException {

    return nodeType(primaryType.getAsString());
  }

  /**
   * @see javax.jcr.Node#getMixinNodeTypes
   */
  public NodeType[] getMixinNodeTypes() throws RepositoryException {

    // should not be null
    if (mixinTypes == null)
      throw new RepositoryException(
          "Data Container implementation error getMixinTypeNames == null");
    NodeType[] mixinNodeTypes = new NodeType[mixinTypes.length];
    for (int i = 0; i < mixinNodeTypes.length; i++) {
      mixinNodeTypes[i] = nodeType(mixinTypes[i].getAsString());
    }

    return mixinNodeTypes;
  }

  /**
   * @see javax.jcr.Node#isNodeType
   */
  public boolean isNodeType(String nodeTypeName) throws RepositoryException {
    NodeType testNodeType;
    try {
      testNodeType = nodeType(nodeTypeName);
    } catch (NoSuchNodeTypeException e) {
      log.error("Node.isNodeType() No such nodetype: " + nodeTypeName);
      return false;
    }

    NodeType[] nodetypes = getAllNodeTypes();
    for (int i = 0; i < nodetypes.length; i++) {
      if (NodeTypeImpl.isSameOrSubType(testNodeType, nodetypes[i]))
        return true;
    }
    return false;
  }

  /**
   * @see javax.jcr.Node#getDefinition
   */
  public NodeDefinition getDefinition() throws RepositoryException {

    return definition;

  }

  /**
   * @see javax.jcr.Node#getCorrespondingNodePath
   */
  public String getCorrespondingNodePath(String workspaceName)
      throws ItemNotFoundException, NoSuchWorkspaceException,
      AccessDeniedException, RepositoryException {
    SessionImpl correspSession = (SessionImpl) ((RepositoryImpl) session
        .getRepository()).login(session.getCredentials(), workspaceName);

    return getCorrespondingNode(correspSession).getPath();
  }

  public NodeImpl getCorrespondingNode(SessionImpl correspSession)
      throws ItemNotFoundException, NoSuchWorkspaceException,
      AccessDeniedException, RepositoryException {
    if (this.isNodeType("mix:referenceable")) {
      try {
        return (NodeImpl) correspSession.getNodeByUUID(getUUID());
      } catch (ItemNotFoundException e) {
      }
    } else {
      Node refAncestor = null;
      List qpathEntries = new ArrayList();
      NodeImpl curNode = this;
      for (int i = getDepth(); i >= 0; i--) {
        NodeImpl ancestor = (NodeImpl) getAncestor(i);
        if (ancestor.isNodeType("mix:referenceable")) {
          NodeImpl correspAncestor = (NodeImpl) correspSession
              .getNodeByUUID(ancestor.getUUID());
          JCRPath.PathElement[] relJCRPath = getLocation()
              .getRelPath(getDepth() - i);
          // System.out.println("LOC. getRelPath "+(getDepth() - i)+" =
          // "+getRelPath(relJCRPath)+" "+getPath());
          try {
            return (NodeImpl) correspAncestor.getNode(getRelPath(relJCRPath));
          } catch (ItemNotFoundException e) {
          }
        }
      }
    }
    try {
      return (NodeImpl) correspSession.getItem(getPath());
    } catch (PathNotFoundException e) {
      throw new PathNotFoundException("No corresponding path for " + getPath()
          + " in " + correspSession.getWorkspace().getName());
    }
  }

  private String getRelPath(JCRPath.PathElement[] relPath)
      throws RepositoryException {
    String path = "";
    for (int i = 0; i < relPath.length; i++) {
      path += relPath[i].getAsString(false);
      if (i < relPath.length - 1)
        path += "/";
    }
    return path;
  }

  /**
   * @see javax.jcr.Node#update
   */
  public void update(String srcWorkspaceName) throws NoSuchWorkspaceException,
      AccessDeniedException, RepositoryException {

    String srcPath;
    try {
      srcPath = getCorrespondingNodePath(srcWorkspaceName);
      session.getTransientNodesManager().getWorkspaceDataManager().delete(this);
    } catch (ItemNotFoundException e) {
      log.debug("No corresponding node in workspace: " + srcWorkspaceName);
      return;
    }

    session.getWorkspace().clone(srcWorkspaceName, srcPath, this.getPath(),
        true);

  }

  /**
   * @see javax.jcr.Node#addMixin
   */
  public void addMixin(String mixinName) throws NoSuchNodeTypeException,
      ConstraintViolationException, VersionException, LockException,
      RepositoryException {

    //doAddMixin(mixinName, false, true);
    doAddMixin(mixinName, true, true, true);
  }
//  public void doAddMixin(String mixinName, boolean autoCreated,
//      boolean addAutoCreated) throws NoSuchNodeTypeException,
//      ConstraintViolationException, VersionException, LockException,
//      RepositoryException {
  public void doAddMixin(String mixinName, boolean registerInDataManager,
      boolean doExternalValidation, boolean addAutoCreated
      ) throws NoSuchNodeTypeException,
      ConstraintViolationException, VersionException, LockException,
      RepositoryException {


    // Add both to mixinNodeTypes and to jcr:mixinTypes property
    NodeType type = nodeType(mixinName);
    log.debug("Node.addMixin " + mixinName + " " + getPath());

    // Mixin or not
    if (type == null || !type.isMixin())
      throw new NoSuchNodeTypeException("Node Type " + mixinName
          + " not found or not mixin type.");

    if (doExternalValidation) {

      // Validate
      if (!canAddMixin(mixinName))
        throw new ConstraintViolationException("Can not add mixin type "
            + mixinName);

      // Check if versionable ancestor is not checked-in
      if (!isCheckedOut())
        throw new VersionException("Node " + getPath()
            + " or its nearest ancestor is checked-in");

      // Check locking
      if (!checkLocking())
        throw new LockException("Node " + getPath() + " is locked ");

    }

    // Prepare mixin values
    JCRName[] newMixin = new JCRName[mixinTypes.length + 1];
    Value[] values = new Value[mixinTypes.length + 1];

    for (int i = 0; i < mixinTypes.length; i++) {
      newMixin[i] = mixinTypes[i];
      // JCRName mixinJCRName = locationFactory.createJCRName(newMixin[i]);
      values[i] = valueFactory.createValue(mixinTypes[i]);
    }
    newMixin[mixinTypes.length] = locationFactory.parseJCRName(mixinName);
    values[mixinTypes.length] = valueFactory
        .createValue(newMixin[mixinTypes.length]);

    // Set mixin property and nodeData
    boolean firstMixin = (mixinTypes.length == 0);
    mixinTypes = newMixin;
    // Should register jcr:mixinTypes and autocreated items if node is not added
    //boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    PropertyImpl prop = doSetProperty("jcr:mixinTypes", values,
        PropertyType.NAME, true, registerInDataManager, false);

    //System.out.println("Node.addMixin Property " + prop.getPath() + " values "
    //    + mixinTypes.length + " " + registerInDataManager);

    
    log.debug("Node.addMixin Property " + prop.getPath() + " values "
        + mixinTypes.length + " " + addAutoCreated);
    
    // Add autocreated items for this nodeType
    if (addAutoCreated)
      addAutoCreatedItems(nodeType(mixinName), registerInDataManager);

  }

  /**
   * @see javax.jcr.Node#removeMixin
   */
  public void removeMixin(String mixinName) throws NoSuchNodeTypeException,
      ConstraintViolationException, RepositoryException {

    // find mixin
    int ind = -1;
    for (int i = 0; i < mixinTypes.length; i++) {
      if (mixinTypes[i].getAsString().equals(mixinName))
        ind = i;
    }
    // no mixin found
    if (ind < 0)
      throw new NoSuchNodeTypeException("No mixin type found " + mixinName
          + " for node " + getPath());

    // A ConstraintViolationException will be thrown either
    // immediately or on save if the removal of a mixin is not
    // allowed. Implementations are free to enforce any policy
    // they like with regard to mixin removal and may differ on
    // when this validation is done.

    // Check if versionable ancestor is not checked-in
    if (!isCheckedOut())
      throw new VersionException("Node " + getPath()
          + " or its nearest ancestor is checked-in");

    // Check locking
    if (!checkLocking())
      throw new LockException("Node " + getPath() + " is locked ");

    // Prepare mixin values
    JCRName[] newMixin = new JCRName[mixinTypes.length - 1];
    Value[] values = new Value[mixinTypes.length - 1];
    for (int i = 0; i < newMixin.length; i++) {
      if (i != ind) {
        newMixin[i] = mixinTypes[i];
        // JCRName mixinJCRName = locationFactory.createJCRName(newMixin[i]);
        values[i] = valueFactory.createValue(mixinTypes[i]);
      }
    }

    // Set mixin property and locally
    mixinTypes = newMixin;
    boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    PropertyImpl prop = doSetProperty("jcr:mixinTypes", values,
        PropertyType.NAME, true, registerInDataManager, false);

  }

  /**
   * @see javax.jcr.Node#canAddMixin
   */
  public boolean canAddMixin(String mixinName) throws RepositoryException {

    if (hasSameOrSubtypeMixin((NodeTypeImpl) nodeType(mixinName))) {
      return false;
    }

    if (getDefinition().isProtected()) {
      return false;
    }

    if (!isCheckedOut())
      return false;

    if (!checkLocking())
      return false;

    return true;
  }

  private boolean hasSameOrSubtypeMixin(NodeTypeImpl type)
      throws RepositoryException, ValueFormatException {

    for (int i = 0; i < mixinTypes.length; i++) {

      if (NodeTypeImpl.isSameOrSubType(type, nodeType(mixinTypes[i]
          .getAsString()))) {
        return true;
      }
    }
    return false;

  }

  /**
   * @see javax.jcr.Node#orderBefore
   */
  public void orderBefore(String srcName, String destName)
      throws UnsupportedRepositoryOperationException,
      ConstraintViolationException, ItemNotFoundException, RepositoryException {

    if (srcName.equals(destName))
      return;

    EntityCollection nodes = (EntityCollection) getNodes();
    NodeImpl srcNode = null;
    NodeImpl destNode = null;
    int size = (int) nodes.getSize();
    if (size < 2)
      throw new UnsupportedRepositoryOperationException(
          "Nothing to order Count of child nodes " + size);

    // index number of each element equlal of NodeImpl.orderNumber
    List list = nodes.getList();

    if (destName == null)
      destNode = (NodeImpl) list.get(size - 1);

    // find dest and src nodes
    while (nodes.hasNext()) {
      NodeImpl node = (NodeImpl) nodes.nextNode();
      if (node.getName().equals(srcName))
        srcNode = node;
      if (destName != null && node.getName().equals(destName))
        destNode = node;
    }

    if (srcNode == null)
      throw new ItemNotFoundException("Src Node not found " + srcName);
    if (destNode == null)
      throw new ItemNotFoundException("Dest Node not found " + destName);

    // reorder nodes
    list.add(destNode.getOrderNumber(), srcNode);
    list.remove(srcNode.getOrderNumber());
    for (int i = 0; i < list.size(); i++) {
      NodeImpl node = (NodeImpl) list.get(i);
      node.setOrderNumber(i);
      node.setState(ItemChangeState.UPDATED);
      dataManager.update(node);
    }

  }

  int getOrderNumber() {
    return orderNumber;
  }

  void setOrderNumber(int orderNumber) {
    this.orderNumber = orderNumber;
  }

  // //////////////////////// OPTIONAL

  // VERSIONING

  public Version checkin() throws VersionException,
      UnsupportedRepositoryOperationException, InvalidItemStateException,
      RepositoryException {
    if (!this.isNodeType("mix:versionable"))
      throw new UnsupportedRepositoryOperationException(
          "Node.checkin() is not supported for not mix:versionable node ");
    if (!this.isCheckedOut())
      return this.getBaseVersion();
    if (session.getTransientNodesManager().hasPendingChanges(this.getLocation(),
        false))
      throw new InvalidItemStateException("Node has pending changes "
          + getPath());
    if (hasProperty("jcr:mergeFailed"))
      throw new VersionException("Node has jcr:mergeMailed " + getPath());

    Version ver = ((VersionHistoryImpl) getVersionHistory()).addVersion(this);

    WorkspaceDataManager wsManager = session.getTransientNodesManager()
        .getWorkspaceDataManager();

    wsManager.update(createChildProperty("jcr:isCheckedOut",
        new Value[] { valueFactory.createValue(false) }, PropertyType.BOOLEAN));
    wsManager.update(createChildProperty("jcr:baseVersion",
        new Value[] { valueFactory.createValue(ver) }, PropertyType.REFERENCE));
    wsManager.update(createChildProperty("jcr:predecessors", new Value[0],
        PropertyType.REFERENCE));

    return ver;
  }

  public void checkout() throws RepositoryException,
      UnsupportedRepositoryOperationException {
    if (!this.isNodeType("mix:versionable"))
      throw new UnsupportedRepositoryOperationException(
          "Node.checkout() is not supported for not mix:versionable node ");

    WorkspaceDataManager wsManager = session.getTransientNodesManager()
        .getWorkspaceDataManager();

    wsManager.update(createChildProperty("jcr:isCheckedOut",
        new Value[] { valueFactory.createValue(true) }, PropertyType.BOOLEAN));

    // The current value of N's jcr:baseVersion is copied to N's
    // jcr:predecessors property.
    wsManager.update(createChildProperty("jcr:predecessors",
        new Value[] { getProperty("jcr:baseVersion").getValue() },
        PropertyType.REFERENCE));

  }

  public boolean isCheckedOut() throws UnsupportedRepositoryOperationException,
      RepositoryException {

    if (isRoot())
      return true;

    if (this.isNodeType("mix:versionable"))
      return getProperty("jcr:isCheckedOut").getBoolean();

    NodeImpl ancestor = (NodeImpl) getParent();
    while (!ancestor.isRoot()) {
      if (ancestor.isNodeType("mix:versionable"))
        return ancestor.isCheckedOut();
      else
        ancestor = (NodeImpl) ancestor.getParent();
    }
    if (ancestor.isNodeType("mix:versionable"))
      return ancestor.isCheckedOut();
    else
      return true;
  }

  public VersionHistory getVersionHistory()
      throws UnsupportedRepositoryOperationException, RepositoryException {

    if (!this.isNodeType("mix:versionable"))
      throw new UnsupportedRepositoryOperationException(
          "Node is not mix:versionable " + getPath());

    String versionHistory = getProperty("jcr:versionHistory").getString();
    
    return (VersionHistory) session.getNodeByUUID(versionHistory);
    /*
    try {
      return ((WorkspaceImpl) session.getWorkspace())
          .getVersionStorageManager().getVersionHistoryByUUID(versionHistory, session);
    } catch (ItemNotFoundException e) {
      throw new RepositoryException("VersionHistory not found for " + getPath()
          + " Probably this versionable node was not saved (" + versionHistory
          + ")");
    }
*/
  }

  public Version getBaseVersion()
      throws UnsupportedRepositoryOperationException, RepositoryException {

    if (!this.isNodeType("mix:versionable"))
      throw new UnsupportedRepositoryOperationException(
          "Node is not versionable " + getPath());
    
    return (Version) session.getNodeByUUID(getProperty("jcr:baseVersion").getString());


//    return ((WorkspaceImpl) session.getWorkspace()).getVersionStorageManager()
//        .getVersionByUUID(getProperty("jcr:baseVersion").getString(), session);

  }

  public void restore(Version version, boolean removeExisting)
      throws UnsupportedRepositoryOperationException, RepositoryException {
    if (!this.isNodeType("mix:versionable"))
      throw new UnsupportedRepositoryOperationException(
          "Node is not versionable " + getPath());

    if (session.hasPendingChanges())
      throw new InvalidItemStateException("Session has pending changes ");

    if (version.getName().equals("jcr:rootVersion"))
      throw new VersionException(
          "It is illegal to call restore() on jcr:rootVersion");

    if (!((VersionHistoryImpl) getVersionHistory())
        .isVersionBelongToThis(version))
      throw new VersionException("Bad version " + version.getPath());

    // LockException

    NodeImpl restored = ((VersionImpl) version).restore(this, removeExisting);

    System.out.println(">>>>>>>>>>>>>>> "+dataManager.dump());

    // reload data
    // loadData(restored.getActualItemData());
  }

  public void restore(String versionName, boolean removeExisting)
      throws VersionException, UnsupportedRepositoryOperationException,
      RepositoryException, InvalidItemStateException {

    // throw VersionException if not found
    VersionImpl version = (VersionImpl) getVersionHistory().getVersion(
        versionName);
    restore(version, removeExisting);

  }

  public void restore(Version version, String relPath, boolean removeExisting)
      throws PathNotFoundException, ItemExistsException,
      ConstraintViolationException, UnsupportedRepositoryOperationException,
      RepositoryException {

    NodeImpl node;
    try {
      node = (NodeImpl) getNode(relPath);
    } catch (PathNotFoundException e) {
      JCRPath newPath = locationFactory.createJCRPath(getLocation(), relPath);
      // throw if not found
      NodeImpl pNode = (NodeImpl) getNode(newPath.makeParentPath().getAsString(
          false));
      // make dummy node in memory (nodeType will be replaced)
      node = this.createChildNode(pNode.getName(), "nt:base", true);
    }

    node.restore(version, removeExisting);
  }

  public void restoreByLabel(String versionLabel, boolean removeExisting)
      throws UnsupportedRepositoryOperationException, RepositoryException {

    VersionImpl version = (VersionImpl) getVersionHistory().getVersionByLabel(
        versionLabel);
    restore(version, removeExisting);

  }

  public NodeIterator merge(String srcWorkspace, boolean bestEffort)
      throws UnsupportedRepositoryOperationException, NoSuchWorkspaceException,
      AccessDeniedException, MergeException, RepositoryException {

    if (session.hasPendingChanges())
      throw new InvalidItemStateException("Session has pending changes ");

    Map failed = new HashMap();

    // get corresponding node
    SessionImpl srcSession = (SessionImpl) ((RepositoryImpl) session
        .getRepository()).login(session.getCredentials(), srcWorkspace);
    NodeImpl correspNode;
    try {
      correspNode = getCorrespondingNode(srcSession);
    } catch (PathNotFoundException e) {
      log.debug("No corresponding node in workspace: " + srcWorkspace);
      return new EntityCollection();
    }

    MergeVisitor visitor = new MergeVisitor((NodeImpl) getParent(), failed,
        bestEffort);
    correspNode.accept(visitor);
    NodeImpl n = visitor.getMergedNode();

    // System.out.println(">>>>>>>>> \n"+d.getDump());

    // ??? remove it
    //NodeDumpVisitor d = new NodeDumpVisitor();
    //visitor.getMergedNode().accept(d);
    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
    dataManager.getWorkspaceDataManager().delete(this);
    dataManager.getWorkspaceDataManager().add(n);
    dataManager.getWorkspaceDataManager().endSessionBatch(session);

    return createMergeFailed(failed);

  }

  private NodeIterator createMergeFailed(Map failed) throws RepositoryException {

    EntityCollection res = new EntityCollection();

    for (Iterator it = failed.keySet().iterator(); it.hasNext();) {

      String uuid = (String) it.next();
      NodeImpl versionable = (NodeImpl) session.getNodeByUUID(uuid);
      res.add(versionable);
      String offendingUuid = (String) failed.get(uuid);
      Value[] mergeFailed;
      boolean exists = false;
      try {
        mergeFailed = versionable.getProperty("jcr:mergeFailed").getValues();
      } catch (PathNotFoundException e) {
        mergeFailed = new Value[0];
      }

      HashSet mf = new HashSet();
      // add "old" values
      for (int i = 0; i < mergeFailed.length; i++) {
        mf.add(mergeFailed[i].getString());
      }
      // if not exists add "new" value and refresh jcr:mergeFailed property
      if (!mf.contains(offendingUuid)) {
        mf.add(offendingUuid);
        Value[] vals = new Value[mf.size()];
        int i = 0;
        for (Iterator ofs = mf.iterator(); ofs.hasNext(); i++)
          vals[i] = valueFactory.createValue((String) ofs.next(),
              PropertyType.REFERENCE);

        PropertyImpl prop = doSetProperty("jcr:mergeFailed", vals,
            PropertyType.REFERENCE, true, true, false);
        getParent().save();
      }
    }

    return res;

  }

  public void doneMerge(Version version) throws VersionException,
      InvalidItemStateException, UnsupportedRepositoryOperationException,
      RepositoryException {
    VersionImpl base = (VersionImpl) getBaseVersion();
    base.addPredecessor(version);
    removeMergeFailed(version);
  }

  public void cancelMerge(Version version) throws VersionException,
      InvalidItemStateException, UnsupportedRepositoryOperationException,
      RepositoryException {
    removeMergeFailed(version);
  }

  private void removeMergeFailed(Version version) throws RepositoryException {
    Value[] mergeFailed;
    boolean exists = false;
    PropertyImpl prop = (PropertyImpl) getProperty("jcr:mergeFailed");
    mergeFailed = prop.getValues();
    ArrayList mf = new ArrayList();
    for (int i = 0; i < mergeFailed.length; i++) {
      // System.out.println("REMOVE FAILED "+mergeFailed[i].getString()+"
      // "+version.getUUID());
      if (!mergeFailed[i].getString().equals(version.getUUID()))
        mf.add(mergeFailed[i].getString());
    }

    if (mf.size() > 0) {
      mergeFailed = new Value[mf.size()];
      for (int i = 0; i < mergeFailed.length; i++) {
        mergeFailed[i] = valueFactory.createValue((String) mf.get(i),
            PropertyType.REFERENCE);
      }
      this.removeChildProperty(prop);
      prop = createChildProperty("jcr:mergeFailed", mergeFailed,
          PropertyType.REFERENCE);
      // dataManager.getWorkspaceDataManager().update(prop);
    } else {
      this.removeChildProperty(prop);
      // dataManager.getWorkspaceDataManager().delete(prop);
    }
    getParent().save();

  }

  // Locks

  public Lock lock(boolean isDeep, boolean isSessionScoped)
      throws UnsupportedRepositoryOperationException, LockException,
      AccessDeniedException, RepositoryException {

    if (!isNodeType("mix:lockable"))
      throw new LockException("Node is not lockable " + getPath());

    Lock lock = session.getLockManager().getLock(this, false);
    if (lock != null)
      throw new LockException("Node is already locked " + getPath() + " on "
          + lock.getNode().getPath());

    if (isDeep && session.getLockManager().getChildLock(this) != null)
      throw new LockException("Lock is deep and node already has locked child "
          + getPath());

    if (session.getTransientNodesManager().hasPendingChanges(getLocation(),
        false))
      throw new InvalidItemStateException("Node has pending unsaved changes "
          + getPath());
    
    PropertyImpl lockOwner = createChildProperty("jcr:lockOwner",
        new Value[] { valueFactory.createValue(session.getUserID()) },
        PropertyType.STRING);
    PropertyImpl lockIsDeep = createChildProperty("jcr:lockIsDeep",
        new Value[] { valueFactory.createValue(isDeep) }, PropertyType.BOOLEAN);
    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
    dataManager.getWorkspaceDataManager().add(lockOwner);
    dataManager.getWorkspaceDataManager().add(lockIsDeep);
    dataManager.getWorkspaceDataManager().endSessionBatch(session);

    Lock newLock = session.getLockManager().addLock(this, isDeep,
        isSessionScoped);
    return newLock;

  }

  public Lock getLock() throws UnsupportedRepositoryOperationException,
      LockException, AccessDeniedException, RepositoryException {
    LockImpl lock = session.getLockManager().getLock(this, false);
    if (lock == null)
      throw new LockException("Lock not found " + getPath());
    return lock;

  }

  public void unlock() throws UnsupportedRepositoryOperationException,
      LockException, AccessDeniedException, RepositoryException {

    if (!session.hasLockToken(session.getLockManager().getLockToken(this)))
      throw new LockException(
          "There are no permission to unlock the node or node is not locked "
              + getPath());

    if (session.getTransientNodesManager().hasPendingChanges(getLocation(),
        false))
      throw new InvalidItemStateException("Node has pending unsaved changes "
          + getPath());

    PropertyImpl lockOwner = (PropertyImpl) getProperty("jcr:lockOwner");
    PropertyImpl lockIsDeep = (PropertyImpl) getProperty("jcr:lockIsDeep");
    removeChildProperty(lockOwner);
    removeChildProperty(lockIsDeep);
    
    dataManager.getWorkspaceDataManager().beginSessionBatch(session);
    dataManager.getWorkspaceDataManager().delete(lockOwner);
    dataManager.getWorkspaceDataManager().delete(lockIsDeep);
    dataManager.getWorkspaceDataManager().endSessionBatch(session);

    session.getLockManager().removeLock(this);
  }

  public boolean holdsLock() throws RepositoryException {
    return (session.getLockManager().getLock(this, true) != null);
  }

  public boolean isLocked() throws RepositoryException {
    return (session.getLockManager().getLock(this, false) != null);
  }

  boolean checkLocking() throws RepositoryException {
    return (!isLocked()
        || session.hasLockToken(session.getLockManager().getLockToken(this)) || session
        .getUserID().equals(SystemIdentity.SYSTEM));
  }

  // ////////////////// Item implementation ////////////////////

  /**
   * @see javax.jcr.Item#accept
   */
  public void accept(ItemVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  /**
   * @see javax.jcr.Item#isNode
   */
  public boolean isNode() {
    return true;
  }

  // ////////////////////////////////////////////////////

  protected void addAutoCreatedItems(NodeType type,
      boolean registerInDataManager) throws RepositoryException,
      ConstraintViolationException {

    NodeDefinition[] nodeDefs = type.getChildNodeDefinitions();
    PropertyDefinition[] propDefs = type.getPropertyDefinitions();

    // Add autocreated child properties
    for (int i = 0; i < propDefs.length; i++) {

      if (propDefs[i] == null) // it is possible for not mandatory propDef
        continue;

      if (propDefs[i].isAutoCreated()) {

        Value[] propVal = propDefs[i].getDefaultValues();
        // there can be null in definition but should not be null value
        if (propVal == null) {
          propVal = new Value[1];
          propVal[0] = null;
        }
        boolean multiple = propDefs[i].isMultiple();
        String name = propDefs[i].getName();
        JCRPath loc = session.getLocationFactory()
            .createJCRPath(getLocation(), name);

        PropertyImpl prop = doSetProperty(name, propVal, propDefs[i]
            .getRequiredType(), multiple, registerInDataManager, false);
      }
    }

    // Add autocreated child nodes
    for (int i = 0; i < nodeDefs.length; i++) {
      if (nodeDefs[i].isAutoCreated()) {
        JCRName nodeName = locationFactory.parseJCRName(nodeDefs[i].getName());
        JCRName primaryTypeName = locationFactory.parseJCRName(nodeDefs[i]
            .getDefaultPrimaryType().getName());

        NodeImpl child = doAddNode(this, nodeName, primaryTypeName, null, true,
            true);
        // put to dataManager also if parent not new
        if (registerInDataManager) {
          dataManager.update(child);
        }

      }
    }

    initAutoCreatedItems(type, registerInDataManager);

  }

  private NodeType nodeType(String name) throws NoSuchNodeTypeException,
      RepositoryException {
    return session.getWorkspace().getNodeTypeManager().getNodeType(name);
  }

  public NodeType[] getAllNodeTypes() throws RepositoryException {
    NodeType primaryType = getPrimaryNodeType();
    NodeType[] mixinTypes = getMixinNodeTypes();
    NodeType[] types = new NodeType[mixinTypes.length + 1];
    types[0] = primaryType;
    for (int j = 0; j < mixinTypes.length; j++) {
      types[j + 1] = mixinTypes[j];
    }

    return types;
  }

  public String[] getMixinTypeNames() throws RepositoryException {
    NodeType[] mixinTypes = getMixinNodeTypes();
    String[] mtNames = new String[mixinTypes.length];
    for (int i = 0; i < mtNames.length; i++)
      mtNames[i] = mixinTypes[i].getName();
    return mtNames;
  }

  public void validateMandatoryChildren() throws ConstraintViolationException,
      RepositoryException {
    ArrayList mandatoryItemDefs = ((NodeTypeImpl) getPrimaryNodeType())
        .getManadatoryItemDefs();
    NodeType[] mixinTypes = getMixinNodeTypes();
    for (int i = 0; i < mixinTypes.length; i++)
      mandatoryItemDefs.addAll(((NodeTypeImpl) mixinTypes[i])
          .getManadatoryItemDefs());
    Iterator defs = mandatoryItemDefs.iterator();
    while (defs.hasNext()) {
      ItemDefinition def = (ItemDefinition) defs.next();
      if ((def instanceof NodeDefinition && !hasNode(def.getName()))
          || (def instanceof PropertyDefinition && !hasProperty(def.getName())))

        throw new ConstraintViolationException("Mandatory item "
            + def.getName() + " not found. Node [" + getPath()
            + " primary type: " + this.getPrimaryNodeType().getName() + "]");
    }
  }

  public void initAutoCreatedItems(NodeType type, boolean registerInDataManager)
      throws RepositoryException {

    if (type.isNodeType("nt:base")) {
      PropertyImpl prop = doSetProperty("jcr:primaryType",
          new Value[] { valueFactory.createValue(primaryType) },
          PropertyType.NAME, false, false, false);
    }
    if (type.isNodeType("mix:referenceable")) {
      doSetProperty("jcr:uuid", new Value[] { valueFactory
          .createValue(internalUUID) }, PropertyType.STRING, false, false,
          false);
    }
    if (type.isNodeType("nt:hierarchyNode")) {
      Value[] values = new Value[] { valueFactory.createValue(dataManager
          .getWorkspaceDataManager().getCurrentTime()) };
      doSetProperty("jcr:created", values, PropertyType.DATE, false, false,
          false);
    }
    if (!session.getAccessManager().getAccessControlPolicy().
        equals(AccessControlPolicy.DISABLE) && 
        type.isNodeType("exo:accessControllable")) {
      Map perms = new HashMap();
      perms.put(SystemIdentity.ANY, PermissionType.DEFAULT_AC);
      setChildrenPermissions(perms);
    }
    if (type.isNodeType("mix:versionable")) {
      

      // Generate uuids
      Uuid versionHistoryUUID = uuidGenerator.generateUUID(getPath()
          + "/jcr:versionHistory");
      Uuid baseVersionUUID = uuidGenerator.generateUUID(getPath()
          + "/jcr:baseVersion");

      // 
      doSetProperty("jcr:versionHistory", new Value[] { valueFactory
          .createValue(versionHistoryUUID) }, PropertyType.REFERENCE, false,
          registerInDataManager, false);
      doSetProperty("jcr:baseVersion", new Value[] { valueFactory
          .createValue(baseVersionUUID) }, PropertyType.REFERENCE, false,
          registerInDataManager, false);
      doSetProperty("jcr:predecessors", new Value[] { valueFactory
          .createValue(baseVersionUUID) }, PropertyType.REFERENCE, true,
          registerInDataManager, false);
      doSetProperty("jcr:isCheckedOut", new Value[] { valueFactory
          .createValue(true) }, PropertyType.BOOLEAN, false,
          registerInDataManager, false);

    }

  }

  // ExtendedNode -----------------------------

  public void setChildrenPermissions(Map permissions)
      throws RepositoryException, AccessDeniedException, AccessControlException {

    checkChildrenPermission(PermissionType.CHANGE_PERMISSION);

    String ownerName;
    // First who make it accessControllable is owner
    if (!isNodeType("exo:accessControllable")) {
      ownerName = session.getUserID();
      addMixin("exo:accessControllable");
    } else {
      if (!childrenACL.getOwner().equals(SystemIdentity.SYSTEM))
        ownerName = childrenACL.getOwner();
      else
        ownerName = session.getUserID();
    }
    childrenACL = new AccessControlList(ownerName, permissions);
    // log.debug("ACL owner"+childrenACL.getOwner());

    // populate exo:accessControllable
    Value[] owner = new Value[] { valueFactory.createValue(childrenACL
        .getOwner()) };
    boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    doSetProperty("exo:owner", owner, PropertyType.STRING, false,
        registerInDataManager, false);

    saveChildPermissionsProperty();
  }

  private void saveChildPermissionsProperty() throws RepositoryException {

    boolean registerInDataManager = (this.state != ItemChangeState.ADDED);
    Map permissions = childrenACL.getPermissions();
    List permValues = new ArrayList();
    for (Iterator i = permissions.keySet().iterator(); i.hasNext();) {
      String identity = (String) i.next();
      String[] perm = (String[]) permissions.get(identity);
      for (int j = 0; j < perm.length; j++)
        permValues.add(new PermissionValue(identity, perm[j]));
    }

    Value[] permArray = new Value[permValues.size()];
    for (int i = 0; i < permArray.length; i++)
      permArray[i] = (Value) permValues.get(i);

    doSetProperty("exo:permissions", permArray,
        ExtendedPropertyType.PERMISSION, true, registerInDataManager, false);

  }

  public AccessControlList getChildrenACL() {
    return childrenACL;
  }

  public void clearChildrenACL() throws RepositoryException,
      AccessControlException {
    checkChildrenPermission(PermissionType.CHANGE_PERMISSION);
    if (isNodeType("exo:accessControllable")) {
      removeMixin("exo:accessControllable");
    }
    childrenACL = new AccessControlList(true);
  }

  public void removeChildrenPermission(String identity)
      throws RepositoryException, AccessControlException {
    checkChildrenPermission(PermissionType.CHANGE_PERMISSION);
    // remove default permission
    if (!isNodeType("exo:accessControllable")) {
      throw new AccessControlException(
          "It is impossible to remove default permissions. Use setPermission(s) instead");
    }
    childrenACL.getPermissions().remove(identity);
    saveChildPermissionsProperty();
  }

  public void setChildrenPermission(String identity, String[] permission)
      throws RepositoryException, AccessControlException {
    // check if changing permission allowed
    checkChildrenPermission(PermissionType.CHANGE_PERMISSION);
    // remove default permission
    if (!isNodeType("exo:accessControllable")) {
      Map perms = new HashMap();
      perms.put(identity, permission);
      setChildrenPermissions(perms);
      // childrenACL = new AccessControlList(false);
    } else {
      childrenACL.getPermissions().put(identity, permission);
      saveChildPermissionsProperty();
    }
  }

  public void checkChildrenPermission(String actions)
      throws AccessControlException, RepositoryException {
    if (!session.getAccessManager().hasPermission(childrenACL, actions,
        session.getUserID()))
      throw new AccessControlException("Permission denied " + getPath() + " : "
          + actions);

  }

  // ---------------------------------------------------------

  /**
   * @return Returns the mixinTypes.
   */
  JCRName[] getMixinTypes() {
    return mixinTypes;
  }

  /**
   * @return Returns the primaryType.
   */
  JCRName getPrimaryType() {
    return primaryType;
  }

  /**
   * @return Returns the childNodes.
   */
  boolean isChildNodesInitialized() {
    return childNodes != null;
  }

  /**
   * @return Returns the childProperties.
   */
  boolean isChildPropertiesInitialized() {
    return childProperties != null;
  }

}