/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.itemvisitors;

import java.util.Stack;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.util.TraversingItemVisitor;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * Copies accepted node to parent structure in memory
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: NodeCopyVisitor.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class NodeCopyVisitor extends TraversingItemVisitor {

  protected Stack parents;
  protected NodeImpl context;
  protected boolean keepUUIDs;

  public NodeCopyVisitor(NodeImpl context, boolean keepUUIDs) {
    super();
    this.context = context;
    this.parents = new Stack();
    parents.push(context);
    this.keepUUIDs = keepUUIDs;
    //System.out.println("Node to copy: "+context.getPath());

  }

  protected void entering(Property property, int level)
      throws RepositoryException {
    
    String name = property.getName();
    Value[] values = ((PropertyImpl) property).getValueArray();
    int type = property.getType();
    //NodeImpl localParent = (NodeImpl) parents.peek();
    PropertyImpl prop = curParent().createChildProperty(name, values, type);
    System.out.println("Copy property: "+prop.getPath()+" "+curParent().getPath());
  }

  protected void entering(Node node, int level) throws RepositoryException {
    if(level == 0)
      return;
/*    
    String name = node.getName();
    String primaryTypeName = node.getPrimaryNodeType().getName();
    String[] mixinNodeTypes = new String[node.getMixinNodeTypes().length];
    for (int i = 0; i < node.getMixinNodeTypes().length; i++) {
      mixinNodeTypes[i] = node.getMixinNodeTypes()[i].getName();
    }
    
    NodeImpl nodeImpl = (NodeImpl)node;
    InternalQName qName = nodeImpl.getActualItemData().getQName();
    InternalQName ptQName = ((NodeData)nodeImpl.getActualItemData()).getPrimaryTypeName();
    InternalQName[] mtQNames = ((NodeData)nodeImpl.getActualItemData()).getMixinTypeNames();
    int index = nodeImpl.getActualItemData().getQPath().getIndex();
    String uuid = (keepUUIDs)?nodeImpl.getActualItemData().getUUID():null;
*/    
    NodeImpl nodeImpl = (NodeImpl)node;
    
    NodeImpl localParent = (NodeImpl) parents.peek();
     
    NodeImpl newNode = localParent.copyChildNodeInmemory(
        nodeImpl.getActualItemData().getQName(),
        (NodeData)nodeImpl.getActualItemData(), keepUUIDs,
        false, false, false, false);
    
//    NodeImpl newNode = localParent.createChildNodeInmemory(qName, ptQName, mtQNames, 
//        uuid, index, false, false, false);
    
/*    
    NodeImpl newNode;

    if(keepUUIDs) {
      newNode = localParent.createChildNode(name, primaryTypeName, mixinNodeTypes, 
        ((NodeImpl)node).getInternalUUID(), true, false);
    } else {
      newNode = localParent.createChildNode(name, primaryTypeName,
        mixinNodeTypes, false);
    }
*/    
    parents.push(newNode);
  }

  protected void leaving(Property property, int level)
      throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    parents.pop();
  }
  
  protected NodeImpl curParent() {
    return (NodeImpl) parents.peek();
  }

}