/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.lock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jcr.AccessDeniedException;
import javax.jcr.RepositoryException;
import javax.jcr.lock.Lock;

import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.SessionLifecycleListener;


/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: LockManager.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class LockManager implements SessionLifecycleListener{

  // lockToken -> lock
  private Map locks;
  
//  private WorkspaceDataManager dataManager;
  
//  private ValueFactoryImpl valueFactory;
  
  public LockManager() {
    locks = new HashMap();
//    this.dataManager = dataManager;
//    this.valueFactory = valueFactory;
  }
  
  
  /**
   * Adds new lock
   * @param node - lock holder
   * @param deep
   * @param sessionScoped
   * @return
   * @throws RepositoryException
   */
  public synchronized Lock addLock(NodeImpl lockHolder, boolean deep, boolean sessionScoped) throws RepositoryException {
    LockData data = new LockData(lockHolder, deep, sessionScoped);
    lockHolder.getSession().addLockToken(getLockToken(lockHolder));
    //System.out.println("TOKEN >> "+lock.getLockToken());
    locks.put(getLockToken(lockHolder), data);
    return new LockImpl(this, data, true, lockHolder);
  }
  
  /**
   * Removes lock
   * @param node - lock holder
   */
  public synchronized void removeLock(NodeImpl lockHolder) {
    //LockData data = (LockData)locks.get(getLockToken(node));
    //lock.setLive(false);
    lockHolder.getSession().removeLockToken(getLockToken(lockHolder));
    locks.remove(getLockToken(lockHolder));
  }
  
  public synchronized LockImpl getLock(NodeImpl node, boolean holdOnly) throws AccessDeniedException, 
    RepositoryException {
    
    if(!node.hasPermission(PermissionType.READ))
      throw new AccessDeniedException("Get Lock Access denied "+node.getPath());
    
    LockData lock = (LockData)locks.get(getLockToken(node));
    
    if(lock == null && !holdOnly) {
      for(Iterator it=locks.values().iterator(); it.hasNext();) {
        LockData lock1 = (LockData)it.next();
        if(lock1.isDeep() &&
           node.getLocation().isDescendantOf(((NodeImpl)lock1.getNode()).getLocation(), false)) {
           lock = lock1;
           break;
        }
      }    
    }
    return createLock(node, lock);
  }
  
  public synchronized LockImpl getChildLock(NodeImpl node) throws AccessDeniedException, 
  RepositoryException {
  
    if(!node.hasPermission(PermissionType.READ))
      throw new AccessDeniedException("Get Lock Access denied "+node.getPath());

    LockData lock = null;
    for(Iterator it=locks.values().iterator(); it.hasNext();) {
      LockData lock1 = (LockData)it.next();
      if(((NodeImpl)lock1.getNode()).getLocation().isDescendantOf(node.getLocation(), false)) {
        lock = lock1;
        break;
      }
    }
    return createLock(node, lock);
  }
  
  public String getLockToken(NodeImpl node) {
    return node.getInternalUUID();
  }

  private LockImpl createLock(NodeImpl node, LockData data) {
    if(data == null)
      return null;

    //String token = ((SessionImpl)node.getSession()).getLockToken(getLockToken(node));
    return new LockImpl(this, data, true, node);
  }
  
  // ---------------- SessionLifecycleListener ----------
  
  public void onCloseSession(SessionImpl session) {
    List remove = new ArrayList();
    for(Iterator it=locks.values().iterator(); it.hasNext();) {
      LockData lock = (LockData)it.next();
      
      try {
        if(lock.isSessionScoped() && lock.getNode().getSession().equals(session))
          remove.add(getLockToken((NodeImpl)lock.getNode())); 
      } catch (RepositoryException e) {
        e.printStackTrace();
      }
    }
    for(int i=0; i<remove.size(); i++) 
      locks.remove(remove.get(i));
    
  }
}
