/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import java.util.StringTokenizer;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.services.jcr.impl.core.value.StringValue;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Created by The eXo Platform SARL
 *  Used as rule for import document view in Session
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: DocViewRule.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class DocViewRule extends CommonRule {

  private Session session;

  protected Session getSession() {
    return session;
  }

  DocViewRule(Node startNode) {
    pushNode(startNode);

    try {
      session = startNode.getSession();
    } catch (RepositoryException ex) {
      log.error("Can't get session from start node");
      throw new RuntimeException("Can't get session from start node", ex);
    }

  }

  public void begin(String namespace, String name, Attributes attributes)
      throws Exception {
    super.begin(namespace, name, attributes);

    createNode(namespace, name, attributes.getValue("jcr:primaryType"));
    //        createNode(namespace,name,attributes.getValue("jcr:primaryType"));

    setNodeMixinTypes(attributes.getValue("jcr:mixinTypes"));
    createNodeProperties(attributes);
  }

  public void body(String namespace, String name, String text) throws Exception {

    if (!"".equals(text)) {
      super.body(namespace, name, text);
      String jcrNamespaceURI = session.getNamespaceURI("jcr");
      createNode(jcrNamespaceURI, "xmltext", "nt:unstructured");
      popNode();

      createStringProperty("jcr:xmlcharacters", text);

    }
  }

  public void end(String namespace, String name) throws Exception {
    super.end(namespace, name);
    popNode();
  }

  //    private void skipProperty(String propertyName, String propertyValue){
  //        log.debug("Skip property name["+propertyName+"] propertyValue["+
  //                propertyValue+"]");
  //    }

  private void setNodeMixinTypes(String types)
      throws javax.jcr.RepositoryException {
    if (types != null) {
      StringTokenizer st = new StringTokenizer(types);
      Node node = peekNode();
      while (st.hasMoreTokens()) {
        String mixinType = st.nextToken();
        log.trace("Add mixin type[" + mixinType + "]");
        node.addMixin(mixinType);
      }
    }
  }

  private void createNodeProperties(Attributes attributes) throws Exception {
    for (int i = 0; i < attributes.getLength(); i++) {
      String propertyName = attributes.getQName(i);
      String propertyValue = attributes.getValue(i);
      createStringProperty(propertyName, propertyValue);
    }
  }

  private Property createStringProperty(String propertyName,
      String propertyValue) throws RepositoryException, SAXException {
    //        return null;
    //        log.debug("Create string proeprty name ["+propertyName+"] value ["+propertyValue+"]");
    return createProperty(propertyName,

    new StringValue(propertyValue), PropertyType.STRING);
  }

}