/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL                               *
 * All rights reserved.                                                    *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.importing;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Created by The eXo Platform SARL
 *  Used to temporary store definition of XML element.
 *  Used on sys view import.
 *  Store name, namespace, attributes of XML element.
 *  Also can store target class - class used as target for import this XML element
 *
 * @author <a href="mailto:alex.kravchuk@gmail.com">Alexander Kravchuk</a>
 * @version $Id: XMLElementDefinition.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class XMLElementDefinition {
  private static final Log log = LogFactory.getLog(XMLElementDefinition.class);

  private String namespaceURI="";
  private String localName="";
  private String body = null;
  private Attributes attributes = new AttributesImpl();
  private Class targetClass = this.getClass();
  private Collection childDefinitions = new ArrayList();
  private boolean proceed = false;



  XMLElementDefinition(String namespaceURI,String localName,
                              Attributes attributes){
      setNamespaceURI(namespaceURI);
      setLocalName(localName);
      setAttributes(attributes);
  }

  public XMLElementDefinition(String namespaceURI,String localName,
                              Attributes attributes, Class targetClass){
      setNamespaceURI(namespaceURI);
      setLocalName(localName);
      setAttributes(attributes);
      setTargetClass(targetClass);
  }



  public void setBody(String body){
      this.body = body;
  }


  public void setNamespaceURI(String value){
      if  (value == null)
          throw new NullPointerException("namespaceURI can't be null");
      this.namespaceURI = value;
  }

  public void setLocalName(String value){
      if  (value == null)
          throw new NullPointerException("localName can't be null");
      this.localName = value;
  }

  public void setAttributes(Attributes attributes){
      if  (attributes == null)
          throw new NullPointerException("attributes can't be null");
      this.attributes = attributes;
  }

  public void setTargetClass(Class clazz){
      if  (clazz == null)
          throw new NullPointerException("clazz can't be null");
      this.targetClass = clazz;
  }


  public void setProceed(boolean proceed){
      this.proceed = proceed;
  }


  public String getBody(){
      return body;
  }



  public boolean isProceed(){
      return proceed;
  }


  public String getNamespaceURI( ){
      return namespaceURI;
  }

  public String getLocalName(){
      return localName;
  }

  public Attributes getAttributes(){
      return attributes;
  }

  public Class getTargetClass(){
      return targetClass;
  }

  /**
   *  Check if target class corespond (can be assigned) to passed class
   *
   * @param cls Class
   * @return boolean Return true if passed class is same or descendant of targetClass
   */
  public boolean isProperClass(Class cls){

    return targetClass.isAssignableFrom(cls);
  }

  public void addChild(XMLElementDefinition child){
      if  (child == null)
          throw new NullPointerException("child can't be null");
      childDefinitions.add(child);
  }

   public void clearChildren(){
       childDefinitions.clear();
   }

   public Iterator getChildrenIterator(){
       return childDefinitions.iterator();
   }

   public int getChildrenCount(){
       return childDefinitions.size();
   }


   public String toString(){

       String strAtts = "atts count [ "+attributes.getLength()+"] (";
       for(int i=0; i<attributes.getLength(); i++){
           strAtts += " (qName["+attributes.getQName(i)+"] value["+
                   attributes.getValue(attributes.getQName(i))+"]);";
       }
       strAtts += " ) ";

       return  "localName["+
               localName+"] body["+body+
               "] targetClass["+targetClass.getName()+
               "] attributes.getLength()["+
               attributes.getLength()+"] namespaceURI ["+namespaceURI+
               "] childDefinitions count ["+childDefinitions.size()+"] "+
               strAtts+
               super.toString();

   }

}
