/****************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: MultiRDBPropertyRecord.java 3510 2005-12-29 17:32:57Z geaz $
 * 
 * @hibernate.joined-subclass table="JCR_MPROPERTY" dynamic-update="true"
 * @hibernate.joined-subclass-key column="ID"
 * @hi--bernate.cache usage="read-write"
 */

public class MultiRDBPropertyRecord extends MultiRDBItemRecord implements
    PropertyRecord {

  protected Set propertyValues;

  protected Integer valuesType;

  public MultiRDBPropertyRecord() {
  }

  /**
   * ---lazy="proxy"---
   * 
   * @hibernate.set cascade="all" inverse="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBValueRecord"
   * @hibernate.collection-key column="PROPERTY_ID" not-null="true"
   * @hibe---rnate.collection-index column="ORDER_NUM"
   *  
   */
  public Set getPropertyValues() {
    return propertyValues;
  }

  public void setPropertyValues(Set values) {
    this.propertyValues = values;
  }

  /**
   * @hibernate.property column="TYPE" not-null="true"
   */
  public Integer getValuesType() {
    return valuesType;
  }

  public void setValuesType(Integer type) {
    this.valuesType = type;

  }

  /**
   * @hibernate.many-to-one column="PARENT_ID" not-null="true"
   *                        class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   */
  public NodeRecord getParentNode() {
    return parentNode;
  }

  public void setParentNode(NodeRecord parent) {
    this.parentNode = parent;
    this.parentUUID = new String(parent.getId());
  }

  // ------------------ PropertyData --------------------------

  public int getType() {
    return getValuesType().intValue();
  }

  public List getValues() {
    ArrayList valueList = null;
    if (propertyValues != null)
      valueList = new ArrayList(propertyValues);

    return valueList;
  }

  public void setType(int type) {
    this.setValuesType(new Integer(type));
  }

  // Value Datas
  public void setValues(List values) throws RepositoryException {
    if (values == null)
      throw new RepositoryException("Set values failed for '" + path
          + "' values == null");
    propertyValues = new HashSet();
    //setPropertyValues(propertyValues);
    for (int i = 0; i < values.size(); i++) {
      MultiRDBValueRecord val = new MultiRDBValueRecord();
      val.setData(((ValueData) values.get(i)).getAsByteArray());
      val.setProperty(this);
      val.setOrderNumber(i);
      propertyValues.add(val);
    }
  }

  public boolean isNode() {
    return false;
  }

  // ---------------------------------------------------------

  public String toString() {
    return "PropertyRecord: " + path + " type: "
        + ExtendedPropertyType.nameFromValue(this.getType()) + " uuid: "
        + getUUID() + " values: " + getPropertyValues();
  }

}