/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.xml;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;


/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: NodeImporter.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class NodeImporter implements ContentHandler {

  private ContentHandler wrappedHandler;
  protected NodeImpl parent;

  NodeImporter(NodeImpl parent) {
    this.parent = parent;
  }
 
  public void characters(char[] ch, int start, int length) throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.characters(ch, start, length);
  }

  public void endDocument() throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.endDocument();
  }

  public void endElement(String namespaceURI, String localName, String qName)
      throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.endElement(namespaceURI, localName, qName);
  }

  public void endPrefixMapping(String prefix) throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.endPrefixMapping(prefix);
  }

  public void ignorableWhitespace(char[] ch, int start, int length)
      throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.ignorableWhitespace(ch, start, length);
  }

  public void processingInstruction(String target, String data)
      throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.processingInstruction(target, data);
  }

  public void setDocumentLocator(Locator locator) {
    if(wrappedHandler == null)
      throw new RuntimeException("Handler is not set.");
    wrappedHandler.setDocumentLocator(locator);
  }

  public void skippedEntity(String name) throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.skippedEntity(name);
  }

  public void startDocument() throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.startDocument();
  }

  public void startElement(String namespaceURI, String localName, String qName,
      Attributes atts) throws SAXException {
    if(wrappedHandler == null) {
      try {
        recognizeDocumentType(namespaceURI, qName);
      } catch (RepositoryException e) {
        throw new SAXException(e);      
      }
    }
    wrappedHandler.startElement(namespaceURI, localName, qName, atts);
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {
    if(wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.startPrefixMapping(prefix, uri);
  }
  
  protected final void recognizeDocumentType(String namespaceURI, String qName) 
  throws RepositoryException {
    
    if ("http://www.jcp.org/jcr/sv/1.0".equals(namespaceURI)
        && qName.equals("sv:node")) {
      wrappedHandler = new SysNodeImporter(parent);
    } else {
      wrappedHandler = new DocNodeImporter(parent);
    }
  }

}
