/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.api.reading;


import java.io.IOException;
import java.io.InputStream;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;
import org.exoplatform.services.jcr.impl.core.value.BooleanValue;
import org.exoplatform.services.jcr.impl.core.value.DateValue;
import org.exoplatform.services.jcr.impl.core.value.StringValue;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestValue.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class TestValue extends JcrAPIBaseTest{

  public void testGetString() throws RepositoryException {
    Value value = new StringValue("text");
    assertEquals("text", value.getString());
  }

  public void testGetDouble() throws RepositoryException {
    Value value = new StringValue("text");
    try {
      value.getDouble();
      fail("exception should have been thrown");
    } catch (ValueFormatException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }

    value = new StringValue("20");
    assertEquals(20, (int)value.getDouble());

    try {
      value.getStream();
      fail("exception should have been thrown");
    } catch (IllegalStateException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }
  }

  public void testGetLong() throws RepositoryException {
    Value value = new StringValue("text");
    try {
      value.getDouble();
      fail("exception should have been thrown");
    } catch (ValueFormatException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }

    value = new StringValue("15");
    assertEquals(15, value.getLong());

    try {
      value.getStream();
      fail("exception should have been thrown");
    } catch (IllegalStateException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }
  }

  public void testGetStream() throws RepositoryException, IOException {
    Value value = new BinaryValue(new String("inputStream").getBytes());
    InputStream iS = value.getStream();
    int aval = iS.available();
    byte[] bytes = new byte[iS.available()];
    iS.read(bytes);
    assertEquals("inputStream", new String(bytes));

    assertEquals(aval, value.getStream().available());

    value = new StringValue("text");
    iS = value.getStream();
    bytes = new byte[2];
    iS.read(bytes);
    assertEquals("te", new String(bytes));
	//Once a Value object has been read once using getStream(),
    // all subsequent calls to getStream() will return the same stream object.
    iS = value.getStream();
    bytes = new byte[2];
    iS.read(bytes);
    assertEquals("xt", new String(bytes));

  }

  public void testGetDate() throws RepositoryException {
    Value value = new StringValue("text");
    try {
      value.getDate();
      fail("exception should have been thrown");
    } catch (ValueFormatException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }

    Calendar calendar = new GregorianCalendar();
    value = new DateValue(calendar);
    assertEquals(calendar, value.getDate());

    try {
      value.getStream();
      fail("exception should have been thrown");
    } catch (IllegalStateException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }
  }

  public void testGetBoolean() throws RepositoryException {
    Value value = valueFactory.createValue(10l);
    try {
      value.getBoolean();
      fail("exception should have been thrown");
    } catch (ValueFormatException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }

    value = new BooleanValue(true);
    assertTrue(value.getBoolean());

    try {
      value.getStream();
      fail("exception should have been thrown");
    } catch (IllegalStateException e) {
    } catch (RepositoryException e) {
      fail("not good exception thrown");
    }
  }

  public void tesGetType() throws Exception {
    Value value = valueFactory.createValue("");
    assertEquals(PropertyType.STRING, value.getType());
    value = valueFactory.createValue(10l);
    assertEquals(PropertyType.LONG, value.getType());
    value = valueFactory.createValue(10.0);
    assertEquals(PropertyType.DOUBLE, value.getType());
    value = valueFactory.createValue(true);
    assertEquals(PropertyType.BOOLEAN, value.getType());
    value = valueFactory.createValue(new GregorianCalendar());
    assertEquals(PropertyType.DATE, value.getType());
    value = valueFactory.createValue("", PropertyType.BINARY);
    assertEquals(PropertyType.BINARY, value.getType());
    value = valueFactory.createValue("uuid", PropertyType.REFERENCE);
    assertEquals(PropertyType.REFERENCE, value.getType());
    value = valueFactory.createValue("jcr:content", PropertyType.NAME);
    assertEquals(PropertyType.NAME, value.getType());
    value = valueFactory.createValue("/content", PropertyType.PATH);
    assertEquals(PropertyType.PATH, value.getType());

  }

  public void testEquals() throws Exception {
  	
    assertTrue(valueFactory.createValue("test").equals(valueFactory.createValue("test")));
    assertFalse(valueFactory.createValue("2").equals(valueFactory.createValue(2l)));
  }

}
