/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.api.writing;


import java.util.Calendar;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;
import org.exoplatform.services.jcr.impl.core.value.StringValue;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestCorrespondingNode.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class TestCorrespondingNode extends JcrAPIBaseTest {

  public void setUp() throws Exception {
     super.setUp();
     if(!((RepositoryImpl) repository).isWorkspaceInitialized("ws2"));
        ((RepositoryImpl) repository).initWorkspace("ws2", "nt:unstructured");
     
  }
  
  public void testDifferentWs() throws RepositoryException {

  	
  }
  
  
  public void testCorrespondingPath() throws RepositoryException {


    Session session2 = repository.login(credentials, "ws2");
    Node root = session2.getRootNode();
    Node file = root.addNode("testCorrespondingPath", "nt:folder").addNode("childNode2", "nt:file");
    Node contentNode = file.addNode("jcr:content", "nt:resource");
    contentNode.setProperty("jcr:data", new BinaryValue("this is the content"));
    contentNode.setProperty("jcr:mimeType", new StringValue("text/html"));
    contentNode.setProperty("jcr:lastModified", session.getValueFactory().createValue(Calendar.getInstance()));

    session2.save();

    //root = session.getRootNode();
    //root.addNode("test", "nt:unstructured");
    //session.save();

    workspace.clone("ws2", "/testCorrespondingPath", "/testCorrespondingPath1", false);

    session = (SessionImpl)repository.login(credentials, WORKSPACE);
    Node node1 = (Node)session.getItem("/testCorrespondingPath1/childNode2/jcr:content");
    //System.out.println(">> "+((SessionImpl)session2).getWorkspaceDataContainer());
    assertEquals("/testCorrespondingPath/childNode2/jcr:content", node1.getCorrespondingNodePath("ws2"));

    session.getRootNode().getNode("testCorrespondingPath1").remove();
    session.save();
    session2.getRootNode().getNode("testCorrespondingPath").remove();
    session2.save();

  }


  public void testNodeUpdate() throws RepositoryException {

    Session session2 = repository.login(credentials, "ws2");
    Node root = session2.getRootNode();
    Node file = root.addNode("testNodeUpdate", "nt:folder").addNode("childNode2", "nt:file");
    Node contentNode = file.addNode("jcr:content", "nt:resource");
    contentNode.setProperty("jcr:data", new BinaryValue("this is the content"));
    contentNode.setProperty("jcr:mimeType", new StringValue("text/html"));
    contentNode.setProperty("jcr:lastModified", session.getValueFactory().createValue(Calendar.getInstance()));

    session2.save();

    //root = session.getRootNode();
    //root.addNode("test", "nt:unstructured");
    //session.save();

    workspace.clone("ws2", "/testNodeUpdate", "/testNodeUpdate1", false);

    session = (SessionImpl)repository.login(credentials, WORKSPACE);

    assertEquals(((Node)session.getItem("/testNodeUpdate1/childNode2/jcr:content")).getUUID(),
        ((Node)session2.getItem("/testNodeUpdate/childNode2/jcr:content")).getUUID());

    Node node2 = (Node)session2.getItem("/testNodeUpdate/childNode2/jcr:content");
    BinaryValue bv = new BinaryValue("this is the NEW content");
    //log.debug("BV >>>>>> "+bv.getString());
    node2.setProperty("jcr:data", bv);

//    node2.setProperty("jcr:data", new BinaryValue("this is the NEW content"));
    assertEquals("this is the NEW content", node2.getProperty("jcr:data").getString());
    node2 = (Node)session2.getItem("/testNodeUpdate");
    node2.save();

    Node node1 = (Node)session.getItem("/testNodeUpdate1/childNode2/jcr:content");
//    log.debug(">>> DATA "+node1.getProperty("jcr:data"));
    assertEquals("this is the content", node1.getProperty("jcr:data").getString());

    node1.update("ws2");
    
    // because of bug
    node1 = (Node)session.getItem("/testNodeUpdate1/childNode2/jcr:content");

    assertEquals("this is the NEW content", node1.getProperty("jcr:data").getString());
    // No needs in save()

    
    Session session = repository.login(credentials, "ws");
    node1 = (Node)session.getItem("/testNodeUpdate1/childNode2/jcr:content");
    assertEquals("this is the NEW content", node1.getProperty("jcr:data").getString());


    session.getRootNode().getNode("testNodeUpdate1").remove();
    session.save();
    session2.getRootNode().getNode("testNodeUpdate").remove();
    session2.save();

  }

/*
  public void testOrderingChild() throws RepositoryException {
        fail("Ordering Child TODO!");
  }
*/
}
