/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.jcr.impl;


import java.io.ByteArrayInputStream;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.services.jcr.BasePortalTest;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryServiceConfiguration;
import org.exoplatform.services.jcr.impl.config.RepositoryServiceConfigurationImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.mock.DummyClientInfo;
import org.exoplatform.services.xml.querying.XMLQueryingService;


/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestService.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class TestService extends BasePortalTest {


	private static String STR_CONFIG = 
"<repository-service default-repository=\"mock\">\n"+
"<repositories><repository name=\"mock\" manager=\"inmemory\" system-workspace=\"ws\">\n"+
"<workspaces><workspace name=\"ws\" container=\"c.mock\"/>\n"+
"</workspaces></repository></repositories><containers>\n"+
"<container name=\"c.mock\" class=\"org.exoplatform.services.jcr.impl.storage.inmemory.WorkspaceContainerImpl\">\n"+
"<properties><property name=\"name\" value=\"value\"></property></properties>\n"+
"</container></containers><managers>\n"+
"<manager name=\"inmemory\" class=\"org.exoplatform.services.jcr.impl.storage.inmemory.RepositoryManagerImpl\"/>\n"+
"</managers></repository-service>";

  protected String getRepositoryName() {
    return "mock";
  }

  private XMLQueryingService xmlQueryingService;

  public void setUp() throws Exception {
    super.setUp();
	xmlQueryingService = (XMLQueryingService) PortalContainer.getInstance()
			.getComponentInstanceOfType(XMLQueryingService.class);

		//      repositoryService = (RepositoryService)
		// PortalContainer.getInstance().
		//          getComponentInstanceOfType(RepositoryService.class);

	PortalContainer container = PortalContainer.getInstance();
	SessionContainer scontainer = container.createSessionContainer(
			"test-session", "gena");
	scontainer.setClientInfo(new DummyClientInfo("gena"));

  }



  public void testSessionContainer() throws Exception {

    SessionContainer scontainer = SessionContainer.getInstance();
    assertEquals("test-session", scontainer.getSessionId());
    assertEquals("gena", scontainer.getRemoteUser());


  }
/*
  public void testConfigParams() throws Exception {
    
      ByteArrayInputStream bis = new ByteArrayInputStream(STR_CONFIG.getBytes());
      RepositoryServiceConfiguration config = new RepositoryServiceConfigurationImpl(bis);
      
      assertNotNull(config.getRepositoryConfiguration("mock"));
      assertTrue("config.getRepositoryEntries().length!=1", config.getRepositoryConfigurations().size() == 1);
      assertNotNull("c.mock is null", config.getContainerConfiguration("c.mock"));
      assertEquals("org.exoplatform.services.jcr.impl.storage.inmemory.WorkspaceContainerImpl",
                    config.getContainerConfiguration("c.mock").getType());
      assertEquals("value", 
                   config.getContainerConfiguration("c.mock").getParameterValue("name"));
      assertTrue("config.getSupportedContainerEntries().length!=1", config.getContainerConfigurations().size() == 1);
      assertNotNull(config.getRepositoryManagerConfiguration("inmemory"));
      assertTrue(config.getRepositoryManagerConfigurations().size() == 1);
    	
    }
*/
  public void testReadFromConfigRegisteredInPortalContainer() throws Exception {
    PortalContainer container = PortalContainer.getInstance();

    RepositoryServiceConfiguration config = (RepositoryServiceConfiguration)container.getComponentInstanceOfType(RepositoryServiceConfiguration.class);
    assertNotNull(config);
    assertTrue(config.getRepositoryConfigurations().size()>0);
    assertNotNull(config.getDefaultRepositoryName());
    log.debug("DEF NAME "+config.getDefaultRepositoryName());
  }
  
  public void testRepositoryRegistrationInPortalContainer() throws Exception {
    PortalContainer container = PortalContainer.getInstance();

    RepositoryService service = (RepositoryService) container.getComponentInstanceOfType(RepositoryService.class);
    assertNotNull(service);
    RepositoryServiceConfiguration config = (RepositoryServiceConfiguration)container.getComponentInstanceOfType(RepositoryServiceConfiguration.class);
    assertNotNull(config);
    assertEquals(service.getConfig(), config);
    
    
/*    
    assertNotNull(service);
    Repository rep = service.getRepository("mock");
    assertNotNull("Repository(1) is Null", rep);

    RepositoryServiceConfig config = (RepositoryServiceConfig) manager.getComponentInstanceOfType(RepositoryServiceConfig.class);
    assertNotNull("Config is null", config);
    ContainerEntry entry = config.getContainerEntry("c.mock");
    assertNotNull("'c.inmemory' entry not found!", entry);

    assertEquals(config, service.getConfig());
    String containerName = entry.getType();
    assertEquals("org.exoplatform.services.jcr.impl.storage.inmemory.WorkspaceContainerImpl", containerName);
    assertNotNull(Class.forName(containerName));

    rep = service.getRepository("mock");
    assertNotNull("Repository(2) is Null", rep);
*/
  }

  public void testInitSystemWorkspace() throws Exception {
    RepositoryImpl repository = (RepositoryImpl)repositoryService.getRepository();
    boolean init = repository.isWorkspaceInitialized(repository.getSystemWorkspaceName());
   
    repository.initWorkspace(repository.getSystemWorkspaceName(), "nt:unstructured");
    init = repository.isWorkspaceInitialized(repository.getSystemWorkspaceName());
    assertTrue(init);
    SessionImpl session = (SessionImpl)repositoryService.getRepository().login();
    assertNotNull(session);

  }

  
  /*  
  public void testXMLConfig() throws Exception {
    String str = "<repositories-config><default-repository name=\"mock\"/><repositories><repository name=\"mock\" manager=\"inmemory\"><workspaces><workspace name=\"ws\" default=\"true\"/></workspaces></repository></repositories><containers><container name=\"c.mock\" class=\"org.exoplatform.services.jcr.impl.storage.inmemory.WorkspaceContainerImpl\"><properties><property name=\"name\" value=\"value\"/></properties></container></containers><managers><manager name=\"inmemory\" class=\"org.exoplatform.services.jcr.impl.storage.inmemory.RepositoryManagerImpl\"/></managers></repositories-config>";
    ByteArrayInputStream bis = new ByteArrayInputStream(str.getBytes());
    RepositoryServiceConfig config = new XMLConfig(xmlQueryingService, bis);
    assertNotNull(config.getRepositoryEntry("mock"));
    assertTrue("config.getRepositoryEntries().length!=1", config.getRepositoryEntries().length == 1);
    assertNotNull("c.mock is null", config.getContainerEntry("c.mock"));
    assertEquals("org.exoplatform.services.jcr.impl.storage.inmemory.WorkspaceContainerImpl",
                  config.getContainerEntry("c.mock").getType());
    assertEquals("value", 
                 config.getContainerEntry("c.mock").getParameterValue("name"));
    assertTrue("config.getSupportedContainerEntries().length!=1", config.getSupportedContainerEntries().length == 1);
    assertNotNull(config.getRepositoryManagerEntry("inmemory"));
    assertTrue(config.getSupportedRepositoryManagerEntries().length == 1);
//    assertNotNull(config.getWorkspaceEntry("mock","ws"));
//    assertTrue(config.getWorkspaceEntries("mock").length == 1);
  }

  public void testReadDefaultXMLConfig() throws Exception {
    RepositoryServiceConfig config = new XMLConfig(xmlQueryingService);
    assertNotNull(config);
    assertTrue(config.getRepositoryEntries().length>0);
    assertNotNull(config.getDefaultRepositoryName());
    log.debug("DEF NAME "+config.getDefaultRepositoryName());


  }

  
  public void testDefaultRepository() throws Exception {
    SessionImpl ticket = (SessionImpl)repositoryService.getRepository().login("ws");
    log.debug("CONTAINER "+ticket.getContainer().getName());
  }


*/
}
