/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.grammar.wiki.impl;

import java.lang.reflect.Method;
import java.util.* ;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Apr 22, 2005
 * @version $Id$
 */
public class IntrospectionInfo {
  private static Map cacheMethod_ = new HashMap() ;
  
  String exp ;
  String beanName ;
  String methodName ;
  List args ;
  boolean error = false ;
  
  public IntrospectionInfo(String exp) {
    this.exp = exp ;
    int index = 2 , start = 2, length = exp.length() - 1;
    for(; index < length; index++) {
      if(exp.charAt(index) == '.') {
        beanName =  exp.substring(start, index) ;
        start = index + 1 ;
        break ;
      } else if(index == length -1) {
        beanName =  exp.substring(start, length) ;
        return ;
      }
        
    }
    for(; index < length; index++) {
      if(exp.charAt(index) == '(') {
        methodName =  exp.substring(start, index) ;
        start = index + 1 ;
        break ;
      } else if(index == length -1) {
        methodName =  exp.substring(start, length) ;
        return ;
      }
    }
    boolean openQuote = false ;
    for(; index < length; index++) {
      if(exp.charAt(index) == '\"')  {
        openQuote = !openQuote ;
      } else  if(exp.charAt(index) == ',' && !openQuote) {
        if(args == null) args = new ArrayList() ;
        args.add(exp.substring(start, index)) ;
        start = index + 1 ;
      } else if(index == length -1 && exp.charAt(index) == ')') {
        String arg = exp.substring(start, index).trim() ;
        if(arg.length() > 0) {
          if(args == null) args = new ArrayList() ;
          args.add(arg) ;
        }
        return ;
      }
    }
    error = true ;
  }
  
  public Object invoke(Object bean) throws Exception {
    Method method = getMethod(bean) ;
    Object[] invokeArgs = null ;
    Class[] types = method.getParameterTypes()   ;
    invokeArgs = new Object[types.length] ;
    for(int i = 0; i < types.length; i++) {
      invokeArgs[i] = getArgumentValue(types[i], (String)args.get(i)) ;
    } 
    Object ret = method.invoke(bean, invokeArgs) ;
    return ret ;
  }
  
  private Method getMethod(Object bean) throws Exception {
    Class clazz = bean.getClass() ;
    String cacheKey = null ;
    if(args != null) {
      cacheKey = clazz.getName() + "+" + methodName + "+" + args.size() ;
    } else {
      cacheKey = clazz.getName() + "+" + methodName ;
    }
    Method method = (Method)cacheMethod_.get(cacheKey) ;
    if(method != null)  return method ;
    
    synchronized(cacheMethod_) {
      if(args == null) {
        method = clazz.getMethod(methodName, null) ;
      } else {
        Method[] methods = clazz.getMethods() ;
        for(int i = 0; i < methods.length; i++) {
          if(methodName.equals(methods[i].getName())) {
            if(methods[i].getParameterTypes().length == args.size()) {
              method = methods[i] ;
              break ;
            }
          }
        }
        if(method == null) {
          throw new Exception("Cannot find the method " + methodName + "for the object " + bean) ;
        }
      }
      cacheMethod_.put(cacheKey, method) ;
    }
    return method ;
  }
 
  
  private Object getArgumentValue(Class type, String value) throws Exception {
    if(type.equals(String.class)) {
      value = value.trim() ;
      if(value.charAt(0) != '\"' && value.charAt(value.length() - 1) != '\"') {
        throw new Exception("Cannot instancitate parameter type " + type + 
                            " for value " + value) ;
      }
      return value.substring(1, value.length() - 1) ;
    }
    
    if(type.equals(Integer.class) || type.equals(Integer.TYPE)) {
      return new Integer(value) ;
    }
    
    if(type.equals(Long.class) || type.equals(Long.TYPE)) {
      return new Long(value) ;
    }
    
    if(type.equals(Double.class) || type.equals(Double.TYPE)) {
      return new Double(value) ;
    }
    
    if(type.equals(Float.class) || type.equals(Float.TYPE)) {
      return new Float(value) ;
    }
    
    if(type.equals(Boolean.class) || type.equals(Boolean.TYPE)) {
      return new Boolean(value) ;
    }
    
    throw new Exception("Cannot instancitate parameter type " + type + 
                        " for value " + value) ;
  }
  
  public String testString(String s) {
    return "hello reply of " + s ;
  }
  
  public int testInteger(int i) {
    return  i ;
  }
  
  public Integer testIntegerObject(Integer i) {
    return  i ;
  }
  
  public String toString()  {
    StringBuffer b = new StringBuffer() ;
    b.append(beanName); 
    if(methodName != null) b.append(".").append(methodName) ;
    b.append("(") ;
    if(args != null) {
      for(int i = 0; i < args.size(); i++) {
        b.append(args.get(i)) ;
        if(i != args.size() - 1) b.append(",") ;
      }
      if(error) b.append(" [error = true]") ;
    }
    b.append(")") ;
    return b.toString() ;
  }
  
  static public String inspect(Map map , String expression) {
    int length = expression.length() ;
    if(length < 3 ||  expression.charAt(0) != '$' || 
       expression.charAt(1) != '{' || expression.charAt(length - 1) != '}') return expression ;
    IntrospectionInfo info = new IntrospectionInfo(expression) ;
    if(info.error) {
      return expression ;
    } else if(info.methodName == null) {
      Object bean = map.get(info.beanName) ;
      if(bean == null) return expression ;
      return bean.toString() ;
    } else {
      try {
        Object bean = map.get(info.beanName) ;
        Object ret = info.invoke(bean) ; 
        return ret.toString() ;
      } catch(Exception ex) {
        System.out.print("Cannot execute expression "  + expression + 
                         " , Error message" + ex.getMessage()) ;
        return expression ;
      }
    }
  }
}