/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.impl;

import java.io.InputStream;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.backup.ImportExportPlugin;
import org.exoplatform.services.backup.Metadata;
import org.exoplatform.services.backup.XMLObjectConverter;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.xml.object.XMLCollection;
import org.exoplatform.xml.object.XMLObject;
/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
public class ImportExportPluginImpl extends ImportExportPlugin {
  static String PLUGIN_NAME = "organization" ;
  static String USER_META_DATA_ENTRY   = "organization/metadata.xml" ;
  static String USER_DATA_ENTRY = "organization/User.xml" ;
  static String USER_PROFILE_DATA_ENTRY = "organization/UserProfile.xml" ;
  static String USER_MEMBERSHIPS_DATA_ENTRY = "organization/Memberships.xml" ;
  static String SERVICE_META_DATA_ENTRY   = "metadata.xml" ;
  
  OrganizationService orgService_ ;

  public ImportExportPluginImpl(OrganizationService service, 
                                InitParams params) throws Exception {
    orgService_ = service ;
    init(params) ;
  }

  public void exportUserData(String userName, ZipOutputStream os) throws Exception {    
    Metadata mdata = new Metadata(getCurrentDataVersion(), getDescription(), "single-user") ;
    createEntry(USER_META_DATA_ENTRY , os, mdata) ; 
    Object object = orgService_.getUserHandler().findUserByName(userName) ;
    createEntry(USER_DATA_ENTRY, os, object) ; 
    object = orgService_.getUserProfileHandler().findUserProfileByName(userName) ;
    createEntry(USER_PROFILE_DATA_ENTRY, os, object) ; 
    object = orgService_.getMembershipHandler().findMembershipsByUser(userName) ;
    createEntry(USER_MEMBERSHIPS_DATA_ENTRY, os, object) ; 
  }

  
  public void importUserData(String username, ZipFile zipFile) throws Exception {        
    Metadata mdata = getMetadata( USER_META_DATA_ENTRY, zipFile) ;
    String dataVersion = mdata.getDataVersion() ;
    XMLObjectConverter converter = getXMLObjectConverter(dataVersion) ;
    //import user data
    XMLObject xmlobject = XMLObject.getXMLObject(getEntry(USER_DATA_ENTRY, zipFile)) ;
    if(converter != null) converter.traverse(xmlobject) ;
    User user = (User) xmlobject.toObject() ;
    orgService_.getUserHandler().createUser(user, false) ;
    //import user profile data
    xmlobject = XMLObject.getXMLObject(getEntry(USER_PROFILE_DATA_ENTRY, zipFile)) ;
    if(converter != null) converter.traverse(xmlobject) ;
    UserProfile up = (UserProfile)xmlobject.toObject() ;
    orgService_.getUserProfileHandler().saveUserProfile(up, false) ;
    //import user memberships
    XMLCollection xmlCol =  
      XMLCollection.getXMLCollection(getEntry(USER_MEMBERSHIPS_DATA_ENTRY, zipFile)) ;
    if(converter != null) converter.traverse(xmlCol) ;
    Collection<Membership> memberships = xmlCol.getCollection() ;
    for(Membership m : memberships) {
      orgService_.getMembershipHandler().createMembership(m, false) ;
    }
  }

  public void exportServiceData( ZipOutputStream os) throws Exception {
    Metadata mdata = new Metadata(getCurrentDataVersion(), getDescription(), "single-service") ;
    createEntry( SERVICE_META_DATA_ENTRY , os, mdata) ; 

    Collection groups = orgService_.getGroupHandler().getAllGroups();
    Iterator i = groups.iterator() ;
    while(i.hasNext()) {
      Group data = (Group) i.next() ;
      ZipEntry entry = new ZipEntry("group" + data.getId()+ ".xml") ;
      os.putNextEntry(entry) ;
      XMLObject xmlobject = new XMLObject(data) ;
      os.write(xmlobject.toByteArray("UTF-8")) ;
      os.closeEntry() ;
    }    
    
    Collection membershipTypes = orgService_.getMembershipTypeHandler().findMembershipTypes();
    
    i = membershipTypes.iterator() ;
    while(i.hasNext()) {
      MembershipType data = (MembershipType) i.next() ;
      ZipEntry entry = new ZipEntry( "membership-type/" +data.getName()+ ".xml") ;
      os.putNextEntry(entry) ;
      XMLObject xmlobject = new XMLObject(data) ;
      os.write(xmlobject.toByteArray("UTF-8")) ;
      os.closeEntry() ;
    }
  }

  public void importServiceData( ZipFile in) throws Exception {  
    Metadata mdata = getMetadata( SERVICE_META_DATA_ENTRY, in) ;
    String dataVersion = mdata.getDataVersion() ;
    XMLObjectConverter converter = getXMLObjectConverter(dataVersion) ;
    Enumeration e = in.entries() ;
    XMLObject xmlobject = null ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      InputStream is = in.getInputStream(entry) ;
      xmlobject = XMLObject.getXMLObject(is) ;
      if(converter != null) converter.traverse(xmlobject) ;
      String entryName = entry.getName() ;
      if( entryName.startsWith("group/")) {        
        GroupImpl data = (GroupImpl) xmlobject.toObject();        
        Group g = orgService_.getGroupHandler().findGroupById( data.getId());
        if(  g != null)  orgService_.getGroupHandler().removeGroup( g, false);       
        orgService_.getGroupHandler().createGroup( data, false);               
      } else if(entryName.startsWith("membership-type/")) {
        MembershipTypeImpl data = (MembershipTypeImpl)xmlobject.toObject() ; 
        orgService_.getMembershipTypeHandler().removeMembershipType( data.getName(), false) ;        
        orgService_.getMembershipTypeHandler().createMembershipType( data, false);
      }
    }   
  }  
}