/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.services.portal.skin.model.Decorator;
import org.exoplatform.services.portal.skin.model.Style;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 10, 2005
 * @version $Id$
 */
public class Util {
  
  static List getRendererOptions(Collection decorators) {
    List options = new ArrayList();
    Iterator i = decorators.iterator() ;
    while(i.hasNext()) {
      Decorator decorator = (Decorator) i.next() ;
      String rendererType = decorator.getRendererType();
      options.add(new SelectItem(rendererType, rendererType));
    }
    return options ;
  }
  
  static List getStyleOptions(Decorator decorator) {
    List styles = decorator.getStyles();
    List options = new ArrayList() ;
    for(int i = 0; i < styles.size(); i++) {
      Style style = (Style)styles.get(i);
      String name = style.getName();
      options.add(new SelectItem(name, name));
    }
    return options ;
  }
  
  /**
   * This method search for all component of the give type in portal and do not 
   * traverse the UIBody component
   */
  static public List findDescendantsOfTypeInPortal(UIPortal uiPortal, Class type) {
    List list = new ArrayList() ;
    findDescendantsOfTypeInPortal(uiPortal, type, list) ;
    return list ;
  }
  
  static private void findDescendantsOfTypeInPortal(UIComponent comp, Class type, List list) {
    if(type.isInstance(comp)) list.add(comp) ;
    List children = comp.getChildren() ;
    for(int i = 0; i < children.size(); i++) {
      UIComponent child = (UIComponent) children.get(i) ;
      if(child instanceof UIBody) continue ;
      findDescendantsOfTypeInPortal(child, type, list) ;
    }
  }
  
  
  static public void updateSelectDecorator(UISelectBox ui, Decorator decorator, String selectStyle) {
    List decoratorOptions = ui.getOptions() ;
    decoratorOptions.clear() ;  
    List styles = decorator.getStyles() ;
    for (int i = 0; i < styles.size(); i++) {
      Style style = (Style) styles.get(i) ;
      String name = style.getName() ;
      decoratorOptions.add(new SelectItem(name, name)) ;
      if(i == 0 && selectStyle == null) selectStyle = name ;
    }
    ui.setValue(selectStyle);
  }
}