/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.management.registry.component;

import java.util.Collection;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.services.portletregistery.Portlet;
import org.exoplatform.services.portletregistery.PortletPermission;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 19 juin 2004
 */
public class UIPortletPermission extends UIExoCommand implements SelectorHandler {
  static String ADD_PERMISSION = "addPermission" ; 
  static String REMOVE_PERMISSION = "removePermission" ; 
  static Parameter[] CANCEL_ACTION_PARAMS = {new Parameter(ACTION , CANCEL_ACTION) } ;
  static Parameter[] ADD_PERMISSION_PARAMS = {new Parameter(ACTION , ADD_PERMISSION) } ;
  static Parameter[] DELETE_PERMISSION_PARAMS = {new Parameter(ACTION , REMOVE_PERMISSION) } ;
  
  private Portlet portlet_ ;

  private Collection permissions_ ;

  public UIPortletPermission() throws Exception {
    setRendererType("VelocityRenderer");
    setId("UIPortletRole");
    
    addFacesListener(new CancelActionListener().setActionToListen(CANCEL_ACTION));
    addFacesListener(new AddPortletPermissionActionListener().setActionToListen(ADD_PERMISSION));
    addFacesListener(new RemovePortletPermissionActionListener().setActionToListen(REMOVE_PERMISSION));
  }
  
  public void setCurrentPortlet(Portlet portlet) throws Exception {
    portlet_ = portlet ;
    PortletRegisteryService service =
      (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
    permissions_ = service.getPortletPermissions(portlet.getId());
  }
  
  public Collection getPortletPermissions() { return permissions_; }

  public Parameter[] getCancelActionParams() { return CANCEL_ACTION_PARAMS ; }
  public Parameter[] getAddPermissionParams() { return ADD_PERMISSION_PARAMS ; }
  public Parameter[] getDeletePermissionParams() { return DELETE_PERMISSION_PARAMS ; }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIPortletPermission.class) ;
  }

  public void select(Selector selector) throws Exception {
    PortletRegisteryService service =
      (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
    PortletPermission permission = service.createPortletPermissionInstance() ;
    permission.setPermissionExpression(selector.getInputValue());
    service.addPortletPermission(portlet_, permission) ;
    permissions_ = service.getPortletPermissions(portlet_.getId());
    selector.setRenderedSibling(UIPortletPermission.class) ;
  }
  
  static public class AddPortletPermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletPermission uicomp = (UIPortletPermission) event.getSource() ;
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser) uicomp.getSibling(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uicomp) ;
      uicomp.setRenderedSibling(UIExoPermissionBrowser.class) ;
    }
  }

  static public class RemovePortletPermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletPermission uicomp = (UIPortletPermission) event.getSource() ;
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.removePortletPermission(event.getParameter(OBJECTID)) ;
      uicomp.permissions_ = service.getPortletPermissions(uicomp.portlet_.getId());
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletPermission uicomp = (UIPortletPermission) event.getSource() ;
      uicomp.setRenderedSibling(UIPortletCategories.class);
    }
  }
}