/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import java.io.InputStream;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.model.PageSet;
import org.hibernate.Session;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 5, 2005
 */
public class UserConfigTemplate {
  
  private String templateConfig_;
  private String templatePageSet_ ;
  private String templateNavigation_ ;
  private NewPortalConfig config_ ;
  private ConfigurationManager cservice_ ;
  private HibernateService hservice_ ;  
  
  public UserConfigTemplate(HibernateService hservice, ConfigurationManager cservice,
                                     InitParams params) throws Exception {
    cservice_ = cservice ;
    hservice_ = hservice ;
    config_ = (NewPortalConfig) params.getObjectParamValues(NewPortalConfig.class).get(0) ;
    String templateUser = config_.getTemplateUser() ;
    String templateLoc = config_.getTemplateLocation() ;
    InputStream is = cservice_.getInputStream(templateLoc + "/" + templateUser + "-config.xml") ;
    templateConfig_ =   IOUtil.getStreamContentAsString(is);
    is = cservice_.getInputStream(templateLoc + "/" + templateUser + "-pages.xml") ;
    templatePageSet_ =    IOUtil.getStreamContentAsString(is);
    is  = cservice_.getInputStream(templateLoc + "/" + templateUser + "-navigation.xml") ;
    templateNavigation_ =   IOUtil.getStreamContentAsString(is);
  }
  
  public void createConfigurationForUser(String owner) throws Exception {
    String config = null ;
    String pageSet = null ;
    String navigation = null ;
    if(config_.isPredefinedUser(owner)) {
      String location = config_.getTemplateLocation() ;
      config = 
        IOUtil.getStreamContentAsString(cservice_.getInputStream(location + "/" + owner + "-config.xml"));
      pageSet = 
        IOUtil.getStreamContentAsString(cservice_.getInputStream(location + "/" + owner + "-pages.xml"));
      navigation = 
        IOUtil.getStreamContentAsString(cservice_.getInputStream(location + "/" + owner + "-navigation.xml"));
    } else {
      config = StringUtils.replace(templateConfig_, "@owner@", owner);
      pageSet = StringUtils.replace(templatePageSet_, "@owner@", owner);
      navigation = StringUtils.replace(templateNavigation_, "@owner@", owner);
    }
    Session session = hservice_.openSession() ;
    session.save(new PortalConfigData(config));
    session.save(new NodeNavigationData(navigation));
    PageSet ps = (PageSet) PortalConfigServiceImpl.fromXML(pageSet, PageSet.class);
    List pages = ps.getPages() ;
    for (int i = 0 ; i <  pages.size(); i++) {
      Page page = (Page) pages.get(i) ;
      session.save(new PageData(page)) ;
    }
    session.flush() ;
  }
}
